@@ -17,6 +17,8 @@ struct player;
 struct city;
 struct ai_choice;
 
+void ai_eval_threat_init(struct player *pplayer);
+void ai_eval_threat_done(struct player *pplayer);
 void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				   struct ai_choice *choice);
 void ai_eval_buildings(struct city *pcity);
