@@ -10,8 +10,11 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
-#ifndef __ADVLEADER_H
-#define __ADVLEADER_H
+#ifndef FC__ADVLEADER_H
+#define FC__ADVLEADER_H
+
+struct player;
+struct ai_choice;
 
 void leader_adjust_tech_choice(struct player *pplayer, 
 			       struct ai_choice *choice, int advisor);
@@ -19,4 +22,4 @@ void leader_adjust_tech_choice(struct player *pplayer,
 void leader_adjust_build_choice(struct player *pplayer, 
 				struct ai_choice *choice, int advisor);
 
-#endif
+#endif  /* FC__ADVLEADER_H */
