@@ -51,75 +51,6 @@
   And these are the days when we look for something other
   /U2 Lemon.
 ******************************************************************************/
-/*
-1 Basic:
-- (serv) AI <player> server command toggles AI on off                DONE
-- (Unit) settler capable of building city                            DONE
-- (City) cities capable of building units/buildings                  DONE
-- (Hand) adjustments of tax/luxuries/science                         DONE
-- (City) happiness control                                           DONE
-- (Hand) change government                                           DONE
-- (Tech) tech management                                             DONE
-2 Medium:
-- better city placement                                              TODO
-- ability to explore island                                          DONE
-- Barbarians                                                         ????
-- better unit building management                                    DONE
-- better improvement management                                      DONE
-- taxcollecters/scientists                                           DONE
-- spend spare trade on buying                                        DONE
-- upgrade units                                                      DONE
-- (Unit) wonders/caravans                                            DONE
-- defense/city value                                                 ????
-- Tax/science/unit producing cities                                  ????
-3 Advanced:
-- ZOC
-- continent defense
-- sea superiority
-- air superiority
-- continent attack
-4 Superadvanced:
-- Transporters (attack on other continents)
-- diplomacy (embassies / tech trading / map trading with other AIs)
-
-
-Step 1 is the basics, can be used for the blank seat people usually have when
-they start a game, the AI will atleast do something other than just sit and 
-wait, for the kill.
-Step 2 will make it do it alot better, and hopefully the barbarians will be in
-action.
-3 if step 1 and 2 gives decent results, the AI will actually have survived the
-building phase and we have to worry about units, step 3 is meant as defense.
-So first in step 4 will we introduce attacking AI. (if it ever gets that far)
-
--- I don't know who wrote this, but I've gone a different direction -- Syela
- */
-
-#ifdef UNUSED
-/**************************************************************************
- update advisors/structures
-**************************************************************************/
-static void ai_before_work(struct player *pplayer)
-{
-
-}
-
-/**************************************************************************
- well am not sure what will happend here yet, maybe some more analysis
-**************************************************************************/
-static void ai_after_work(struct player *pplayer)
-{
-
-}
-
-/**************************************************************************
- Trade tech and stuff, this one will probably be blank for a long time.
-**************************************************************************/
-static void ai_manage_diplomacy(struct player *pplayer)
-{
-
-}
-#endif /* UNUSED */
 
 /**************************************************************************
  handle spaceship related stuff
@@ -146,7 +77,6 @@ static void ai_manage_taxes(struct player *pplayer)
 {
   struct government *g = get_gov_pplayer(pplayer);
   int gnow = pplayer->economic.gold;
-  Impr_Type_id id;
   int trade = 0, m, n, i, expense = 0, tot;
   int waste[40]; /* waste with N elvises */
   int elvises[11] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
@@ -170,15 +100,17 @@ static void ai_manage_taxes(struct player *pplayer)
     city_refresh(pcity);
     trade += pcity->trade_prod * city_tax_bonus(pcity) / 100;
     freelog(LOG_DEBUG, "%s has %d trade.", pcity->name, pcity->trade_prod);
-    for (id = 0; id < game.num_impr_types; id++)
-      if (city_got_building(pcity, id)) expense += improvement_upkeep(pcity,id);
+    built_impr_iterate(pcity, id) {
+      expense += improvement_upkeep(pcity, id);
+    } built_impr_iterate_end;
+
   city_list_iterate_end;
 
   pplayer->ai.est_upkeep = expense;
 
-  if (!trade) { /* can't return right away - thanks for the evidence, Muzz */
+  if (trade == 0) { /* can't return right away - thanks for the evidence, Muzz */
     city_list_iterate(pplayer->cities, pcity) 
-      if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer,1))
+      if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer, TRUE))
         ai_scientists_taxmen(pcity);
     city_list_iterate_end;
     return; /* damn division by zero! */
@@ -191,9 +123,10 @@ static void ai_manage_taxes(struct player *pplayer)
     /* this code must be ABOVE the elvises[] if SIMPLISTIC is off */
     freelog(LOG_DEBUG, "Does %s want to be bigger? %d",
 		  pcity->name, wants_to_be_bigger(pcity));
-    if (government_has_flag(g, G_RAPTURE_CITY_GROWTH) && pcity->size >= g->rapture_size  &&
-        pcity->food_surplus > 0 && !pcity->ppl_unhappy[4] && wants_to_be_bigger(pcity) &&
-	ai_fuzzy(pplayer,1)) {
+    if (government_has_flag(g, G_RAPTURE_CITY_GROWTH)
+	&& pcity->size >= g->rapture_size && pcity->food_surplus > 0
+	&& pcity->ppl_unhappy[4] == 0 && pcity->ppl_angry[4] == 0
+	&& wants_to_be_bigger(pcity) && ai_fuzzy(pplayer, TRUE)) {
       freelog(LOG_DEBUG, "%d happy people in %s",
 		    pcity->ppl_happy[4], pcity->name);
       n = ((pcity->size/2) - pcity->ppl_happy[4]) * 20;
@@ -205,7 +138,7 @@ static void ai_manage_taxes(struct player *pplayer)
       tot = 0;
       for (i = 0; i <= 10; i++) {
         if (pcity->trade_prod * i * city_tax_bonus(pcity) >= n * 100) {
-	  if (!tot) freelog(LOG_DEBUG, "%s celebrates at %d.",
+	  if (tot == 0) freelog(LOG_DEBUG, "%s celebrates at %d.",
 			    pcity->name, i * 10);
           hhjj[i] += (pcity->was_happy ? m : m/2);
           tot++;
@@ -214,7 +147,9 @@ static void ai_manage_taxes(struct player *pplayer)
     } /* hhjj[i] is (we think) the desirability of partying with lux = 10 * i */
 /* end elevated code block */
 
-    n = (pcity->ppl_unhappy[4] - pcity->ppl_happy[4]) * 20; /* need this much lux */
+    /* need this much lux */
+    n = (2 * pcity->ppl_angry[4] + pcity->ppl_unhappy[4] -
+	 pcity->ppl_happy[4]) * 20;
 
 /* this could be an unholy CPU glutton; it's only really useful when we need
    lots of luxury, like with pcity->size = 12 and only a temple */
@@ -226,7 +161,7 @@ static void ai_manage_taxes(struct player *pplayer)
 
     for (i = 1; i <= pcity->size; i++) {
       m = ai_make_elvis(pcity);
-      if (m) {
+      if (m != 0) {
         waste[i] = waste[i-1] + m;
       } else {
         while (i <= pcity->size) {
@@ -301,13 +236,13 @@ static void ai_manage_taxes(struct player *pplayer)
     city_refresh(pcity);
     add_adjust_workers(pcity);
     city_refresh(pcity);
-    if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer,1))
+    if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer, TRUE))
       ai_scientists_taxmen(pcity);
     if (pcity->shield_surplus < 0 || city_unhappy(pcity) ||
         pcity->food_stock + pcity->food_surplus < 0) 
        emergency_reallocate_workers(pplayer, pcity);
     if (pcity->shield_surplus < 0) {
-      defender = 0;
+      defender = NULL;
       unit_list_iterate(pcity->units_supported, punit)
         incity = map_get_city(punit->x, punit->y);
         if (incity && pcity->shield_surplus < 0) {
@@ -320,13 +255,13 @@ static void ai_manage_taxes(struct player *pplayer)
               if (unit_vulnerability_virtual(punit) <
                   unit_vulnerability_virtual(defender)) {
 		freelog(LOG_VERBOSE, "Disbanding %s in %s",
-			unit_types[punit->type].name, pcity->name);
+			unit_type(punit)->name, pcity->name);
                 pack.unit_id = punit->id;
                 handle_unit_disband(pplayer, &pack);
                 city_refresh(pcity);
               } else {
 		freelog(LOG_VERBOSE, "Disbanding %s in %s",
-			unit_types[defender->type].name, pcity->name);
+			unit_type(defender)->name, pcity->name);
                 pack.unit_id = defender->id;
                 handle_unit_disband(pplayer, &pack);
                 city_refresh(pcity);
@@ -339,7 +274,7 @@ static void ai_manage_taxes(struct player *pplayer)
             handle_unit_change_homecity(pplayer, &pack);
             city_refresh(pcity);
 	    freelog(LOG_VERBOSE, "Reassigning %s from %s to %s",
-		    unit_types[punit->type].name, pcity->name, incity->name);
+		    unit_type(punit)->name, pcity->name, incity->name);
           }
         } /* end if */
       unit_list_iterate_end;
@@ -348,7 +283,7 @@ static void ai_manage_taxes(struct player *pplayer)
           if (punit != defender && pcity->shield_surplus < 0) {
 	    /* the defender MUST NOT be disbanded! -- Syela */
 	    freelog(LOG_VERBOSE, "Disbanding %s's %s",
-		    pcity->name, unit_types[punit->type].name);
+		    pcity->name, unit_type(punit)->name);
             pack.unit_id = punit->id;
             handle_unit_disband_safe(pplayer, &pack, &myiter);
             city_refresh(pcity);
@@ -366,7 +301,6 @@ static void ai_manage_taxes(struct player *pplayer)
 /**************************************************************************
  change the government form, if it can and there is a good reason
 **************************************************************************/
-#ifndef NEW_GOV_EVAL
 static void ai_manage_government(struct player *pplayer)
 {
   int goal;
@@ -420,80 +354,19 @@ static void ai_manage_government(struct player *pplayer)
   }
 }
 
-#else  /* following may need updating before enabled --dwp */
-
-#define DEBUG_AMG_RATING        (0)
-#define DEBUG_AMG_BOOST		(0)
-#define DEBUG_AMG_CHANGED	(0)
-#define DEBUG_AMG_CHOICE	(1)
-static void ai_manage_government(struct player *pplayer)
-{
-  static int prev_rating[10][10];  /* these are for debugging only */
-  static int prev_choice[10];      /* -- SKi*/
-  int best_government = pplayer->government,
-      best_rating = ai_evaluate_government(pplayer, get_gov_pplayer(pplayer));
-  int i;
-
-  for (i=0; i<game.government_count; ++i) {
-    struct government *g = &governments[ i ];
-    int rating;
-
-    rating = ai_evaluate_government (pplayer, g);
-    /* give bonus to current government */
-/*    if (i == pplayer->government)
-      rating += 2500; */
-
-    if (DEBUG_AMG_RATING && prev_rating[pplayer->player_no][i] != rating && (rating > 0 || rating > prev_rating[pplayer->player_no][i]))
-      freelog (LOG_DEBUG, "%d: ai_manage_government: %s: %s: %d (%+d)", game.year, pplayer->name, g->name, rating, rating - prev_rating[pplayer->player_no][i]);
-    prev_rating[pplayer->player_no][i] = rating;
-    /* compare this government with the best so far */
-    if (rating > best_rating || best_government == -1) {
-      if (can_change_to_government(pplayer, i)) {
-        /* Use this thing! -- SKi */
-        best_government = i;
-        best_rating = rating;
-      } else {
-        if (get_invention(pplayer, g->required_tech) != TECH_KNOWN && rating > 0) {
-          /* Research this thing! -- SKi */
-	  if (DEBUG_AMG_BOOST)
-            freelog (LOG_DEBUG, "%d: ai_manage_government: boosting %s for %s (%d -> %d)",
-              game.year,
-	      advances[ g->required_tech ].name, pplayer->name,
-              pplayer->ai.tech_want[ g->required_tech ],
-              pplayer->ai.tech_want[ g->required_tech ] + rating * 2);
-          pplayer->ai.tech_want[ g->required_tech ] += 250 + rating / 2; /* need constant modifier? */
-        }
-      }
-    }
-  }
-  if ((DEBUG_AMG_CHANGED && prev_choice[pplayer->player_no] != best_government) || DEBUG_AMG_CHOICE)
-    freelog (LOG_DEBUG, "%d: %s chooses new government: %s", game.year, pplayer->name, governments[best_government].name);
-  prev_choice[pplayer->player_no] = best_government;
-
-  if (pplayer->government != best_government) {
-    ai_government_change(pplayer, best_government);
-  }
-  return;
-}
-#endif /* NEW_GOV_EVAL */
-
 /**************************************************************************
  Main AI routine.
 **************************************************************************/
 void ai_do_first_activities(struct player *pplayer)
 {
-#ifdef UNUSED
-  ai_before_work(pplayer); 
-#endif
   ai_manage_units(pplayer); /* STOP.  Everything else is at end of turn. */
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
 void ai_do_last_activities(struct player *pplayer)
 {
-/*  ai_manage_units(pplayer);  very useful to include this! -- Syela */
-/* I finally realized how stupid it was to call manage_units in update_player_ac
-instead of right before it.  Managing units before end-turn reset now. -- Syela */
-  calculate_tech_turns(pplayer); /* has to be here thanks to the above */
   ai_manage_cities(pplayer);
   /* manage cities will establish our tech_wants. */
   /* if I were upgrading units, which I'm not, I would do it here -- Syela */ 
@@ -501,22 +374,18 @@ instead of right before it.  Managing units before end-turn reset now. -- Syela
   ai_manage_taxes(pplayer); 
   freelog(LOG_DEBUG, "Managing %s's government.", pplayer->name);
   ai_manage_government(pplayer);
-#ifdef UNUSED
-  ai_manage_diplomacy(pplayer);
-#endif
   ai_manage_tech(pplayer); 
   freelog(LOG_DEBUG, "Managing %s's spaceship.", pplayer->name);
   ai_manage_spaceship(pplayer);
   freelog(LOG_DEBUG, "Managing %s's taxes.", pplayer->name);
-#ifdef UNUSED
-  ai_after_work(pplayer);
-#endif
   freelog(LOG_DEBUG, "Done with %s.", pplayer->name);
 }
 
 
-int is_unit_choice_type(enum choice_type type)
+/**************************************************************************
+  ...
+**************************************************************************/
+bool is_unit_choice_type(enum choice_type type)
 {
    return type == CT_NONMIL || type == CT_ATTACKER || type == CT_DEFENDER;
 }
-
