@@ -39,10 +39,6 @@
 /**************************************************************************
 .. calculate next government wish.
 **************************************************************************/
-#ifndef NEW_GOV_EVAL
-/*
-   regression testing
-*/
 static Tech_Type_id get_government_tech(struct player *plr)
 {
   int goal = get_nation_by_plr(plr)->goals.government;
@@ -70,28 +66,6 @@ static Tech_Type_id get_government_tech(struct player *plr)
   }
 }
 
-#else  /* following may need updating before enabled --dwp */
-
-static Tech_Type_id get_government_tech(struct player *plr)
-{
-  int i, rating;
-  int best_government = -1, best_rating = 0;
-
-  for (i = 0; i < game.government_count; ++i) {
-    struct government *g = get_government(i);
-    rating = ai_evaluate_government (plr, g);
-    if (rating > best_rating &&
-        get_invention (plr, g->required_tech) != TECH_KNOWN) {
-      best_rating = rating;
-      best_government = i;
-    }
-  }
-  if (best_government == -1)
-    return A_NONE;
-  return get_government(best_government)->required_tech;
-}
-#endif /* NEW_GOV_EVAL */
-
 /**************************************************************************
   Returns tech corresponding to players wonder goal from nations[],
   if it makes sense, and wonder is not already built and not obsolete.
@@ -102,7 +76,7 @@ static Tech_Type_id get_wonder_tech(struct player *plr)
   int building = get_nation_by_plr(plr)->goals.wonder;
   
   if (improvement_exists(building)
-      && !game.global_wonders[building]
+      && game.global_wonders[building] == 0
       && !wonder_obsolete(building)) {
     Tech_Type_id tech = improvement_types[building].tech_req;
 
@@ -112,21 +86,23 @@ static Tech_Type_id get_wonder_tech(struct player *plr)
   return A_NONE;
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
 static void ai_next_tech_goal_default(struct player *pplayer, 
 				      struct ai_choice *choice)
 {
-  struct nation_type *prace;
+  struct nation_type *prace = get_nation_by_plr(pplayer);
   int bestdist = A_LAST + 1;
   int dist, i;
   Tech_Type_id goal = A_NONE;
   Tech_Type_id tech;
 
-  prace = get_nation_by_plr(pplayer);
   for (i = 0 ; i < MAX_NUM_TECH_GOALS; i++) {
     Tech_Type_id j = prace->goals.tech[i];
     if (!tech_exists(j) || get_invention(pplayer, j) == TECH_KNOWN) 
       continue;
-    dist = tech_goal_turns(pplayer, j);
+    dist = num_unknown_techs_for_goal(pplayer, j);
     if (dist < bestdist) { 
       bestdist = dist;
       goal = j;
@@ -135,7 +111,7 @@ static void ai_next_tech_goal_default(struct player *pplayer,
   } 
   tech = get_government_tech(pplayer);
   if (tech != A_NONE && tech_exists(tech)) {
-    dist = tech_goal_turns(pplayer, tech);
+    dist = num_unknown_techs_for_goal(pplayer, tech);
     if (dist < bestdist) { 
       bestdist = dist;
       goal = tech;
@@ -143,7 +119,7 @@ static void ai_next_tech_goal_default(struct player *pplayer,
   }
   tech = get_wonder_tech(pplayer);
   if (tech != A_NONE) {
-    dist = tech_goal_turns(pplayer, tech);
+    dist = num_unknown_techs_for_goal(pplayer, tech);
     if (dist < bestdist) { 
 /*    bestdist = dist; */ /* useless, reinclude when adding a new if statement */
       goal = tech;
@@ -158,7 +134,7 @@ static void ai_next_tech_goal_default(struct player *pplayer,
 static void adjust_tech_choice(struct player *pplayer, struct ai_choice *cur, 
 			       struct ai_choice *best, int advisor)
 {
-  if (cur->want) {
+  if (cur->want != 0) {
     leader_adjust_tech_choice(pplayer, cur, advisor);
     copy_if_better_choice(cur, best);
   }    
@@ -166,29 +142,6 @@ static void adjust_tech_choice(struct player *pplayer, struct ai_choice *cur,
 
 /* Syela-code starts here ................................. */
 
-static void find_prerequisites(struct player *pplayer, int i, int *prereq)
-{
-  /* add tech_want[i] / j to all subtechs */
-  Tech_Type_id t1, t2;
-  int known;
-
-  t1 = advances[i].req[0];
-  t2 = advances[i].req[1];
-  if(t1>=A_LAST || t2>=A_LAST) return;
-  known=get_invention(pplayer, t1);
-  if (known != TECH_KNOWN) {
-    prereq[t1]++;
-    if (known == TECH_UNKNOWN)
-      find_prerequisites(pplayer, t1, prereq);
-  }
-  known=get_invention(pplayer, t2);
-  if (known!= TECH_KNOWN) {
-    prereq[t2]++;
-    if (known== TECH_UNKNOWN)
-      find_prerequisites(pplayer, t2, prereq);
-  }
-}
-
 /**************************************************************************
   ...
 **************************************************************************/
@@ -200,36 +153,27 @@ static void ai_select_tech(struct player *pplayer, struct ai_choice *choice,
   int num_cities_nonzero;
   int values[A_LAST];
   int goal_values[A_LAST];
-  int prereq[A_LAST];
-  unsigned char cache[A_LAST][(A_LAST+7)/8];   /* bit vector for tech pairs */
 
-#define CACHE_SET(i,k)  cache[i][(k)/8] |= (1<<((k)%8))
-#define CACHE_TEST(i,k) cache[i][(k)/8] &  (1<<((k)%8))
-  
   if((num_cities_nonzero = city_list_size(&pplayer->cities)) < 1)
     num_cities_nonzero = 1;
   memset(values, 0, sizeof(values));
   memset(goal_values, 0, sizeof(goal_values));
-  memset(cache, 0, sizeof(cache));
   for (i = A_FIRST; i < game.num_tech_types; i++) {
-    j = pplayer->ai.tech_turns[i];
-    if (j) { /* if we already got it we don't want it */
+    j = num_unknown_techs_for_goal(pplayer, i);
+    if (j > 0) { /* if we already got it we don't want it */
       values[i] += pplayer->ai.tech_want[i];
-      memset(prereq, 0, sizeof(prereq));
-      find_prerequisites(pplayer, i, prereq);
       for (k = A_FIRST; k < game.num_tech_types; k++) {
-        if (prereq[k]) {
-	  CACHE_SET(i,k);
-          values[k] += pplayer->ai.tech_want[i] / j;
-        }
+	if (is_tech_a_req_for_goal(pplayer, k, i)) {
+	  values[k] += pplayer->ai.tech_want[i] / j;
+	}
       }
     }
   }
 
   for (i = A_FIRST; i < game.num_tech_types; i++) {
-    if (pplayer->ai.tech_turns[i]) {
+    if (num_unknown_techs_for_goal(pplayer, i) > 0) {
       for (k = A_FIRST; k < game.num_tech_types; k++) {
-	if (CACHE_TEST(i,k)) {
+	if (is_tech_a_req_for_goal(pplayer, k, i)) {
           goal_values[i] += values[k];
         }
       }
@@ -239,8 +183,8 @@ static void ai_select_tech(struct player *pplayer, struct ai_choice *choice,
 setting goal to Republic and learning Monarchy, but that's what it's supposed
 to be doing; it just looks strange. -- Syela */
       
-      goal_values[i] /= pplayer->ai.tech_turns[i];
-      if (pplayer->ai.tech_turns[i]<6) {
+      goal_values[i] /= num_unknown_techs_for_goal(pplayer, i);
+      if (num_unknown_techs_for_goal(pplayer, i) < 6) {
 	freelog(LOG_DEBUG, "%s: want = %d, value = %d, goal_value = %d",
 		advances[i].name, pplayer->ai.tech_want[i],
 		values[i], goal_values[i]);
@@ -274,23 +218,18 @@ to be doing; it just looks strange. -- Syela */
   }
   return;
 }
-#undef CACHE_SET
-#undef CACHE_TEST
 
+/**************************************************************************
+  ...
+**************************************************************************/
 static void ai_select_tech_goal(struct player *pplayer, struct ai_choice *choice)
 {
-  ai_select_tech(pplayer, 0, choice);
-}
-
-void calculate_tech_turns(struct player *pplayer)
-{
-  Tech_Type_id i;
-  memset(pplayer->ai.tech_turns, 0, sizeof(pplayer->ai.tech_turns));
-  for (i = A_FIRST; i < game.num_tech_types; i++) {
-    pplayer->ai.tech_turns[i] = tech_goal_turns(pplayer, i);
-  }
+  ai_select_tech(pplayer, NULL, choice);
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
 void ai_next_tech_goal(struct player *pplayer)
 {
   struct ai_choice bestchoice, curchoice;
@@ -298,7 +237,6 @@ void ai_next_tech_goal(struct player *pplayer)
   bestchoice.choice = A_NONE;      
   bestchoice.want   = 0;
 
-  calculate_tech_turns(pplayer);
   ai_select_tech_goal(pplayer, &curchoice);
   copy_if_better_choice(&curchoice, &bestchoice); /* not dealing with the rest */
 
@@ -320,21 +258,26 @@ void ai_next_tech_goal(struct player *pplayer)
   if (bestchoice.want == 0) {/* remove when the ai is done */
     ai_next_tech_goal_default(pplayer, &bestchoice); 
   }
-  if (bestchoice.want) 
+  if (bestchoice.want != 0) 
     pplayer->ai.tech_goal = bestchoice.choice;
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
 void ai_manage_tech(struct player *pplayer)
 {
   struct ai_choice choice, gol;
   int penalty;
 
-  penalty = (pplayer->got_tech ? 0 : pplayer->research.researched);
+  penalty = (pplayer->got_tech ? 0 : pplayer->research.bulbs_researched);
 
   ai_select_tech(pplayer, &choice, &gol);
   if (choice.choice != pplayer->research.researching) {
-    if ((choice.want - choice.type) > penalty &&             /* changing */
-   penalty + pplayer->research.researched <= research_time(pplayer)) {
+    /* changing */
+    if ((choice.want - choice.type) > penalty &&
+	penalty + pplayer->research.bulbs_researched <=
+	total_bulbs_required(pplayer)) {
       freelog(LOG_DEBUG, "%s switching from %s to %s with penalty of %d.",
 	      pplayer->name, advances[pplayer->research.researching].name,
 	      advances[choice.choice].name, penalty);
