@@ -26,23 +26,24 @@ enum cursor_hover_state {
 
 extern int hover_unit; /* unit hover_state applies to */
 extern enum cursor_hover_state hover_state;
-extern int draw_goto_line;
+extern bool draw_goto_line;
+extern bool non_ai_unit_focus;
 
 void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo);
 void do_unit_goto(int x, int y);
 void do_unit_nuke(struct unit *punit);
 void do_unit_paradrop_to(struct unit *punit, int x, int y);
-void do_unit_connect(struct unit *punit, int x, int y);
 void do_unit_patrol_to(struct unit *punit, int x, int y);
 void do_map_click(int xtile, int ytile);
 
 void set_hover_state(struct unit *punit, enum cursor_hover_state state);
 void handle_advance_focus(struct packet_generic_integer *packet);
 void request_center_focus_unit(void);
-void request_move_unit_direction(struct unit *punit, int dx, int dy); /* used by key_xxx */
+void request_move_unit_direction(struct unit *punit, int dir);
 void request_new_unit_activity(struct unit *punit, enum unit_activity act);
-void request_new_unit_activity_targeted(struct unit *punit, enum unit_activity act,
-					int tgt);
+void request_new_unit_activity_targeted(struct unit *punit,
+					enum unit_activity act,
+					enum tile_special_type tgt);
 void request_unit_auto(struct unit *punit);
 void request_unit_build_city(struct unit *punit);
 void request_unit_caravan_action(struct unit *punit, enum packet_type action);
@@ -94,7 +95,7 @@ void blink_active_unit(void);
 void update_unit_pix_label(struct unit *punit);
 
 void process_caravan_arrival(struct unit *punit);
-void process_diplomat_arrival(struct unit *punit, int victim_id);
+void process_diplomat_arrival(struct unit *pdiplomat, int victim_id);
 
 void key_cancel_action(void);
 void key_city_names_toggle(void);
