@@ -23,12 +23,16 @@
 #include "packets.h"
 #include "support.h"
 
+#include "climisc.h"
 #include "clinet.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 
 #include "chatline.h"
 
+struct genlist	history_list;
+int		history_pos;
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -39,9 +43,20 @@ void inputline_return(GtkWidget *w, gpointer data)
 
   theinput = gtk_entry_get_text(GTK_ENTRY(w));
   
-  if(GTK_WIDGET_IS_SENSITIVE(top_vbox) && *theinput) {
+  if (*theinput) {
     mystrlcpy(apacket.message, theinput, MAX_LEN_MSG-MAX_LEN_USERNAME+1);
     send_packet_generic_message(&aconnection, PACKET_CHAT_MSG, &apacket);
+
+    if (genlist_size(&history_list) >= MAX_CHATLINE_HISTORY) {
+      void *data;
+
+      data=genlist_get(&history_list, -1);
+      genlist_unlink(&history_list, data);
+      free(data);
+    }
+
+    genlist_insert(&history_list, mystrdup(theinput), 0);
+    history_pos=-1;
   }
 
   gtk_entry_set_text(GTK_ENTRY(w), "");
@@ -50,11 +65,12 @@ void inputline_return(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void append_output_window(char *astring)
+void real_append_output_window(const char *astring)
 {
   gtk_text_freeze(GTK_TEXT(main_message_area));
   gtk_text_insert(GTK_TEXT(main_message_area), NULL, NULL, NULL, "\n", -1);
-  gtk_text_insert(GTK_TEXT(main_message_area), NULL, NULL, NULL, astring, -1);
+  gtk_text_insert(GTK_TEXT(main_message_area), NULL, NULL, NULL, astring,
+		  -1);
   gtk_text_thaw(GTK_TEXT(main_message_area));
 
   /* move the scrollbar forward by a ridiculous amount */
@@ -67,18 +83,10 @@ void append_output_window(char *astring)
  the are somewhat related and append_output_window is already here.  --dwp
 **************************************************************************/
 void log_output_window(void)
-{ 
-  char *theoutput;
-  FILE *fp;
-  
-  append_output_window(_("Exporting output window to civgame.log ..."));
-  theoutput = gtk_editable_get_chars(GTK_EDITABLE(main_message_area), 0, -1);
-  fp = fopen("civgame.log", "w"); /* should allow choice of name? */
-  fprintf(fp, "%s", theoutput);
-  fclose(fp);
-  append_output_window(_("Export complete."));
+{
+  write_chatline_content(gtk_editable_get_chars
+			 (GTK_EDITABLE(main_message_area), 0, -1));
 }
-
 /**************************************************************************
 ...
 **************************************************************************/
