@@ -17,7 +17,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -121,7 +120,7 @@ static void rates_set_values(int tax, int no_tax_scroll,
 
   if(tax!=rates_tax_value) {
     my_snprintf(buf, sizeof(buf), "%3d%%", tax);
-    if ( strcmp( buf, GTK_LABEL(rates_tax_label)->label) )
+    if (strcmp(buf, GTK_LABEL(rates_tax_label)->label) != 0)
 	gtk_set_label(rates_tax_label,buf);
     if(!no_tax_scroll)
     {
@@ -134,7 +133,7 @@ static void rates_set_values(int tax, int no_tax_scroll,
 
   if(lux!=rates_lux_value) {
     my_snprintf(buf, sizeof(buf), "%3d%%", lux);
-    if ( strcmp( buf, GTK_LABEL(rates_lux_label)->label) )
+    if (strcmp(buf, GTK_LABEL(rates_lux_label)->label) != 0)
 	gtk_set_label(rates_lux_label,buf);
     if(!no_lux_scroll)
     {
@@ -147,7 +146,7 @@ static void rates_set_values(int tax, int no_tax_scroll,
 
   if(sci!=rates_sci_value) {
     my_snprintf(buf, sizeof(buf), "%3d%%", sci);
-    if ( strcmp( buf, GTK_LABEL(rates_sci_label)->label) )
+    if (strcmp(buf, GTK_LABEL(rates_sci_label)->label) != 0)
 	gtk_set_label(rates_sci_label,buf);
     if(!no_sci_scroll)
     {
@@ -381,17 +380,37 @@ static void option_ok_command_callback(GtkWidget *widget, gpointer data)
   for (o=options; o->name; ++o) {
     switch (o->type) {
     case COT_BOOL:
-      *(o->p_value) = GTK_TOGGLE_BUTTON(o->p_gui_data)->active;
+      *(o->p_bool_value) = GTK_TOGGLE_BUTTON(o->p_gui_data)->active;
       break;
     case COT_INT:
       dp = gtk_entry_get_text(GTK_ENTRY(o->p_gui_data));
-      *(o->p_value) = atoi(dp);
+      sscanf(dp, "%d", o->p_int_value);
+      break;
+    case COT_STR:
+      if (o->p_string_vals) {
+	mystrlcpy(o->p_string_value,
+		  gtk_entry_get_text(GTK_ENTRY
+				     (GTK_COMBO(o->p_gui_data)->entry)),
+		  o->string_length);
+      } else {
+	mystrlcpy(o->p_string_value,
+		  gtk_entry_get_text(GTK_ENTRY(o->p_gui_data)),
+		  o->string_length);
+      }
       break;
     }
   }
 
   gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(option_dialog_shell);
+
+  if (map_scrollbars) {
+    gtk_widget_show(map_horizontal_scrollbar);
+    gtk_widget_show(map_vertical_scrollbar);
+  } else {
+    gtk_widget_hide(map_horizontal_scrollbar);
+    gtk_widget_hide(map_vertical_scrollbar);
+  }
 }
 
 
@@ -433,12 +452,24 @@ static void create_option_dialog(void)
       break;
     case COT_INT:
       label = gtk_label_new(_(o->description));
-      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
       gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i+1);
       o->p_gui_data = gtk_entry_new_with_max_length(5);
       gtk_widget_set_usize(o->p_gui_data, 45, 0);
       gtk_table_attach_defaults(GTK_TABLE(table), o->p_gui_data, 1, 2, i, i+1);
       break;
+    case COT_STR:
+      label = gtk_label_new(_(o->description));
+      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+      gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, i, i+1);
+      if (o->p_string_vals) {
+        o->p_gui_data = gtk_combo_new();
+      } else {
+        o->p_gui_data = gtk_entry_new();
+      }
+      gtk_widget_set_usize(o->p_gui_data, 150, 0);
+      gtk_table_attach_defaults(GTK_TABLE(table), o->p_gui_data, 1, 2, i, i+1);
+      break;
     }
   }
 
@@ -469,12 +500,31 @@ void popup_option_dialog(void)
   for (o=options; o->name; ++o) {
     switch (o->type) {
     case COT_BOOL:
-      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(o->p_gui_data), *(o->p_value));
+      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(o->p_gui_data),
+				  *(o->p_bool_value));
       break;
     case COT_INT:
-      my_snprintf(valstr, sizeof(valstr), "%d", *(o->p_value));
+      my_snprintf(valstr, sizeof(valstr), "%d", *(o->p_int_value));
       gtk_entry_set_text(GTK_ENTRY(o->p_gui_data), valstr);
       break;
+    case COT_STR:
+      if (o->p_string_vals) {
+	int i;
+	GList *items = NULL;
+	const char **vals = (*o->p_string_vals) ();
+
+	for (i = 0; vals[i]; i++) {
+	  if (strcmp(vals[i], o->p_string_value) == 0) {
+	    continue;
+	  }
+	  items = g_list_append(items, (gpointer) vals[i]);
+	}
+	items = g_list_prepend(items, (gpointer) o->p_string_value);
+	gtk_combo_set_popdown_strings(GTK_COMBO(o->p_gui_data), items);
+      } else {
+	gtk_entry_set_text(GTK_ENTRY(o->p_gui_data), o->p_string_value);
+      }
+      break;
     }
   }
 
