@@ -17,7 +17,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include <gtk/gtk.h>
 
@@ -58,23 +57,20 @@ GdkCursor *		drop_cursor;
 GdkCursor *		nuke_cursor;
 GdkCursor *		patrol_cursor;
 
-static SPRITE *ctor_sprite_mask(GdkPixmap *mypixmap, GdkPixmap *mask,
-				int width, int height);
-
 /***************************************************************************
 ...
 ***************************************************************************/
-int isometric_view_supported(void)
+bool isometric_view_supported(void)
 {
-  return 1;
+  return TRUE;
 }
 
 /***************************************************************************
 ...
 ***************************************************************************/
-int overhead_view_supported(void)
+bool overhead_view_supported(void)
 {
-  return 1;
+  return TRUE;
 }
 
 /***************************************************************************
@@ -84,25 +80,39 @@ int overhead_view_supported(void)
 #define COLOR_MOTTO_FACE_G    0x71
 #define COLOR_MOTTO_FACE_B    0xE3
 
+/**************************************************************************
+...
+**************************************************************************/
+void gtk_draw_shadowed_string(GdkDrawable *drawable,
+			      GdkFont *fontset,
+			      GdkGC *black_gc,
+			      GdkGC *white_gc,
+			      gint x, gint y, const gchar *string)
+{
+  gdk_draw_string(drawable, fontset, black_gc, x + 1, y + 1, string);
+  gdk_draw_string(drawable, fontset, white_gc, x, y, string);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
 void load_intro_gfx( void )
 {
-  int tot, lin, y, w;
+  int tot, y, w, descent;
   char s[64];
   GdkColor face;
   GdkGC *face_gc;
-  char *motto = freeciv_motto();
+  const char *motto = freeciv_motto();
 
   /* metrics */
 
-  lin=main_font->ascent+main_font->descent;
-
   /* get colors */
 
   face.red  = COLOR_MOTTO_FACE_R<<8;
   face.green= COLOR_MOTTO_FACE_G<<8;
   face.blue = COLOR_MOTTO_FACE_B<<8;
 
-  gdk_imlib_best_color_get (&face);
+  gdk_imlib_best_color_get(&face);
 
   /* Main graphic */
 
@@ -111,39 +121,41 @@ void load_intro_gfx( void )
 
   face_gc = gdk_gc_new(root_window);
 
-  y=intro_gfx_sprite->height-(2*lin);
+  y = intro_gfx_sprite->height - (2 * gdk_string_height(main_fontset, motto));
 
-  w = gdk_string_width(main_font, motto);
-  gdk_gc_set_foreground (face_gc, &face);
-  gdk_draw_string(intro_gfx_sprite->pixmap, main_font,
-		  face_gc, tot/2-w/2, y, motto);
+  w = gdk_string_width(main_fontset, motto);
+  gdk_gc_set_foreground(face_gc, &face);
+  gdk_draw_string(intro_gfx_sprite->pixmap, main_fontset,
+		  face_gc, tot / 2 - w / 2, y, motto);
 
-  gdk_gc_destroy (face_gc);
+  gdk_gc_destroy(face_gc);
 
   /* Minimap graphic */
 
   radar_gfx_sprite = load_gfxfile(minimap_intro_filename);
-  tot=radar_gfx_sprite->width;
-
-  y=radar_gfx_sprite->height-(lin+((int)(1.5*main_font->descent)));
-
-  w = gdk_string_width(main_font, word_version());
-  gdk_draw_string(radar_gfx_sprite->pixmap, main_font,
-		  toplevel->style->black_gc, (tot/2-w/2)+1, y+1, word_version());
-  gdk_draw_string(radar_gfx_sprite->pixmap, main_font,
-		  toplevel->style->white_gc, tot/2-w/2, y, word_version());
-
-  y+=lin;
+  tot = radar_gfx_sprite->width;
 
   my_snprintf(s, sizeof(s), "%d.%d.%d%s",
 	      MAJOR_VERSION, MINOR_VERSION,
 	      PATCH_VERSION, VERSION_LABEL);
-  w = gdk_string_width( main_font, s );
-  gdk_draw_string(radar_gfx_sprite->pixmap, main_font,
-		  toplevel->style->black_gc, (tot/2-w/2)+1, y+1, s);
-  gdk_draw_string(radar_gfx_sprite->pixmap, main_font,
-		  toplevel->style->white_gc, tot/2-w/2, y, s);
 
+  gdk_string_extents(main_fontset, s, NULL, NULL, &w, NULL, &descent);
+  y = radar_gfx_sprite->height - descent - 5;
+  
+  gtk_draw_shadowed_string(radar_gfx_sprite->pixmap,
+			   main_fontset,
+			   toplevel->style->black_gc,
+			   toplevel->style->white_gc,
+			   tot / 2 - w / 2, y, s);
+
+  w  = gdk_string_width(main_fontset, word_version());
+  y -= gdk_string_height(main_fontset, s) + 3;
+
+  gtk_draw_shadowed_string(radar_gfx_sprite->pixmap,
+			   main_fontset,
+			   toplevel->style->black_gc,
+			   toplevel->style->white_gc,
+			   tot / 2 - w / 2, y, word_version());
   /* done */
 
   return;
@@ -156,19 +168,21 @@ struct Sprite *crop_sprite(struct Sprite *source,
 			   int x, int y,
 			   int width, int height)
 {
-  GdkPixmap *mypixmap, *mask;
+  GdkPixmap *mypixmap, *mask = NULL;
 
   mypixmap = gdk_pixmap_new(root_window, width, height, -1);
 
   gdk_draw_pixmap(mypixmap, civ_gc, source->pixmap, x, y, 0, 0,
 		  width, height);
 
-  mask=gdk_pixmap_new(mask_bitmap, width, height, 1);
-  gdk_draw_rectangle(mask, mask_bg_gc, TRUE, 0, 0, -1, -1 );
-  	    
-  gdk_draw_pixmap(mask, mask_fg_gc, source->mask, x, y, 0, 0,
-		  width, height);
-  
+  if (source->has_mask) {
+    mask = gdk_pixmap_new(mask_bitmap, width, height, 1);
+    gdk_draw_rectangle(mask, mask_bg_gc, TRUE, 0, 0, -1, -1 );
+
+    gdk_draw_pixmap(mask, mask_fg_gc, source->mask,
+		    x, y, 0, 0, width, height);
+  }
+
   return ctor_sprite_mask(mypixmap, mask, width, height);
 }
 
@@ -263,11 +277,12 @@ SPRITE *ctor_sprite_mask( GdkPixmap *mypixmap, GdkPixmap *mask,
     SPRITE *mysprite = fc_malloc(sizeof(SPRITE));
 
     mysprite->pixmap	= mypixmap;
-    mysprite->mask	= mask;
+
+    mysprite->has_mask = (mask != NULL);
+    mysprite->mask = mask;
 
     mysprite->width	= width;
     mysprite->height	= height;
-    mysprite->has_mask	= 1;
 
     return mysprite;
 }
@@ -293,6 +308,7 @@ char **gfx_fileextensions(void)
   static char *ext[] =
   {
     "xpm",
+    "png",
     NULL
   };
 
@@ -311,7 +327,7 @@ struct Sprite *load_gfxfile(const char *filename)
 
   if(!(im=gdk_imlib_load_image((char*)filename))) {
     freelog(LOG_FATAL, "Failed reading XPM file: %s", filename);
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   mysprite=fc_malloc(sizeof(struct Sprite));
@@ -320,7 +336,7 @@ struct Sprite *load_gfxfile(const char *filename)
 
   if(!gdk_imlib_render (im, w, h)) {
     freelog(LOG_FATAL, "failed render of sprite struct for %s", filename);
-    exit(1);
+    exit(EXIT_FAILURE);
   }
   
   mysprite->pixmap    = gdk_imlib_move_image (im);
@@ -330,7 +346,7 @@ struct Sprite *load_gfxfile(const char *filename)
   mysprite->width     = w;
   mysprite->height    = h;
 
-  gdk_imlib_destroy_image (im);
+  gdk_imlib_kill_image (im);
 
   return mysprite;
 }
@@ -393,3 +409,232 @@ void free_intro_radar_sprites(void)
     radar_gfx_sprite=NULL;
   }
 }
+
+/***************************************************************************
+ Draws a black border around the sprite. This is done by parsing the
+ mask and replacing the first and last pixel in every column and row
+ by a black one.
+***************************************************************************/
+void sprite_draw_black_border(SPRITE * sprite)
+{
+  GdkImage *mask_image, *pixmap_image;
+  int x, y;
+
+  pixmap_image =
+      gdk_image_get(sprite->pixmap, 0, 0, sprite->width, sprite->height);
+
+  mask_image =
+      gdk_image_get(sprite->mask, 0, 0, sprite->width, sprite->height);
+
+  /* parsing columns */
+  for (x = 0; x < sprite->width; x++) {
+    for (y = 0; y < sprite->height; y++) {
+      if (gdk_image_get_pixel(mask_image, x, y) != 0) {
+	gdk_image_put_pixel(pixmap_image, x, y, 0);
+	break;
+      }
+    }
+
+    for (y = sprite->height - 1; y > 0; y--) {
+      if (gdk_image_get_pixel(mask_image, x, y) != 0) {
+	gdk_image_put_pixel(pixmap_image, x, y, 0);
+	break;
+      }
+    }
+  }
+
+  /* parsing rows */
+  for (y = 0; y < sprite->height; y++) {
+    for (x = 0; x < sprite->width; x++) {
+      if (gdk_image_get_pixel(mask_image, x, y) != 0) {
+	gdk_image_put_pixel(pixmap_image, x, y, 0);
+	break;
+      }
+    }
+
+    for (x = sprite->width - 1; x > 0; x--) {
+      if (gdk_image_get_pixel(mask_image, x, y) != 0) {
+	gdk_image_put_pixel(pixmap_image, x, y, 0);
+	break;
+      }
+    }
+  }
+
+  gdk_draw_image(sprite->pixmap, civ_gc, pixmap_image, 0, 0, 0, 0,
+		 sprite->width, sprite->height);
+
+  gdk_image_destroy(mask_image);
+  gdk_image_destroy(pixmap_image);
+}
+
+/***************************************************************************
+  Scales a sprite. If the sprite contains a mask, the mask is scaled
+  as as well. The algorithm produces rather fast but beautiful
+  results.
+***************************************************************************/
+SPRITE* sprite_scale(SPRITE *src, int new_w, int new_h)
+{
+  SPRITE *dst = ctor_sprite_mask(NULL, NULL, new_w, new_h);
+  GdkImage *xi_src, *xi_dst, *xb_src;
+  guint32 pixel;
+  int xoffset_table[4096];
+  int x, xoffset, xadd, xremsum, xremadd;
+  int y, yoffset, yadd, yremsum, yremadd;
+
+  xi_src = gdk_image_get(src->pixmap, 0, 0, src->width, src->height);
+
+  xi_dst = gdk_image_new(GDK_IMAGE_FASTEST,
+			 gdk_window_get_visual(root_window), new_w, new_h);
+
+  /* for each pixel in dst, calculate pixel offset in src */
+  xadd = src->width / new_w;
+  xremadd = src->width % new_w;
+  xoffset = 0;
+  xremsum = new_w / 2;
+
+  for (x = 0; x < new_w; ++x) {
+    xoffset_table[x] = xoffset;
+    xoffset += xadd;
+    xremsum += xremadd;
+    if (xremsum >= new_w) {
+      xremsum -= new_w;
+      ++xoffset;
+    }
+  }
+
+  yadd = src->height / new_h;
+  yremadd = src->height % new_h;
+  yoffset = 0;
+  yremsum = new_h / 2;
+
+  if (src->has_mask) {
+    xb_src = gdk_image_get(src->mask, 0, 0, src->width, src->height);
+
+    dst->mask = gdk_pixmap_new(root_window, new_w, new_h, 1);
+    gdk_draw_rectangle(dst->mask, mask_bg_gc, TRUE, 0, 0, -1, -1);
+
+    for (y = 0; y < new_h; ++y) {
+      for (x = 0; x < new_w; ++x) {
+	pixel = gdk_image_get_pixel(xi_src, xoffset_table[x], yoffset);
+	gdk_image_put_pixel(xi_dst, x, y, pixel);
+
+	if (gdk_image_get_pixel(xb_src, xoffset_table[x], yoffset) != 0) {
+	  gdk_draw_point(dst->mask, mask_fg_gc, x, y);
+	}
+      }
+
+      yoffset += yadd;
+      yremsum += yremadd;
+      if (yremsum >= new_h) {
+	yremsum -= new_h;
+	++yoffset;
+      }
+    }
+
+    gdk_image_destroy(xb_src);
+  } else {
+    for (y = 0; y < new_h; ++y) {
+      for (x = 0; x < new_w; ++x) {
+	pixel = gdk_image_get_pixel(xi_src, xoffset_table[x], yoffset);
+	gdk_image_put_pixel(xi_dst, x, y, pixel);
+      }
+
+      yoffset += yadd;
+      yremsum += yremadd;
+      if (yremsum >= new_h) {
+	yremsum -= new_h;
+	++yoffset;
+      }
+    }
+  }
+
+  dst->pixmap = gdk_pixmap_new(root_window, new_w, new_h, -1);
+  gdk_draw_image(dst->pixmap, civ_gc, xi_dst, 0, 0, 0, 0, new_w, new_h);
+  gdk_image_destroy(xi_src);
+  gdk_image_destroy(xi_dst);
+
+  dst->has_mask = src->has_mask;
+  return dst;
+}
+
+/***************************************************************************
+ Method returns the bounding box of a sprite. It assumes a rectangular
+ object/mask. The bounding box contains the border (pixel which have
+ unset pixel as neighbours) pixel.
+***************************************************************************/
+void sprite_get_bounding_box(SPRITE * sprite, int *start_x,
+			     int *start_y, int *end_x, int *end_y)
+{
+  GdkImage *mask_image;
+  int i, j;
+
+  if (!sprite->has_mask || !sprite->mask) {
+    *start_x = 0;
+    *start_y = 0;
+    *end_x = sprite->width - 1;
+    *end_y = sprite->height - 1;
+    return;
+  }
+
+  mask_image =
+      gdk_image_get(sprite->mask, 0, 0, sprite->width, sprite->height);
+
+
+  /* parses mask image for the first column that contains a visible pixel */
+  *start_x = -1;
+  for (i = 0; i < sprite->width && *start_x == -1; i++) {
+    for (j = 0; j < sprite->height; j++) {
+      if (gdk_image_get_pixel(mask_image, i, j) != 0) {
+	*start_x = i;
+	break;
+      }
+    }
+  }
+
+  /* parses mask image for the last column that contains a visible pixel */
+  *end_x = -1;
+  for (i = sprite->width - 1; i >= *start_x && *end_x == -1; i--) {
+    for (j = 0; j < sprite->height; j++) {
+      if (gdk_image_get_pixel(mask_image, i, j) != 0) {
+	*end_x = i;
+	break;
+      }
+    }
+  }
+
+  /* parses mask image for the first row that contains a visible pixel */
+  *start_y = -1;
+  for (i = 0; i < sprite->height && *start_y == -1; i++) {
+    for (j = *start_x; j <= *end_x; j++) {
+      if (gdk_image_get_pixel(mask_image, j, i) != 0) {
+	*start_y = i;
+	break;
+      }
+    }
+  }
+
+  /* parses mask image for the last row that contains a visible pixel */
+  *end_y = -1;
+  for (i = sprite->height - 1; i >= *end_y && *end_y == -1; i--) {
+    for (j = *start_x; j <= *end_x; j++) {
+      if (gdk_image_get_pixel(mask_image, j, i) != 0) {
+	*end_y = i;
+	break;
+      }
+    }
+  }
+
+  gdk_image_destroy(mask_image);
+}
+ 
+/*********************************************************************
+ Crops all blankspace from a sprite (insofar as is possible as a rectangle)
+*********************************************************************/
+SPRITE *crop_blankspace(SPRITE *s)
+{
+  int x1, y1, x2, y2;
+
+  sprite_get_bounding_box(s, &x1, &y1, &x2, &y2);
+
+  return crop_sprite(s, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
+}
