@@ -81,13 +81,13 @@ typedef struct help_tree_node {
   int turns_to_tech;
 } help_tndata;
 
-char *help_ilabel_name[6] =
+static char *help_ilabel_name[6] =
 { N_("Cost:"), NULL, N_("Upkeep:"), NULL, N_("Requirement:"), NULL };
 
-char *help_wlabel_name[6] =
+static char *help_wlabel_name[6] =
 { N_("Cost:"), NULL, N_("Requirement:"), NULL, N_("Obsolete by:"), NULL };
 
-char *help_ulabel_name[5][5] =
+static char *help_ulabel_name[5][5] =
 {
     { N_("Cost:"),		NULL, NULL, N_("Attack:"),	NULL },
     { N_("Defense:"),		NULL, NULL, N_("Move:")	,	NULL },
@@ -96,7 +96,7 @@ char *help_ulabel_name[5][5] =
     { N_("Requirement:"),	NULL, NULL, N_("Obsolete by:"),	NULL }
 };
 
-char *help_tlabel_name[4][5] =
+static char *help_tlabel_name[4][5] =
 {
     { N_("Move/Defense:"),	NULL, NULL, N_("Food/Res/Trade:"),	NULL },
     { N_("Sp1 F/R/T:"),		NULL, NULL, N_("Sp2 F/R/T:"),		NULL },
@@ -118,9 +118,8 @@ static void select_help_item_string(const char *item,
 *****************************************************************/
 static void set_title_topic(char *topic)
 {
-  if(!strcmp(topic, "Freeciv")
-     || !strcmp(topic, "About")
-     || !strcmp(topic, _("About"))) {
+  if (strcmp(topic, "Freeciv") == 0 || strcmp(topic, "About") == 0
+      || strcmp(topic, _("About")) == 0) {
     gtk_frame_set_label(GTK_FRAME(help_frame), freeciv_name_version());
   } else {
     gtk_frame_set_label(GTK_FRAME(help_frame), topic);
@@ -135,14 +134,14 @@ void popdown_help_dialog(void)
 {
   if(help_dialog_shell) {
     gtk_widget_destroy(help_dialog_shell);
-    help_dialog_shell=0;
+    help_dialog_shell = NULL;
   }
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void popup_help_dialog_typed(char *item, enum help_page_type htype)
+void popup_help_dialog_typed(const char *item, enum help_page_type htype)
 {
 /*
   Position x, y;
@@ -168,7 +167,7 @@ void popup_help_dialog_typed(char *item, enum help_page_type htype)
 Not sure if this should call _(item) as it does, or whether all
 callers of this function should do so themselves... --dwp
 *****************************************************************/
-void popup_help_dialog_string(char *item)
+void popup_help_dialog_string(const char *item)
 {
   popup_help_dialog_typed(_(item), HELP_ANY);
 }
@@ -177,7 +176,7 @@ void popup_help_dialog_string(char *item)
 Called when destroying a node in the help_tree
 *****************************************************************/
 static void destroy_help_tndata(gpointer data) {
-  if (data != NULL)
+  if (data)
     free(data);
 }
 
@@ -188,12 +187,9 @@ duplicate subtrees are generated.
 static int help_tndata_compare(gpointer d1, gpointer d2) {
   help_tndata *a = (help_tndata *)d1;
   help_tndata *b = (help_tndata *)d2;
-  if (a == NULL)
+  if (!a)
     return (b != NULL);
-  if (a->tech == b->tech)
-    return FALSE;
-  else /* a->tech != b->tech */
-    return TRUE;
+  return (a->tech != b->tech);
 }      
 
 /**************************************************************************
@@ -214,7 +210,7 @@ static void create_tech_tree(GtkCTree *ctree, int tech, int levels,
   char          label [MAX_LEN_NAME+3];
   gboolean      leaf;
   help_tndata  *d = fc_malloc(sizeof(help_tndata));
-  int           original;
+  bool           original;
 
   text[0] = label;
   if ( advances[tech].req[0] == A_LAST && advances[tech].req[1] == A_LAST )
@@ -238,14 +234,11 @@ static void create_tech_tree(GtkCTree *ctree, int tech, int levels,
   default:		      bg = COLOR_STD_WHITE;	      break;
   }
   d->tech = tech;
-  d->turns_to_tech = tech_goal_turns(game.player_ptr, tech);
+  d->turns_to_tech = num_unknown_techs_for_goal(game.player_ptr, tech);
   l = gtk_ctree_find_by_row_data_custom(ctree, NULL, d, (GCompareFunc)help_tndata_compare);
-  if (l != NULL) {
-    /* l is the original in the tree. */
-    original = FALSE;
-  } else {
-    original = TRUE;
-  }
+  /* l is the original in the tree. */
+  original = (l == NULL);
+
   my_snprintf(label, sizeof(label), "%s:%d", advances[tech].name,
 	      d->turns_to_tech);
 
@@ -295,7 +288,7 @@ static int help_tech_tree_mouse_callback(GtkWidget *w, GdkEventButton *ev)
   int row;
   int col;
   /* check we have an event for the ctree */
-  if ((ev == NULL) || (ev->window != clist->clist_window))
+  if (!ev || (ev->window != clist->clist_window))
     return FALSE;
   /* check there is a row where the user clicked */
   if (gtk_clist_get_selection_info(clist, ev->x, ev->y, &row, &col) == FALSE)
@@ -378,8 +371,8 @@ static void help_hyperlink_callback(GtkWidget *w, enum help_page_type type)
   /* May be able to skip, or may need to modify, advances[A_NONE].name
      below, depending on which i18n is done elsewhere.
   */
-  if (strcmp(s, _("(Never)")) && strcmp(s, _("None"))
-      && strcmp(s, advances[A_NONE].name))
+  if (strcmp(s, _("(Never)")) != 0 && strcmp(s, _("None")) != 0
+      && strcmp(s, advances[A_NONE].name) != 0)
     select_help_item_string(s, type);
 }
 
@@ -391,7 +384,8 @@ static GtkWidget *help_hyperlink_new(GtkWidget *label, enum help_page_type type)
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
   gtk_container_add(GTK_CONTAINER(button), label);
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-  	  GTK_SIGNAL_FUNC(help_hyperlink_callback), (gpointer)type);
+		     GTK_SIGNAL_FUNC(help_hyperlink_callback),
+		     GINT_TO_POINTER(type));
 
   return button;
 }
@@ -402,7 +396,8 @@ static GtkWidget *help_slink_new(gchar *txt, enum help_page_type type)
 
   button = gtk_button_new_with_label(txt);
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-  	  GTK_SIGNAL_FUNC(help_hyperlink_callback), (gpointer)type);
+		     GTK_SIGNAL_FUNC(help_hyperlink_callback),
+		     GINT_TO_POINTER(type));
 
   return button;
 }
@@ -802,7 +797,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
   create_help_page(HELP_TECH);
 
-  if (i<game.num_tech_types&&i!=A_NONE) {
+  if (!is_future_tech(i)) {
     gtk_container_foreach(GTK_CONTAINER(help_vbox), (GtkCallback)gtk_widget_destroy, NULL);
 
     gtk_clist_freeze(GTK_CLIST(help_tree));
@@ -825,7 +820,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
     gtk_container_add(GTK_CONTAINER(help_vbox), w);
 
-    for(j=0; j<game.num_impr_types; ++j) {
+    impr_type_iterate(j) {
       if(i==improvement_types[j].tech_req) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
@@ -834,7 +829,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
         w = help_slink_new(improvement_types[j].name,
 			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = gtk_label_new(".");
+        w = gtk_label_new(Q_("?techhelp:."));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       }
       if(i==improvement_types[j].obsolete_by) {
@@ -845,11 +840,12 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
         w = help_slink_new(improvement_types[j].name,
 			  is_wonder(j)?HELP_WONDER:HELP_IMPROVEMENT);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = gtk_label_new(".");
+        w = gtk_label_new(Q_("?techhelp:."));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       }
-    }
-    for(j=0; j<game.num_unit_types; ++j) {
+    } impr_type_iterate_end;
+
+    unit_type_iterate(j) {
       if(i!=get_unit_type(j)->tech_requirement) continue;
       hbox = gtk_hbox_new(FALSE, 0);
       gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
@@ -857,9 +853,9 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       w = help_slink_new(get_unit_type(j)->name, HELP_UNIT);
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-      w = gtk_label_new(".");
+      w = gtk_label_new(Q_("?techhelp:."));
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-    }
+    } unit_type_iterate_end;
 
     for(j=0; j<game.num_tech_types; ++j) {
       if(i==advances[j].req[0]) {
@@ -870,7 +866,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = help_slink_new(advances[j].name, HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = gtk_label_new(".");
+          w = gtk_label_new(Q_("?techhelp:."));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	}
 	else {
@@ -1043,7 +1039,7 @@ static void help_update_government(const struct help_item *pitem,
 {
   char *buf = &long_buffer[0];
 
-  if (gov==NULL) {
+  if (!gov) {
     strcat(buf, pitem->text);
   } else {
     helptext_government(buf, gov-governments, pitem->text);
