@@ -28,52 +28,78 @@
 /****************************************************************
 ...
 *****************************************************************/
-char *input_dialog_get_input(GtkWidget *button)
+static void input_dialog_close(GtkWidget * shell)
 {
-  char *dp;
-  GtkWidget *winput;
-      
-  winput=gtk_object_get_data(GTK_OBJECT(button->parent->parent->parent),
-	"iinput");
-  
-  dp=gtk_entry_get_text(GTK_ENTRY(winput));
- 
-  return dp;
-}
+  GtkWidget *parent = gtk_object_get_data(GTK_OBJECT(shell),
+					  "parent");
+  void (*cancel_callback) (gpointer) = gtk_object_get_data(GTK_OBJECT(shell),
+							   "cancel_callback");
+  gpointer cancel_data = gtk_object_get_data(GTK_OBJECT(shell),
+					     "cancel_data");
 
+  if (cancel_callback) {
+    cancel_callback(cancel_data);
+  }
+
+  gtk_widget_set_sensitive(parent, TRUE);
+}
 
 /****************************************************************
 ...
 *****************************************************************/
-void input_dialog_destroy(GtkWidget *button)
+static void input_dialog_ok_callback(GtkWidget * w, gpointer data)
 {
-  GtkWidget *parent;
+  GtkWidget *shell = (GtkWidget *) data;
+  GtkWidget *parent = gtk_object_get_data(GTK_OBJECT(shell),
+					  "parent");
+  void (*ok_callback) (const char *, gpointer) =
+      gtk_object_get_data(GTK_OBJECT(shell),
+			  "ok_callback");
+  gpointer ok_data = gtk_object_get_data(GTK_OBJECT(shell),
+					 "ok_data");
+  GtkWidget *input = gtk_object_get_data(GTK_OBJECT(shell), "iinput");
 
-  parent=gtk_object_get_data(GTK_OBJECT(button->parent->parent->parent),
-	"parent");
+  ok_callback(gtk_entry_get_text(GTK_ENTRY(input)), ok_data);
 
   gtk_widget_set_sensitive(parent, TRUE);
 
-  gtk_widget_destroy(button->parent->parent->parent);
+  gtk_widget_destroy(shell);
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
-static gint input_dialog_del_callback(GtkWidget *w, GdkEvent *ev, gpointer data)
+static void input_dialog_cancel_callback(GtkWidget * w, gpointer data)
 {
-  gtk_widget_set_sensitive((GtkWidget *)data, TRUE);
+  GtkWidget *shell = (GtkWidget *) data;
+  
+  input_dialog_close(shell);
+
+  gtk_widget_destroy(shell);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+static gint input_dialog_del_callback(GtkWidget * w, GdkEvent * ev,
+				      gpointer data)
+{
+  GtkWidget *shell = (GtkWidget *) data;
+
+  input_dialog_close(shell);
+
   return FALSE;
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-GtkWidget *input_dialog_create(GtkWidget *parent, char *dialogname, 
-			   char *text, char *postinputtest,
-			   void *ok_callback, gpointer ok_cli_data, 
-			   void *cancel_callback, gpointer cancel_cli_data)
+GtkWidget *input_dialog_create(GtkWidget * parent, char *dialogname,
+			       char *text, char *postinputtest,
+			       void (*ok_callback) (const char *, gpointer),
+			       gpointer ok_data,
+			       void (*cancel_callback) (gpointer),
+			       gpointer cancel_data)
 {
   GtkWidget *shell, *label, *input, *ok, *cancel;
   
@@ -81,7 +107,7 @@ GtkWidget *input_dialog_create(GtkWidget *parent, char *dialogname,
   
   shell=gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(shell),"delete_event",
-	GTK_SIGNAL_FUNC(input_dialog_del_callback), parent);
+	GTK_SIGNAL_FUNC(input_dialog_del_callback), shell);
 
   gtk_window_set_title(GTK_WINDOW(shell), dialogname);
   
@@ -95,26 +121,30 @@ GtkWidget *input_dialog_create(GtkWidget *parent, char *dialogname,
   gtk_entry_set_text(GTK_ENTRY(input),postinputtest);
   
   gtk_signal_connect(GTK_OBJECT(input), "activate",
-	GTK_SIGNAL_FUNC(ok_callback), ok_cli_data);
+		     GTK_SIGNAL_FUNC(input_dialog_ok_callback), shell);
 
   ok=gtk_button_new_with_label(_("Ok"));
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(shell)->action_area),
 	ok, TRUE, TRUE, 0);
   gtk_signal_connect(GTK_OBJECT(ok), "clicked",
-	GTK_SIGNAL_FUNC(ok_callback), ok_cli_data);
+		     GTK_SIGNAL_FUNC(input_dialog_ok_callback), shell);
 
   cancel=gtk_button_new_with_label(_("Cancel"));
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(shell)->action_area),
 	cancel, TRUE, TRUE, 0);
   gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
-	GTK_SIGNAL_FUNC(cancel_callback), cancel_cli_data);
+	GTK_SIGNAL_FUNC(input_dialog_cancel_callback), shell);
 
   gtk_widget_grab_focus(input);
     
   gtk_set_relative_position(parent, shell, 10, 10);
 
-  gtk_object_set_data(GTK_OBJECT(shell), "iinput",input);
-  gtk_object_set_data(GTK_OBJECT(shell), "parent",parent);
+  gtk_object_set_data(GTK_OBJECT(shell), "iinput", input);
+  gtk_object_set_data(GTK_OBJECT(shell), "parent", parent);
+  gtk_object_set_data(GTK_OBJECT(shell), "ok_callback", ok_callback);
+  gtk_object_set_data(GTK_OBJECT(shell), "ok_data", ok_data);
+  gtk_object_set_data(GTK_OBJECT(shell), "cancel_callback", cancel_callback);
+  gtk_object_set_data(GTK_OBJECT(shell), "cancel_data", cancel_data);
 
   gtk_widget_show_all(GTK_DIALOG(shell)->vbox);
   gtk_widget_show_all(GTK_DIALOG(shell)->action_area);
