@@ -51,15 +51,14 @@
 
 #include "menu.h"
 
-GtkItemFactory *item_factory=NULL;
+static GtkItemFactory *item_factory = NULL;
+static enum unit_activity road_activity;
 
 static void menus_rename(const char *path, char *s);
 
 /****************************************************************
 ...
 *****************************************************************/
-enum unit_activity road_activity;
-
 enum MenuID {
   MENU_END_OF_LIST=0,
 
@@ -136,6 +135,8 @@ enum MenuID {
   MENU_HELP_CONNECTING,
   MENU_HELP_CONTROLS,
   MENU_HELP_CHATLINE,
+  MENU_HELP_WORKLIST_EDITOR,
+  MENU_HELP_CMA,
   MENU_HELP_PLAYING,
   MENU_HELP_IMPROVEMENTS,
   MENU_HELP_UNITS,
@@ -148,7 +149,7 @@ enum MenuID {
   MENU_HELP_HAPPINESS,
   MENU_HELP_SPACE_RACE,
   MENU_HELP_COPYING,
-  MENU_HELP_ABOUT,
+  MENU_HELP_ABOUT
 };
 
 
@@ -184,8 +185,7 @@ static void game_menu_callback(gpointer callback_data,
     disconnect_from_server();
     break;
   case MENU_GAME_QUIT:
-    exit(0);
-    break;
+    exit(EXIT_SUCCESS);
   }
 }
 
@@ -204,7 +204,7 @@ static void kingdom_menu_callback(gpointer callback_data,
     popup_find_dialog();
     break;
   case MENU_KINGDOM_WORKLISTS:
-    popup_worklists_dialog(game.player_ptr);
+    popup_worklists_report(game.player_ptr);
     break;
   case MENU_KINGDOM_REVOLUTION:
     popup_revolution_dialog();
@@ -304,9 +304,7 @@ static void orders_menu_callback(gpointer callback_data,
       struct unit *punit = get_unit_in_focus();
       /* Enable the button for adding to a city in all cases, so we
 	 get an eventual error message from the server if we try. */
-      if (can_unit_build_city(punit)
-	  || (unit_flag(punit->type, F_CITIES)
-	      && map_get_city(punit->x, punit->y))) {
+      if (can_unit_add_or_build_city(punit)) {
 	key_unit_build_city();
       } else {
 	key_unit_build_wonder();
@@ -466,6 +464,12 @@ static void help_menu_callback(gpointer callback_data,
   case MENU_HELP_CHATLINE:
     popup_help_dialog_string(HELP_CHATLINE_ITEM);
     break;
+  case MENU_HELP_WORKLIST_EDITOR:
+    popup_help_dialog_string(HELP_WORKLIST_EDITOR_ITEM);
+    break;
+  case MENU_HELP_CMA:
+    popup_help_dialog_string(HELP_CMA_ITEM);
+    break;
   case MENU_HELP_PLAYING:
     popup_help_dialog_string(HELP_PLAYING_ITEM);
     break;
@@ -735,6 +739,10 @@ static GtkItemFactoryEntry menu_items[]	=
 	help_menu_callback,	MENU_HELP_CONTROLS					},
   { "/" N_("Help") "/" N_("C_hatline"),			NULL,
 	help_menu_callback,	MENU_HELP_CHATLINE					},
+  { "/" N_("Help") "/" N_("_Worklist Editor"),			NULL,
+	help_menu_callback,	MENU_HELP_WORKLIST_EDITOR				},
+  { "/" N_("Help") "/" N_("Citizen _Management"),			NULL,
+	help_menu_callback,	MENU_HELP_CMA						},
   { "/" N_("Help") "/" N_("_Playing"),			NULL,
 	help_menu_callback,	MENU_HELP_PLAYING					},
   { "/" N_("Help") "/sep1",				NULL,
@@ -794,7 +802,7 @@ static const char *translate_menu_path(const char *path, int remove_uline)
   freelog(LOG_DEBUG, "trans: %s", in.str);
 
   s = in.str;
-  while ((tok=strtok(s, "/")) != NULL) {
+  while ((tok=strtok(s, "/"))) {
     len = strlen(tok);
     freelog(LOG_DEBUG, "tok \"%s\", len %d", tok, len);
     if (len && tok[0] == '<' && tok[len-1] == '>') {
@@ -833,7 +841,7 @@ static const char *translate_menu_path(const char *path, int remove_uline)
 *****************************************************************/
 void setup_menus(GtkWidget *window, GtkWidget **menubar)
 {
-  const int nmenu_items = sizeof(menu_items)/sizeof(menu_items[0]);
+  const int nmenu_items = ARRAY_SIZE(menu_items);
   GtkAccelGroup *accel;
   int i;
 
@@ -961,8 +969,13 @@ void update_menus(void)
     menus_set_sensitive("<main>/_Reports", TRUE);
     menus_set_sensitive("<main>/_Kingdom", TRUE);
     menus_set_sensitive("<main>/_View", TRUE);
-    menus_set_sensitive("<main>/_Orders", TRUE);
-  
+    menus_set_sensitive("<main>/_Orders", !client_is_observer());
+
+    menus_set_sensitive("<main>/_Kingdom/_Tax Rates", !client_is_observer());
+    menus_set_sensitive("<main>/_Kingdom/Work_lists", !client_is_observer());
+    menus_set_sensitive("<main>/_Kingdom/_Revolution",
+			!client_is_observer());
+
     menus_set_sensitive("<main>/_Reports/S_paceship",
 			(game.player_ptr->spaceship.state!=SSHIP_NONE));
 
@@ -984,6 +997,12 @@ void update_menus(void)
     menus_set_sensitive("<main>/_View/Focus Unit", !draw_units);
     menus_set_active("<main>/_View/Fog of War", draw_fog_of_war);
 
+    /* Remaining part of this function: Update Orders menu */
+
+    if (client_is_observer()) {
+      return;
+    }
+
     if((punit=get_unit_in_focus())) {
       char *irrfmt = _("Change to %s (_I)");
       char *minfmt = _("Change to %s (_M)");
@@ -1000,10 +1019,8 @@ void update_menus(void)
       /* Enable the button for adding to a city in all cases, so we
 	 get an eventual error message from the server if we try. */
       menus_set_sensitive("<main>/_Orders/_Build City",
-			  (can_unit_build_city(punit) ||
-                           (unit_flag(punit->type, F_CITIES)
-			    && map_get_city(punit->x, punit->y)) ||
-                           unit_can_help_build_wonder_here(punit)));
+			  can_unit_add_or_build_city(punit) ||
+			  unit_can_help_build_wonder_here(punit));
       menus_set_sensitive("<main>/_Orders/Build _Road",
                           (can_unit_do_activity(punit, ACTIVITY_ROAD) ||
                            can_unit_do_activity(punit, ACTIVITY_RAILROAD) ||
@@ -1048,10 +1065,10 @@ void update_menus(void)
                            && diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION,
 						     punit->x, punit->y)));
       menus_set_sensitive("<main>/_Orders/Explode Nuclear",
-			  unit_flag(punit->type, F_NUCLEAR));
-      if (unit_flag(punit->type, F_CARAVAN))
+			  unit_flag(punit, F_NUCLEAR));
+      if (unit_flag(punit, F_HELP_WONDER))
 	menus_rename("<main>/_Orders/_Build City", _("Help _Build Wonder"));
-      else if (unit_flag(punit->type, F_CITIES)) {
+      else if (unit_flag(punit, F_CITIES)) {
 	if (map_get_city(punit->x, punit->y))
 	  menus_rename("<main>/_Orders/_Build City", _("Add to City (_B)"));
 	else
@@ -1060,10 +1077,10 @@ void update_menus(void)
       else 
 	menus_rename("<main>/_Orders/_Build City", _("_Build City"));
  
-      if (unit_flag(punit->type, F_CARAVAN))
+      if (unit_flag(punit, F_TRADE_ROUTE))
 	menus_rename("<main>/_Orders/Build _Road", _("Make Trade _Route"));
-      else if (unit_flag(punit->type, F_SETTLERS)) {
-	if (map_get_tile(punit->x,punit->y)->special&S_ROAD) {
+      else if (unit_flag(punit, F_SETTLERS)) {
+	if (map_has_special(punit->x, punit->y, S_ROAD)) {
 	  roadtext = _("Build _Railroad");
 	  road_activity=ACTIVITY_RAILROAD;  
 	} 
@@ -1083,7 +1100,7 @@ void update_menus(void)
 	  my_snprintf (irrtext, sizeof(irrtext), irrfmt,
 		   (get_tile_type(tinfo->irrigation_result))->terrain_name);
 	}
-      else if ((map_get_tile(punit->x,punit->y)->special&S_IRRIGATION) &&
+      else if (map_has_special(punit->x, punit->y, S_IRRIGATION) &&
 	       player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND))
 	{
 	  sz_strlcpy (irrtext, _("Bu_ild Farmland"));
@@ -1108,12 +1125,12 @@ void update_menus(void)
       else
 	menus_rename("<main>/_Orders/Build _Fortress", _("Build _Fortress"));
 
-      if (unit_flag(punit->type, F_PARATROOPERS))
+      if (unit_flag(punit, F_PARATROOPERS))
 	menus_rename("<main>/_Orders/Clean _Pollution", _("_Paradrop"));
       else
 	menus_rename("<main>/_Orders/Clean _Pollution", _("Clean _Pollution"));
 
-      if (!unit_flag(punit->type, F_SETTLERS))
+      if (!unit_flag(punit, F_SETTLERS))
 	menus_rename("<main>/_Orders/_Auto Settler", _("_Auto Attack"));
       else
 	menus_rename("<main>/_Orders/_Auto Settler", _("_Auto Settler"));
