@@ -14,7 +14,6 @@
 #include <config.h>
 #endif
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -444,21 +443,20 @@ static void create_improvements_list(struct city *pcity)
 
   if(wnd)
   {
-    int i;
     int any_improvements=FALSE;
+
     DoMethod(listview, MUIM_NList_InsertSingle, 100-1,MUIV_NList_Insert_Bottom);
-    for(i=0; i<game.num_impr_types; i++)
-    {
-      if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i))
+    built_impr_iterate(pcity, i) {
+      if (i != B_PALACE && !is_wonder(i))
       {
         DoMethod(listview, MUIM_NList_InsertSingle, i+100,MUIV_NList_Insert_Bottom);
         any_improvements = TRUE;
       }
-    }
+    } built_impr_iterate_end;
 
     if (any_improvements)
     {
-      DoMethod(listview, MUIM_NList_InsertSingle, i+game.num_impr_types,MUIV_NList_Insert_Bottom);
+      DoMethod(listview, MUIM_NList_InsertSingle, B_LAST + 100, MUIV_NList_Insert_Bottom);
     }
 
     DoMethod(wnd,MUIM_Notify, MUIA_Window_CloseRequest, TRUE, app, 4, MUIM_CallHook, &civstandard_hook, spy_close, wnd);
@@ -502,8 +500,12 @@ void popup_incite_dialog(struct city *pcity)
 {
   char buf[128];
 
-  if(game.player_ptr->economic.gold>=pcity->incite_revolt_cost)
-  {
+  if (pcity->incite_revolt_cost == INCITE_IMPOSSIBLE_COST) {
+    my_snprintf(buf, sizeof(buf), _("You can't incite a revolt in %s."),
+		pcity->name);
+    popup_message_dialog(main_wnd, _("City can't be incited!"), buf,
+			 _("Darn"), diplomat_incite_no, 0, 0);
+  } else if (game.player_ptr->economic.gold >= pcity->incite_revolt_cost) {
     my_snprintf(buf, sizeof(buf),_("Incite a revolt for %d gold?\nTreasury contains %d gold."),
             pcity->incite_revolt_cost, game.player_ptr->economic.gold);
     diplomat_target_id = pcity->id;
@@ -784,7 +786,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
  
     diplomat_target_id=pcity->id;
 
-    if(!unit_flag(punit->type, F_SPY))
+    if(!unit_flag(punit, F_SPY))
     {
       if(diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, dest_x, dest_y))
       {
@@ -943,7 +945,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
       msg_dlg[i].label = NULL;
 
       popup_message_dialog_args(main_wnd, _("Subvert Enemy Unit"),
-                               (!unit_flag(punit->type, F_SPY))?
+                               (!unit_flag(punit, F_SPY))?
                                 _("Sir, the diplomat is waiting for your command"):
                                 _("Sir, the spy is waiting for your command"),
                                 msg_dlg);
@@ -955,7 +957,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 /****************************************************************
 ...
 *****************************************************************/
-int diplomat_dialog_is_open(void)
+bool diplomat_dialog_is_open(void)
 {
   return diplomat_dialog_open;
 }
@@ -1076,7 +1078,7 @@ void popup_caravan_dialog(struct unit *punit, struct city *phomecity, struct cit
 /****************************************************************
 ...
 *****************************************************************/
-int caravan_dialog_is_open(void)
+bool caravan_dialog_is_open(void)
 {
   return caravan_dialog;
 }
@@ -1183,16 +1185,18 @@ static void pillage_button(struct popup_message_data *msg)
 /****************************************************************
 ...
 *****************************************************************/
-void popup_pillage_dialog(struct unit *punit, int may_pillage)
+void popup_pillage_dialog(struct unit *punit,
+			  enum tile_special_type may_pillage)
 {
   if(!is_showing_pillage_dialog)
   {
-    int may_pillage_save = may_pillage;
+    enum tile_special_type may_pillage_save = may_pillage;
     int count=0;
 
-    while (may_pillage)
+    while (may_pillage != S_NO_SPECIAL)
     {
-      int what = get_preferred_pillage(may_pillage);
+      enum tile_special_type what = get_preferred_pillage(may_pillage);
+
       may_pillage &= (~(what | map_get_infrastructure_prerequisite (what)));
       count++;
     }
@@ -1211,7 +1215,7 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
 
 	for(i=0;i<count;i++)
 	{
-	  int what = get_preferred_pillage (may_pillage);
+	  enum tile_special_type what = get_preferred_pillage (may_pillage);
 	
           msg_dlg[i].label = mystrdup(map_get_infrastructure_text(what));
           msg_dlg[i].function = (APTR)pillage_button;
@@ -1513,7 +1517,7 @@ void popup_unit_select_dialog(struct tile *ptile)
       for(i=0;i<n;i++)
       {
         struct unit *punit=unit_list_get(&ptile->units, i);
-        struct unit_type *punittemp=get_unit_type(punit->type);
+        struct unit_type *punittemp=unit_type(punit);
         struct city *pcity;
 
         Object *o;
@@ -1634,8 +1638,6 @@ static void nations_ok(void)
     return;
   }
 
-  packet.name[0]=toupper(packet.name[0]);
-
   send_packet_alloc_nation(&aconnection, &packet);
 }
 
@@ -1823,35 +1825,9 @@ void popdown_races_dialog(void)
 /****************************************************************
 ...
 *****************************************************************/
-void races_toggles_set_sensitive(int bits1, int bits2)
+void races_toggles_set_sensitive(struct packet_nations_used *packet)
 {
-// remove me
-#ifdef DISABLED
-  int i, selected, mybits;
-
-  mybits=bits1;
-
-  for(i=0; i<game.playable_nation_count && i<32; i++) {
-    if(mybits&1)
-      gtk_widget_set_sensitive( races_toggles[i], FALSE );
-    else
-      gtk_widget_set_sensitive( races_toggles[i], TRUE );
-    mybits>>=1;
-  }
-
-  mybits=bits2;
-
-  for(i=32; i<game.playable_nation_count; i++) {
-    if(mybits&1)
-      gtk_widget_set_sensitive( races_toggles[i], FALSE );
-    else
-      gtk_widget_set_sensitive( races_toggles[i], TRUE );
-    mybits>>=1;
-  }
-
-  if((selected=races_buttons_get_current())==-1)
-     return;
-#endif
+  /* not implemented */
 }
 
 /****************************************************************
@@ -1916,4 +1892,3 @@ void popup_upgrade_dialog(struct unit *punit)
     }
   }
 }
-
