@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -28,7 +29,6 @@
 #include <proto/graphics.h>
 
 #include "city.h"
-#include "colors.h"
 #include "fcintl.h"
 #include "game.h"
 #include "genlist.h"
@@ -112,14 +112,10 @@ static void set_title_topic(char *topic)
 {
   char *text;
 
-  if (!strcmp(topic, "Freeciv")
-      || !strcmp(topic, "About")
-      || !strcmp(topic, _("About")))
-  {
+  if (strcmp(topic, "Freeciv") == 0 || strcmp(topic, "About") == 0
+      || strcmp(topic, _("About")) == 0) {
     text = freeciv_name_version();
-  }
-  else
-  {
+  } else {
     text = topic;
   }
 
@@ -159,8 +155,8 @@ static void help_hyperlink(Object ** text_obj)
   char *s = (char *) xget(*text_obj, MUIA_Text_Contents); /* works also for ColorButtons */
   enum help_page_type type = (enum help_page_type) xget(*text_obj, MUIA_UserData);
 
-  if (strcmp(s, _("(Never)")) && strcmp(s, _("None"))
-      && strcmp(s, advances[A_NONE].name))
+  if (strcmp(s, _("(Never)")) != 0 && strcmp(s, _("None")) != 0
+      && strcmp(s, advances[A_NONE].name) != 0)
   {
     int idx;
 
@@ -280,7 +276,7 @@ static void UpdateTechButton(Object *o, int tech)
 /****************************************************************
 ...
 *****************************************************************/
-void popup_help_dialog_typed(char *item, enum help_page_type htype)
+void popup_help_dialog_typed(const char *item, enum help_page_type htype)
 {
   if (!help_wnd)
   {
@@ -345,7 +341,7 @@ void popup_help_dialog_typed(char *item, enum help_page_type htype)
 /****************************************************************
 ...
 *****************************************************************/
-void popup_help_dialog_string(char *item)
+void popup_help_dialog_string(const char *item)
 {
   popup_help_dialog_typed(_(item), HELP_ANY);
 }
@@ -399,8 +395,8 @@ static void create_tech_tree(Object *tree, APTR parent, int tech, int levels)
     return;
   }
   
-  my_snprintf(label,sizeof(label), "%s:%d",advances[tech].name,
-                        tech_goal_turns(game.player_ptr,tech));
+  my_snprintf(label, sizeof(label), "%s:%d", advances[tech].name,
+	      num_unknown_techs_for_goal(game.player_ptr, tech));
 
   o = ColorTextObject,
       ButtonFrame,
@@ -720,7 +716,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
   create_help_page(HELP_TECH);
 
-  if (i < game.num_tech_types && i != A_NONE)
+  if (!is_future_tech(i))
   {
     if (help_tech_group)
     {
@@ -739,8 +735,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
       if (help_tech_group)
       {
-	for (j = 0; j < game.num_impr_types; ++j)
-	{
+	impr_type_iterate(j) {
 	  Object *o, *button;
 	  if (i != improvement_types[j].tech_req)
 	    continue;
@@ -755,10 +750,9 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
 	  if (o)
 	    DoMethod(help_tech_group, OM_ADDMEMBER, o);
-	}
+	} impr_type_iterate_end;
 
-	for (j = 0; j < game.num_unit_types; ++j)
-	{
+	unit_type_iterate(j) {
 	  Object *o, *button;
 	  if (i != get_unit_type(j)->tech_requirement)
 	    continue;
@@ -773,7 +767,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
 	  if (o)
 	    DoMethod(help_tech_group, OM_ADDMEMBER, o);
-	}
+	} unit_type_iterate_end;
 
 
 	for (j = 0; j < game.num_tech_types; ++j)
