@@ -36,6 +36,7 @@
 #include "timing.h"
 
 #include "civclient.h"
+#include "climisc.h"
 #include "citydlg.h"
 #include "control.h"
 #include "colors.h"
@@ -53,38 +54,6 @@
 #include "overviewclass.h"
 #include "scrollbuttonclass.h"
 
-/**************************************************************************
-  Find the "best" city to associate with the selected tile.  
-    a.  A city working the tile is the best
-    b.  If no city is working the tile, choose a city that could work the tile
-    c.  If multiple cities could work it, choose the most recently "looked at"
-    d.  If none of the cities were looked at last, choose "randomly"
-    e.  If another player is working the tile, or no cities can work it,
-	return NULL
-**************************************************************************/
-static struct city *find_city_near_tile(int x, int y)
-{
-  struct tile *ptile=map_get_tile(x, y);
-  struct city *pcity, *pcity2;
-  static struct city *last_pcity=NULL;
-
-  if((pcity=ptile->worked))  {
-    if(pcity->owner==game.player_idx)  return last_pcity=pcity;   /* rule a */
-    else return NULL;	 /* rule e */
-  }
-
-  pcity2 = NULL;
-  city_map_iterate(i, j)  {
-    pcity = map_get_city(x+i-2, y+j-2);
-    if(pcity && pcity->owner==game.player_idx && 
-       get_worker_city(pcity,4-i,4-j)==C_TILE_EMPTY)  {  /* rule b */
-      if(pcity==last_pcity) return pcity;  /* rule c */
-      pcity2 = pcity;
-    }
-  } city_map_iterate_end;
-  return last_pcity = pcity2;
-}
-
 /****************************************************************
  TilePopWindow Custom Class
 *****************************************************************/
@@ -132,7 +101,7 @@ static ULONG TilePopWindow_New(struct IClass *cl, Object * o, struct opSet *msg)
       text_obj = TextObject, MUIA_Text_Contents, s, End;
       DoMethod(group, OM_ADDMEMBER, text_obj);
 
-      if (ptile->special & S_HUT)
+      if (tile_has_special(ptile, S_HUT))
       {
 	text_obj = TextObject, MUIA_Text_Contents, _("Minor Tribe Village"), End;
 	DoMethod(group, OM_ADDMEMBER, text_obj);
@@ -140,7 +109,8 @@ static ULONG TilePopWindow_New(struct IClass *cl, Object * o, struct opSet *msg)
 
       if ((pcity = map_get_city(xtile, ytile)))
       {
-	my_snprintf(s, sizeof(s), _("City: %s(%s)"), pcity->name, get_nation_name(game.players[pcity->owner].nation));
+	my_snprintf(s, sizeof(s), _("City: %s(%s)"), pcity->name,
+		    get_nation_name(city_owner(pcity)->nation));
 	text_obj = TextObject, MUIA_Text_Contents, s, End;
 
 	if(text_obj)
@@ -164,7 +134,7 @@ static ULONG TilePopWindow_New(struct IClass *cl, Object * o, struct opSet *msg)
       if ((punit = find_visible_unit(ptile)) && !pcity)
       {
 	char cn[64];
-	struct unit_type *ptype = get_unit_type(punit->type);
+	struct unit_type *ptype = unit_type(punit);
 	cn[0] = '\0';
 	if (punit->owner == game.player_idx)
 	{
@@ -173,7 +143,8 @@ static ULONG TilePopWindow_New(struct IClass *cl, Object * o, struct opSet *msg)
 	  if (pcity)
 	    my_snprintf(cn, sizeof(cn), "/%s", pcity->name);
 	}
-	my_snprintf(s, sizeof(s), _("Unit: %s(%s%s)"), ptype->name, get_nation_name(game.players[punit->owner].nation), cn);
+	my_snprintf(s, sizeof(s), _("Unit: %s(%s%s)"), ptype->name,
+		    get_nation_name(unit_owner(punit)->nation), cn);
 
 	text_obj = TextObject, MUIA_Text_Contents, s, End;
 	DoMethod(group, OM_ADDMEMBER, text_obj);
@@ -488,27 +459,8 @@ static void show_desc_at_tile(Object *o, struct Map_Data *data, int x, int y)
     }
 
     if (draw_city_productions && (pcity->owner==game.player_idx)) {
-      int turns;
-      struct unit_type *punit_type;
-      struct impr_type *pimprovement_type;
+      get_city_mapview_production(pcity, buffer, sizeof(buffer));
 
-      turns = city_turns_to_build(pcity, pcity->currently_building,
-				  pcity->is_building_unit);
-
-      if (pcity->is_building_unit) {
-	punit_type = get_unit_type(pcity->currently_building);
-	my_snprintf(buffer, sizeof(buffer), "%s %d",
-		    punit_type->name, turns);
-      } else {
-	pimprovement_type =
-	  get_improvement_type(pcity->currently_building);
-	if (pcity->currently_building == B_CAPITAL) {
-	  sz_strlcpy(buffer, pimprovement_type->name);
-	} else {
-	  my_snprintf(buffer, sizeof(buffer), "%s %d",
-		      pimprovement_type->name, turns);
-	}
-      }
       w = TextLength(rp, buffer, strlen(buffer));
       pix_x = _mleft(o) + canvas_x + NORMAL_TILE_WIDTH/2 - w/2;
 
@@ -560,12 +512,11 @@ static void Map_Priv_ShowCityDescriptions(Object *o, struct Map_Data *data)
         }
       }
     } else { /* is_isometric */
-      int x, y;
       int x1, y1;
       for (x1 = 0; x1 < map_canvas_store_twidth; x1++) {
-        x = (map_view_x0 + x1) % map.xsize;
         for (y1 = 0; y1 < map_canvas_store_theight; y1++) {
-	  y = map_view_y0 + y1;
+          int x = map_view_x0 + x1;
+	  int y = map_view_y0 + y1;
 	  if (normalize_map_pos(&x, &y)) {
 	    show_desc_at_tile(o, data, x, y);
 	  }
@@ -1059,10 +1010,25 @@ static ULONG Map_Show(struct IClass * cl, Object * o, Msg msg)
       InstallLayerInfoHook(data->map_layerinfo, LAYERS_NOBACKFILL);
       if ((data->map_layer = CreateBehindHookLayer(data->map_layerinfo, data->map_bitmap, 0, 0, _mwidth(o) - 1, _mheight(o) - 1, LAYERSIMPLE, LAYERS_NOBACKFILL, NULL)))
       {
+      	int xsize;
+      	int ysize;
+
+	/* determine sizes. The usually way was to use map.xsize (or map.ysize) but
+	 * this won't work anymore */
+	if (data->overview_object)
+	{
+	    xsize = xget(data->overview_object, MUIA_Overview_Width);
+	    ysize = xget(data->overview_object, MUIA_Overview_Height);
+	} else
+	{
+	    xsize = map.xsize;
+	    ysize = map.ysize;
+	}
+
 	if (data->hscroller_object)
 	{
 	  SetAttrs(data->hscroller_object,
-		   MUIA_Prop_Entries, map.xsize + xget(o, MUIA_Map_HorizVisible) - 1,
+		   MUIA_Prop_Entries, xsize + xget(o, MUIA_Map_HorizVisible) - 1,
 		   MUIA_Prop_Visible, xget(o, MUIA_Map_HorizVisible),
 		   MUIA_NoNotify, TRUE,
 		   TAG_DONE);
@@ -1071,7 +1037,7 @@ static ULONG Map_Show(struct IClass * cl, Object * o, Msg msg)
 	if (data->vscroller_object)
 	{
 	  SetAttrs(data->vscroller_object,
-		   MUIA_Prop_Entries, map.ysize,
+		   MUIA_Prop_Entries, ysize,
 		   MUIA_Prop_Visible, xget(o, MUIA_Map_VertVisible),
 		   MUIA_NoNotify, TRUE,
 		   TAG_DONE);
@@ -1141,56 +1107,59 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
       	return 0;
       }
 
-// implement me
-#ifdef DISABLED
       if (data->update == 6)
       {
-      	/* Draw Mushroom */
-	int x, y, w, h;
+      	int x = data->mushroom_x0;
+      	int y = data->mushroom_y0;
+
 	APTR cliphandle = MUI_AddClipping(muiRenderInfo(o), _mleft(o), _mtop(o), _mwidth(o), _mheight(o));
 
-	for(y=0; y<3; y++) {
-	  for(x=0; x<3; x++) {
-	    int map_x = map_canvas_adjust_x(x - 1 + data->mushroom_x0) * get_normal_tile_width();
-	    int map_y = map_canvas_adjust_y(y - 1 + data->mushroom_y0) * get_normal_tile_height();
-	    struct Sprite *mysprite = sprites.explode.nuke[y][x];
+	if (is_isometric)
+	{
+	  /* Do I get points for style? */
+/*	  char boom[] = "Really Loud BOOM!!!";
+	  int w = gdk_string_width(city_productions_font, boom);
+	  int canvas_x, canvas_y;
 
-	    put_sprite_overlay( _rp(o), mysprite, _mleft(o) + map_x, _mtop(o) + map_y);
-	  }
+	  get_canvas_xy(x, y, &canvas_x, &canvas_y);
+	  draw_shadowed_string(map_canvas->window, main_font,
+			 toplevel->style->black_gc,
+			 toplevel->style->white_gc,
+			 canvas_x + NORMAL_TILE_WIDTH / 2 - w / 2,
+			 canvas_y + NORMAL_TILE_HEIGHT,
+			 boom); */
+
+	} else
+	{
+	  int x_itr, y_itr;
+	  int canvas_x, canvas_y;
+
+	  for (y_itr=0; y_itr<3; y_itr++) {
+	    for (x_itr=0; x_itr<3; x_itr++) {
+	      struct Sprite *mysprite = sprites.explode.nuke[y_itr][x_itr];
+	      get_canvas_xy(x + x_itr - 1, y + y_itr - 1, &canvas_x, &canvas_y);
+	      put_sprite_overlay( _rp(o), mysprite, _mleft(o) + canvas_x, _mtop(o) + canvas_y);
+  	    }
+          }
 	}
 
-	TimeDelay( UNIT_VBLANK, 1,0);
-
-	/* Restore the map */
-	x = map_canvas_adjust_x(data->mushroom_x0-1) * get_normal_tile_width();
-	y = map_canvas_adjust_y(data->mushroom_y0-1) * get_normal_tile_height();
-
-	w = get_normal_tile_width() * 3;
-	h = get_normal_tile_height() * 3;
-
-	if (x<0) {
-	  w +=x;
-	  x = 0;
-	}
-
-	if (y<0) {
-	  h +=y;
-	  y = 0;
-	}
-
-	BltBitMapRastPort(data->map_bitmap, x, y,
-			  _rp(o), _mleft(o) + x, _mtop(o) + y,
-			  w, h, 0xc0);
-
 	MUI_RemoveClipping(muiRenderInfo(o), cliphandle);
+	TimeDelay( UNIT_VBLANK, 1,0);
+
+	BltBitMapRastPort(data->map_bitmap, 0, 0,
+			  _rp(o), _mleft(o), _mtop(o),
+			  _mwidth(o), _mheight(o), 0xc0);
+
       	return 0;
       }
 
+// implement me
+#ifdef DISABLED
       if (data->update == 7)
       {
       	/* Draw City Workers */
 	APTR cliphandle = MUI_AddClipping(muiRenderInfo(o), _mleft(o), _mtop(o), _mwidth(o), _mheight(o));
-	int color;
+	int color, is_real;
 	int x,y;
 	struct city *pcity = data->worker_pcity;
 
@@ -1202,8 +1171,13 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	  if (data->worker_colornum == 3) data->worker_colornum = 0;
 	}
 
-	x = map_canvas_adjust_x(pcity->x);
-	y = map_canvas_adjust_y(pcity->y);
+	/* This used to use map_canvas_adjust_[xy], but that system is
+	   no good.  I've adjusted it to use a manual system instead. */
+	is_real = normalize_map_pos(&pcity->x, &pcity->y);
+	assert(is_real);
+	if (pcity->x < map_view_x0) {
+	  pcity->x += map.xsize;
+	}
 
 	SetAPen(_rp(o),color);
 
@@ -1211,7 +1185,7 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	{
 	  enum city_tile_type t = get_worker_city(pcity, i, j);
 
-	  if (!(i==2 && j==2))
+	  if (!is_city_center(i, j))
 	  {
 	    int pix_x = (x + i - 2) * get_normal_tile_width() + _mleft(o);
 	    int pix_y = (y + j - 2) * get_normal_tile_height() + _mtop(o);
@@ -1264,15 +1238,16 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	    MUI_RemoveClipping(muiRenderInfo(o), cliphandle);
 	    return 0;
 	  } else {
-	    really_draw_segment(data->map_layer->rp, 0, 0, src_x, src_y, dir, 0);
-	    really_draw_segment(_rp(o), _mleft(o), _mtop(o), src_x, src_y, dir, 0);
+	    really_draw_segment(data->map_layer->rp, 0, 0, src_x, src_y, dir,
+				FALSE);
+	    really_draw_segment(_rp(o), _mleft(o), _mtop(o), src_x, src_y,
+				dir, FALSE);
 	  }
 	} else {
-	  int dest_x, dest_y;
-	  dest_x = src_x + DIR_DX[dir];
-	  dest_y = src_y + DIR_DY[dir];
-	  assert(is_real_tile(dest_x, dest_y));
-	  normalize_map_pos(&dest_x, &dest_y);
+	  int dest_x, dest_y, is_real;
+
+	  is_real = MAPSTEP(dest_x, dest_y, src_x, src_y, dir);
+	  assert(is_real);
 
 	  /* A previous line already marks the place */
 	  if (get_drawn(src_x, src_y, dir)) {
@@ -1283,8 +1258,10 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
               put_line(_rp(o), _mleft(o),_mtop(o),src_x, src_y, dir);
 	    }
 	    if (tile_visible_mapcanvas(dest_x, dest_y)) {
-	      put_line(data->map_layer->rp, 0,0,dest_x, dest_y, 7-dir);
-	      put_line(_rp(o), _mleft(o),_mtop(o),dest_x, dest_y, 7-dir);
+	      put_line(data->map_layer->rp, 0, 0,
+		       dest_x, dest_y, DIR_REVERSE(dir));
+	      put_line(_rp(o), _mleft(o), _mtop(o),
+		       dest_x, dest_y, DIR_REVERSE(dir));
 	    }
 	    increment_drawn(src_x, src_y, dir);
 	  }
@@ -1302,14 +1279,12 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
       	int dir = data->segment_dir;
 	APTR cliphandle = MUI_AddClipping(muiRenderInfo(o), _mleft(o), _mtop(o), _mwidth(o), _mheight(o));
 
+	int dest_x, dest_y, is_real;
+
+	is_real = MAPSTEP(dest_x, dest_y, src_x, src_y, dir);
+	assert(is_real);
+
 	if (is_isometric) {
-	  int dest_x, dest_y;
-
-	  dest_x = src_x + DIR_DX[dir];
-	  dest_y = src_y + DIR_DY[dir];
-	  assert(is_real_tile(dest_x, dest_y));
-	  normalize_map_pos(&dest_x, &dest_y);
-
 	  assert(get_drawn(src_x, src_y, dir));
 	  decrement_drawn(src_x, src_y, dir);
 
@@ -1318,11 +1293,9 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	    update_map_canvas(MIN(src_x, dest_x), MIN(src_y, dest_y),
 			src_x == dest_x ? 1 : 2,
 			src_y == dest_y ? 1 : 2,
-			1);
+			TRUE);
 	  }
 	} else {
-	  int dest_x = src_x + DIR_DX[dir];
-	  int dest_y = src_y + DIR_DY[dir];
 	  int drawn = get_drawn(src_x, src_y, dir);
 
 	  assert(drawn > 0);
@@ -1331,24 +1304,25 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	    decrement_drawn(src_x, src_y, dir);
 	  } else {
 	    decrement_drawn(src_x, src_y, dir);
-	    refresh_tile_mapcanvas(src_x, src_y, 1); /* !! */
-	    assert(is_real_tile(dest_x, dest_y));
-	    normalize_map_pos(&dest_x, &dest_y);
-	    refresh_tile_mapcanvas(dest_x, dest_y, 1); /* !! */
+	    refresh_tile_mapcanvas(src_x, src_y, TRUE); /* !! */
+	    refresh_tile_mapcanvas(dest_x, dest_y, TRUE); /* !! */
 	    if (NORMAL_TILE_WIDTH%2 == 0 || NORMAL_TILE_HEIGHT%2 == 0) {
-	      if (dir == 2) { /* Since the tile doesn't have a middle we draw an extra pixel
-				 on the adjacent tile when drawing in this direction. */
+	      int is_real;
+
+	      if (dir == DIR8_NORTHEAST) {
+		/* Since the tile doesn't have a middle we draw an extra pixel
+		   on the adjacent tile when drawing in this direction. */
 		dest_x = src_x + 1;
 		dest_y = src_y;
-		assert(is_real_tile(dest_x, dest_y));
-		normalize_map_pos(&dest_x, &dest_y);
-		refresh_tile_mapcanvas(dest_x, dest_y, 1); /* !! */
-	      } else if (dir == 5) { /* the same */
+		is_real = normalize_map_pos(&dest_x, &dest_y);
+		assert(is_real);
+		refresh_tile_mapcanvas(dest_x, dest_y, TRUE);	/* !! */
+	      } else if (dir == DIR8_SOUTHWEST) {	/* the same */
 		dest_x = src_x;
 		dest_y = src_y + 1;
-		assert(is_real_tile(dest_x, dest_y));
-		normalize_map_pos(&dest_x, &dest_y);
-		refresh_tile_mapcanvas(dest_x, dest_y, 1); /* !! */
+		is_real = normalize_map_pos(&dest_x, &dest_y);
+		assert(is_real);
+		refresh_tile_mapcanvas(dest_x, dest_y, TRUE);	/* !! */
 	      }
 	    }
 	  }
@@ -1434,16 +1408,22 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	      /* A diagonal scrolling has happened */
 	      int newwidth = xget(o, MUIA_Map_HorizVisible);
 	      int newheight = xget(o, MUIA_Map_VertVisible);
-	      int x_itr,y_itr,map_x,map_y,canvas_x,canvas_y;
+	      int map_x,map_y;
 
 	      /* Draw the upper or lower complete free horiz space */
-	      for (y_itr = y; y_itr < y + height; y_itr++) {
-		for (x_itr = data->horiz_first; x_itr < x + newwidth; x_itr++) {
-		  map_x = map_adjust_x(x_itr);
-		  map_y = y_itr;
- 
-		  get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y);
-		  put_tile(data->map_layer->rp, map_x,map_y,canvas_x,canvas_y,0);
+	      for (map_y = y; map_y < y + height; map_y++) {
+		for (map_x = data->horiz_first; map_x < x + newwidth;
+		     map_x++) {
+		  int canvas_x, canvas_y;
+
+		  /*
+		   * We don't normalize until later because we want to
+		   * draw black tiles for unreal positions.
+		   */
+		  if (get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y)) {
+		    put_tile(data->map_layer->rp, map_x, map_y, canvas_x,
+			     canvas_y, 0);
+		  }
 		}
 	      }
 
@@ -1528,16 +1508,12 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	    int x1 = x_itr;
 	    int y1 = y_itr;
 	    if (normalize_map_pos(&x1, &y1)) {
-	      int dir;
-	      for (dir = 0; dir < 8; dir++) {
+	      adjc_dir_iterate(x1, y1, x2, y2, dir) {
 		if (get_drawn(x1, y1, dir)) {
-		  int x2 = x1 + DIR_DX[dir];
-	          int y2 = y1 + DIR_DY[dir];
-	          if (normalize_map_pos(&x2, &y2)) {
-		    really_draw_segment(data->map_layer->rp, 0, 0, x1, y1, dir, 1);
-	          }
-	        }
-	      }
+		  really_draw_segment(data->map_layer->rp, 0, 0, x1, y1,
+				      dir, TRUE);
+		}
+	      } adjc_dir_iterate_end;
 	    }
           }
         }
@@ -1585,28 +1561,31 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	}
       } else
       {
-	int x_itr, y_itr;
-	int canvas_x, canvas_y;
+        int map_x, map_y;
 
-	for (y_itr=y; y_itr<y+height; y_itr++) {
-	  for (x_itr=x; x_itr<x+width; x_itr++) {
-	    int map_x = map_adjust_x(x_itr);
-	    int map_y = y_itr; /* not adjusted;, we want to draw black tiles */
+	for (map_y = y; map_y < y + height; map_y++) {
+	  for (map_x = x; map_x < x + width; map_x++) {
+	    int canvas_x, canvas_y;
 
-	    get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y);
-	    if (tile_visible_mapcanvas(map_x, map_y)) {
-	      put_tile(data->map_layer->rp,map_x, map_y, canvas_x, canvas_y, 0);
+	    /*
+	     * We don't normalize until later because we want to draw
+	     * black tiles for unreal positions.
+	     */
+	    if (get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y)) {
+	      put_tile(data->map_layer->rp, map_x, map_y, canvas_x,
+		       canvas_y, 0);
 	    }
 	  }
 	}
 
         if (blit_all) {
           BltBitMapRastPort(data->map_bitmap,0,0,_rp(o),_mleft(o),_mtop(o),_mwidth(o),_mheight(o),0xc0);
-	  Map_Priv_ShowCityDescriptions(o, data);
+//	  Map_Priv_ShowCityDescriptions(o, data);
         } else
 	if (write_to_screen) {
 	  LONG pix_width = width * get_normal_tile_width();
 	  LONG pix_height = height * get_normal_tile_height();
+	  int canvas_x, canvas_y;
 
 	  get_canvas_xy(x, y, &canvas_x, &canvas_y);
 
@@ -1628,6 +1607,9 @@ static ULONG Map_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg)
 	  }
 	}
       }
+
+      if  (!(msg->flags & MADF_DRAWUPDATE) || blit_all)
+	Map_Priv_ShowCityDescriptions(o, data);
     }
 
     data->update = 0;
@@ -1833,15 +1815,15 @@ static ULONG Map_ContextMenuBuild(struct IClass * cl, Object * o, struct MUIP_Co
 		  Map_InsertCommand(&list, _("Sentry"), PACK_USERDATA(punit, MENU_ORDER_SENTRY));
 		if (can_unit_do_activity(punit, ACTIVITY_PILLAGE))
 		  Map_InsertCommand(&list, _("Pillage"), PACK_USERDATA(punit, MENU_ORDER_PILLAGE));
-		if (can_unit_do_auto(punit) && unit_flag(punit->type, F_SETTLERS))
+		if (can_unit_do_auto(punit) && unit_flag(punit, F_SETTLERS))
 		  Map_InsertCommand(&list, _("Auto Settler"), PACK_USERDATA(punit, MENU_ORDER_AUTO_SETTLER));
-		if (can_unit_do_auto(punit) && !unit_flag(punit->type, F_SETTLERS))
+		if (can_unit_do_auto(punit) && !unit_flag(punit, F_SETTLERS))
 		  Map_InsertCommand(&list, _("Auto Attack"), PACK_USERDATA(punit, MENU_ORDER_AUTO_ATTACK));
 		if (can_unit_do_activity(punit, ACTIVITY_EXPLORE))
 		  Map_InsertCommand(&list, _("Auto Explore"), PACK_USERDATA(punit, MENU_ORDER_AUTO_EXPLORE));
 		if (can_unit_paradrop(punit))
 		  Map_InsertCommand(&list, _("Paradrop"), PACK_USERDATA(punit, MENU_ORDER_POLLUTION));
-		if (unit_flag(punit->type, F_NUCLEAR))
+		if (unit_flag(punit, F_NUCLEAR))
 		  Map_InsertCommand(&list, _("Explode Nuclear"), PACK_USERDATA(punit, MENU_ORDER_NUKE));
 		if (get_transporter_capacity(punit) > 0)
 		  Map_InsertCommand(&list, _("Unload"), PACK_USERDATA(punit, MENU_ORDER_UNLOAD));
@@ -1853,7 +1835,7 @@ static ULONG Map_ContextMenuBuild(struct IClass * cl, Object * o, struct MUIP_Co
 		if (can_unit_do_activity(punit, ACTIVITY_IRRIGATE))
 		{
 		  static char irrtext[64];
-		  if ((map_get_tile(punit->x, punit->y)->special & S_IRRIGATION) &&
+		  if (map_has_special(punit->x, punit->y, S_IRRIGATION) &&
 		      player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND))
 		  {
 		    sz_strlcpy(irrtext, _("Build Farmland"));
@@ -2152,59 +2134,6 @@ DISPATCHERPROTO(Map_Dispatcher)
  CityMap Custom Class
 *****************************************************************/
 
-/**************************************************************************
-...
-**************************************************************************/
-static void city_get_canvas_xy(int map_x, int map_y, int *canvas_x, int *canvas_y)
-{
-  if (is_isometric) {
-    int diff_xy;
-
-    /* The line at y=0 isometric has constant x+y=1(tiles) */
-    diff_xy = (map_x + map_y) - (1);
-    *canvas_y = diff_xy/2 * NORMAL_TILE_HEIGHT + (diff_xy%2) * (NORMAL_TILE_HEIGHT/2);
-
-    /* The line at x=0 isometric has constant x-y=-3(tiles) */
-    diff_xy = map_x - map_y;
-    *canvas_x = (diff_xy + 3) * NORMAL_TILE_WIDTH/2;
-  } else {
-    *canvas_x = map_x * NORMAL_TILE_WIDTH;
-    *canvas_y = map_y * NORMAL_TILE_HEIGHT;
-  }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-static void city_get_map_xy(int canvas_x, int canvas_y, int *map_x, int *map_y)
-{
-  if (is_isometric) {
-    *map_x = -2;
-    *map_y = 2;
-
-    /* first find an equivalent position on the left side of the screen. */
-    *map_x += canvas_x/NORMAL_TILE_WIDTH;
-    *map_y -= canvas_x/NORMAL_TILE_WIDTH;
-    canvas_x %= NORMAL_TILE_WIDTH;
-
-    /* Then move op to the top corner. */
-    *map_x += canvas_y/NORMAL_TILE_HEIGHT;
-    *map_y += canvas_y/NORMAL_TILE_HEIGHT;
-    canvas_y %= NORMAL_TILE_HEIGHT;
-
-    assert(NORMAL_TILE_WIDTH == 2*NORMAL_TILE_HEIGHT);
-    canvas_y *= 2; /* now we have a square. */
-    if (canvas_x + canvas_y > NORMAL_TILE_WIDTH/2) (*map_x)++;
-    if (canvas_x + canvas_y > 3 * NORMAL_TILE_WIDTH/2) (*map_x)++;
-    if (canvas_x - canvas_y > NORMAL_TILE_WIDTH/2)  (*map_y)--;
-    if (canvas_y - canvas_x > NORMAL_TILE_WIDTH/2)  (*map_y)++;
-  } else {
-    *map_x = canvas_x/NORMAL_TILE_WIDTH;
-    *map_y = canvas_y/NORMAL_TILE_HEIGHT;
-  }
-}
-
-
 struct MUI_CustomClass *CL_CityMap;
 
 Object *MakeCityMap(struct city *pcity)
@@ -2354,23 +2283,19 @@ static ULONG CityMap_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg
          that overlaps another fogged tile, as on the main map, as no tiles in
          the city radius can be fogged. */
 
-      city_map_iterate(x, y) {
-      	int map_x = pcity->x + x - 2;
-      	int map_y = pcity->y + y - 2;
-	if (normalize_map_pos(&map_x, &map_y) && tile_is_known(map_x, map_y)) {
+      city_map_checked_iterate(pcity->x, pcity->y, x, y, map_x, map_y) {
+	if (tile_get_known(map_x, map_y)) {
 	  int canvas_x, canvas_y;
-	  city_get_canvas_xy(x, y, &canvas_x, &canvas_y);
+	  city_pos_to_canvas_pos(x, y, &canvas_x, &canvas_y);
 	  put_one_tile_full(_rp(o), map_x, map_y, canvas_x + _mleft(o), canvas_y + _mtop(o), 1);
 	}
-      } city_map_iterate_end;
+      } city_map_checked_iterate_end;
 
       /* We have to put the output afterwards or it will be covered. */
-      city_map_iterate(x, y) {
-	int map_x = pcity->x + x - 2;
-	int map_y = pcity->y + y - 2;
-	if (normalize_map_pos(&map_x, &map_y) && tile_is_known(map_x, map_y)) {
+      city_map_checked_iterate(pcity->x, pcity->y, x, y, map_x, map_y) {
+	if (tile_get_known(map_x, map_y)) {
 	  int canvas_x, canvas_y;
-	  city_get_canvas_xy(x, y, &canvas_x, &canvas_y);
+	  city_pos_to_canvas_pos(x, y, &canvas_x, &canvas_y);
 	  if (pcity->city_map[x][y]==C_TILE_WORKER) {
 	    put_city_output_tile(_rp(o),
 			     city_get_food_tile(x, y, pcity),
@@ -2379,19 +2304,17 @@ static ULONG CityMap_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg
 			     _mleft(o) + canvas_x, _mtop(o) + canvas_y,0,0);
           }
         }
-      } city_map_iterate_end;
+      } city_map_checked_iterate_end;
 
       /* This sometimes will draw one of the lines on top of a city or
          unit pixmap. This should maybe be moved to put_one_tile_pixmap()
          to fix this, but maybe it wouldn't be a good idea because the
          lines would get obscured. */
-      city_map_iterate(x, y) {
-	int map_x = pcity->x + x - 2;
-	int map_y = pcity->y + y - 2;
-	if (normalize_map_pos(&map_x, &map_y) && tile_is_known(map_x, map_y))
+      city_map_checked_iterate(pcity->x, pcity->y, x, y, map_x, map_y) {
+	if (tile_get_known(map_x, map_y))
 	{
 	  int canvas_x, canvas_y;
-	  city_get_canvas_xy(x, y, &canvas_x, &canvas_y);
+	  city_pos_to_canvas_pos(x, y, &canvas_x, &canvas_y);
 
           canvas_x += _mleft(o);
           canvas_y += _mtop(o);
@@ -2411,7 +2334,7 @@ static ULONG CityMap_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg
             Draw(rp,canvas_x, canvas_y+NORMAL_TILE_HEIGHT/2);
           }
         }
-      } city_map_iterate_end;
+      } city_map_checked_iterate_end;
     } else
     {
     	int x,y;
@@ -2419,53 +2342,42 @@ static ULONG CityMap_Draw(struct IClass * cl, Object * o, struct MUIP_Draw * msg
       {
         for (x = 0; x < CITY_MAP_SIZE; x++)
         {
-	  LONG tilex = pcity->x + x - CITY_MAP_SIZE / 2;
-	  LONG tiley = pcity->y + y - CITY_MAP_SIZE / 2;
-
 	  LONG x1 = _mleft(o) + x * get_normal_tile_width();
 	  LONG y1 = _mtop(o) + y * get_normal_tile_height();
 	  LONG x2 = x1 + get_normal_tile_width() - 1;
 	  LONG y2 = y1 + get_normal_tile_height() - 1;
 
-	  if (!(x == 0 && y == 0) && !(x == 0 && y == CITY_MAP_SIZE - 1) &&
-	      !(x == CITY_MAP_SIZE - 1 && y == 0) &&
-	      !(x == CITY_MAP_SIZE - 1 && y == CITY_MAP_SIZE - 1) &&
-	       (tiley >= 0 && tiley < map.ysize))
-	  {
-	    if (tile_is_known(tilex,tiley))
-	    {
+	  if (is_valid_city_coords(x, y)) {
+	    int tilex, tiley;
+
+	    if (city_map_to_map(&tilex, &tiley, pcity, x, y)
+		&& tile_get_known(tilex, tiley)) {
 	      put_tile(_rp(o), tilex, tiley, x1, y1, 1);
 
-	      if (pcity->city_map[x][y] == C_TILE_WORKER)
-	      {
-	        put_city_output_tile(_rp(o),
-	                             city_get_food_tile(x, y, pcity),
-	                             city_get_shields_tile(x, y, pcity),
-	                             city_get_trade_tile(x, y, pcity),
+	      if (pcity->city_map[x][y] == C_TILE_WORKER) {
+		put_city_output_tile(_rp(o),
+				     city_get_food_tile(x, y, pcity),
+				     city_get_shields_tile(x, y, pcity),
+				     city_get_trade_tile(x, y, pcity),
 				     _mleft(o), _mtop(o), x, y);
-
-	      } else
-	      {
-	        if (pcity->city_map[x][y] == C_TILE_UNAVAILABLE)
-	        {
-	          SetAPen(rp, data->red_color);
-	          Move(rp, x1, y1);
-	          Draw(rp, x2, y1);
-	          Draw(rp, x2, y2);
-	          Draw(rp, x1, y2);
-	          Draw(rp, x1, y1 + 1);
-	        }
+	      } else {
+		if (pcity->city_map[x][y] == C_TILE_UNAVAILABLE) {
+		  SetAPen(rp, data->red_color);
+		  Move(rp, x1, y1);
+		  Draw(rp, x2, y1);
+		  Draw(rp, x2, y2);
+		  Draw(rp, x1, y2);
+		  Draw(rp, x1, y1 + 1);
+		}
 	      }
-	    } else
-	    {
+	    } else {
 	      SetAPen(rp, data->black_color);
 	      RectFill(rp, x1, y1, x2, y2);
 	    }
-	  } else
-	  {
-	    if (msg->flags & MADF_DRAWOBJECT)
-	    {
-	      DoMethod(o,MUIM_DrawBackground,x1,y1,x2-x1+1,y2-y1+1,x1,y1,0);
+	  } else {
+	    if (msg->flags & MADF_DRAWOBJECT) {
+	      DoMethod(o, MUIM_DrawBackground, x1, y1, x2 - x1 + 1,
+		       y2 - y1 + 1, x1, y1, 0);
 	    }
 	  }
 	}
@@ -2489,7 +2401,7 @@ static ULONG CityMap_HandleInput(struct IClass * cl, Object * o, struct MUIP_Han
 	if (_isinobject(msg->imsg->MouseX, msg->imsg->MouseY))
 	{
 	  int x,y;
-	  city_get_map_xy(msg->imsg->MouseX - _mleft(o), msg->imsg->MouseY - _mtop(o), &x, &y);
+	  canvas_pos_to_city_pos(msg->imsg->MouseX - _mleft(o), msg->imsg->MouseY - _mtop(o), &x, &y);
 	  data->click.x = x;
 	  data->click.y = y;
 	  set(o, MUIA_CityMap_Click, &data->click);
