@@ -17,7 +17,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <assert.h>
 
 #include <libraries/mui.h>
@@ -41,6 +40,7 @@
 #include "helpdlg.h"
 #include "gui_main.h"
 #include "repodlgs.h"
+#include "repodlgs_common.h"
 
 /* Amiga Client stuff */
 
@@ -61,10 +61,8 @@ static Object *science_researched_group;
 static STRPTR *help_goal_entries;
 static STRPTR *help_research_entries;
 
-int delay_report_update = 0;
-
 static void create_trade_report_dialog(void);
-void create_activeunits_report_dialog(int make_modal);
+void create_activeunits_report_dialog(bool make_modal);
 
 /******************************************************************
  GUI independend
@@ -86,30 +84,12 @@ void request_player_research(int to)
   send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RESEARCH);
 }
 
-
-/******************************************************************
- Turn off updating of reports
-*******************************************************************/
-void report_update_delay_on(void)
-{
-  delay_report_update = 1;
-}
-
-/******************************************************************
- Turn on updating of reports
-*******************************************************************/
-
-void report_update_delay_off(void)
-{
-  delay_report_update = 0;
-}
-
 /******************************************************************
 ...
 *******************************************************************/
 void update_report_dialogs(void)
 {
-  if (delay_report_update)
+  if (is_report_dialogs_frozen())
     return;
   activeunits_report_dialog_update();
   economy_report_dialog_update();
@@ -162,7 +142,9 @@ static void science_goal(ULONG * newgoal)
     else
     {
       request_player_tech_goal(to);
-      DoMethod(science_steps_text, MUIM_SetAsString, MUIA_Text_Contents, _("(%ld steps)"), tech_goal_turns(game.player_ptr, to));
+      DoMethod(science_steps_text, MUIM_SetAsString, MUIA_Text_Contents,
+	       _("(%ld steps)"),
+	       num_unknown_techs_for_goal(game.player_ptr, to));
     }
   }
 }
@@ -206,7 +188,7 @@ static void science_researched(ULONG * tech)
 /****************************************************************
 ...
 ****************************************************************/
-void popup_science_dialog(int make_modal)
+void popup_science_dialog(bool make_modal)
 {
   static STRPTR def_entries[2];
   STRPTR *goal_entries = def_entries;
@@ -260,7 +242,7 @@ void popup_science_dialog(int make_modal)
   {
     if (get_invention(game.player_ptr, i) != TECH_KNOWN &&
 	advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST &&
-	tech_goal_turns(game.player_ptr, i) < 11)
+	num_unknown_techs_for_goal(game.player_ptr, i) < 11)
       j++;
   }
   if (game.player_ptr->ai.tech_goal == A_NONE)
@@ -279,7 +261,7 @@ void popup_science_dialog(int make_modal)
       {
 	if (get_invention(game.player_ptr, i) != TECH_KNOWN &&
 	    advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST &&
-	    tech_goal_turns(game.player_ptr, i) < 11)
+	    num_unknown_techs_for_goal(game.player_ptr, i) < 11)
 	{
 	  if (i == game.player_ptr->ai.tech_goal)
 	    science_goal_active = j;
@@ -334,7 +316,15 @@ void popup_science_dialog(int make_modal)
   if (science_wnd)
   {
     char *report_title = get_report_title(_("Science Advisor"));
-    settextf(science_title_text, _("%s\n(%d turns/advance)"), report_title,tech_turns_to_advance(game.player_ptr));
+    int turns_to_advance = tech_turns_to_advance(game.player_ptr);
+
+    if (turns_to_advance == FC_INFINITY) {
+      settextf(science_title_text, _("%s\n(no research)"), report_title);
+    } else {
+      settextf(science_title_text,
+	       PL_("%s\n(%d turn/advance)", "%s\n(%d turns/advance)",
+		   turns_to_advance), report_title, turns_to_advance);
+    }
     free(report_title);
 
     DoMethod(science_cycle_group, MUIM_Group_InitChange);
@@ -371,12 +361,17 @@ void popup_science_dialog(int make_modal)
 	DoMethod(science_research_popup, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime, app, 4, MUIM_CallHook, &civstandard_hook, science_research, MUIV_TriggerValue);
 	DoMethod(science_goal_popup, MUIM_Notify, MUIA_Cycle_Active, MUIV_EveryTime, app, 4, MUIM_CallHook, &civstandard_hook, science_goal, MUIV_TriggerValue);
 
-	DoMethod(status_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld/%ld", game.player_ptr->research.researched, research_time(game.player_ptr));
+	DoMethod(status_text, MUIM_SetAsString, MUIA_Text_Contents,
+		 "%ld/%ld", game.player_ptr->research.bulbs_researched,
+		 total_bulbs_required(game.player_ptr));
       }
     }
     DoMethod(science_cycle_group, MUIM_Group_ExitChange);
 
-    DoMethod(science_steps_text, MUIM_SetAsString, MUIA_Text_Contents, _("(%ld steps)"), tech_goal_turns(game.player_ptr, game.player_ptr->ai.tech_goal));
+    DoMethod(science_steps_text, MUIM_SetAsString, MUIA_Text_Contents,
+	     _("(%ld steps)"), num_unknown_techs_for_goal(game.player_ptr,
+							  game.player_ptr->
+							  ai.tech_goal));
 
     DoMethod(science_researched_group, MUIM_Group_InitChange);
     DoMethod(science_researched_group, MUIM_AutoGroup_DisposeChilds);
@@ -424,13 +419,6 @@ void science_dialog_update(void)
  
 ****************************************************************/
 
-struct trade_imprv_entry
-{
-  int type;
-  int count;
-  struct city *pcity;
-};
-
 static Object *trade_wnd;
 static Object *trade_title_text;
 static Object *trade_imprv_listview;
@@ -445,9 +433,9 @@ static Object *trade_sellall_button;
 /****************************************************************
  Constructor of a new entry in the trade listview
 *****************************************************************/
-HOOKPROTONHNO(trade_imprv_construct, struct trade_imprv_entry *, struct trade_imprv_entry *entry)
+HOOKPROTONHNO(trade_imprv_construct, struct improvement_entry *, struct improvement_entry *entry)
 {
-  struct trade_imprv_entry *newentry = (struct trade_imprv_entry *) AllocVec(sizeof(*newentry), 0);
+  struct improvement_entry *newentry = (struct improvement_entry *) AllocVec(sizeof(*newentry), 0);
   if (newentry)
   {
     *newentry = *entry;
@@ -458,7 +446,7 @@ HOOKPROTONHNO(trade_imprv_construct, struct trade_imprv_entry *, struct trade_im
 /****************************************************************
  Destructor of a entry in the trades listview
 *****************************************************************/
-HOOKPROTONHNO(trade_imprv_destruct, void, struct trade_imprv_entry *entry)
+HOOKPROTONHNO(trade_imprv_destruct, void, struct improvement_entry *entry)
 {
   FreeVec(entry);
 }
@@ -466,7 +454,7 @@ HOOKPROTONHNO(trade_imprv_destruct, void, struct trade_imprv_entry *entry)
 /****************************************************************
  Display function for the trade listview
 *****************************************************************/
-HOOKPROTONH(trade_imprv_render, void, char **array, struct trade_imprv_entry *entry)
+HOOKPROTONH(trade_imprv_render, void, char **array, struct improvement_entry *entry)
 {
   if (entry)
   {
@@ -474,15 +462,11 @@ HOOKPROTONH(trade_imprv_render, void, char **array, struct trade_imprv_entry *en
     static char coststr[16];
     static char utotal[16];
 
-    struct city *pcity = entry->pcity;
-    int j = entry->type;
-    int cost = entry->count * improvement_upkeep(pcity, j);
-
     my_snprintf(count, sizeof(count), "%5d", entry->count);
-    my_snprintf(coststr, sizeof(coststr), "%5d", improvement_upkeep(pcity, j));
-    my_snprintf(utotal, sizeof(utotal), "%6d", cost);
+    my_snprintf(coststr, sizeof(coststr), "%5d", entry->cost);
+    my_snprintf(utotal, sizeof(utotal), "%6d", entry->total_cost);
 
-    *array++ = get_improvement_name(j);
+    *array++ = get_improvement_name(entry->type);
     *array++ = count;
     *array++ = coststr;
     *array = utotal;
@@ -499,7 +483,7 @@ HOOKPROTONH(trade_imprv_render, void, char **array, struct trade_imprv_entry *en
 /****************************************************************
 ...
 ****************************************************************/
-void popup_economy_report_dialog(int make_modal)
+void popup_economy_report_dialog(bool make_modal)
 {
   if (!trade_wnd)
     create_trade_report_dialog();
@@ -515,11 +499,11 @@ void popup_economy_report_dialog(int make_modal)
 *****************************************************************/
 static void trade_sell(int *data)
 {
-  struct trade_imprv_entry *entry;
+  struct improvement_entry *entry;
   DoMethod(trade_imprv_listview, MUIM_NList_GetEntry, MUIV_NList_GetEntry_Active, &entry);
   if (entry)
   {
-    int i = entry->type;
+    Impr_Type_id i = entry->type;
     int count = 0, gold = 0;
     char str[128];
 
@@ -534,8 +518,6 @@ static void trade_sell(int *data)
 
       packet.city_id = pcity->id;
       packet.build_id = i;
-      packet.name[0] = '\0';
-      packet.worklist.name[0] = '\0';
       send_packet_city_request(&aconnection, &packet, PACKET_CITY_SELL);
     }
     city_list_iterate_end
@@ -605,10 +587,10 @@ void economy_report_dialog_update(void)
 {
   int tax, total;
   char *report_title;
-  struct city *pcity;
-  struct trade_imprv_entry entry;
+  static struct improvement_entry entries[B_LAST];
+  int i, entries_used = 0;
 
-  if (delay_report_update)
+  if (is_report_dialogs_frozen())
     return;
 
   if (!trade_wnd)
@@ -622,42 +604,12 @@ void economy_report_dialog_update(void)
 
   set(trade_imprv_listview, MUIA_NList_Quiet, TRUE);
   DoMethod(trade_imprv_listview, MUIM_NList_Clear);
-  total = tax = 0;
 
-  if ((pcity = city_list_get(&game.player_ptr->cities, 0)))
-  {
-    int j;
+  get_economy_report_data(entries, &entries_used, &total, &tax);
 
-    for (j = 0; j < game.num_impr_types; j++)
-    {
-      if (!is_wonder(j))
-      {
-	int cost, count = 0;
-	city_list_iterate(game.player_ptr->cities, pcity)
-	  if (city_got_building(pcity, j))
-	  count++;
-	city_list_iterate_end;
-
-	if (!count)
-	  continue;
-	cost = count * improvement_upkeep(pcity, j);
-
-	entry.type = j;
-	entry.count = count;
-	entry.pcity = pcity;
-
-	DoMethod(trade_imprv_listview, MUIM_NList_InsertSingle, &entry, MUIV_NList_Insert_Bottom);
-
-	total += cost;
-      }
-    }
-
-
-    city_list_iterate(game.player_ptr->cities, pcity)
-      tax += pcity->tax_total;
-    if (!pcity->is_building_unit && pcity->currently_building == B_CAPITAL)
-      tax += pcity->shield_surplus;
-    city_list_iterate_end;
+  for (i = 0; i < entries_used; i++) {
+    DoMethod(trade_imprv_listview, MUIM_NList_InsertSingle, &entries[i],
+	     MUIV_NList_Insert_Bottom);
   }
 
   set(trade_imprv_listview, MUIA_NList_Quiet, FALSE);
@@ -684,7 +636,7 @@ static Object *actunit_upgrade_button;
 /****************************************************************
 ...
 ****************************************************************/
-void popup_activeunits_report_dialog(int make_modal)
+void popup_activeunits_report_dialog(bool make_modal)
 {
   if (!actunit_wnd)
     create_activeunits_report_dialog(make_modal);
@@ -829,7 +781,7 @@ static void actunit_upgrade(void)
 /****************************************************************
 ...
 *****************************************************************/
-void create_activeunits_report_dialog(int make_modal)
+void create_activeunits_report_dialog(bool make_modal)
 {
   if (actunit_wnd)
     return;
@@ -899,7 +851,7 @@ void activeunits_report_dialog_update(void)
 
   if (!actunit_wnd)
     return;
-  if (delay_report_update)
+  if (is_report_dialogs_frozen())
     return;
 
   report_title = get_report_title(_("Military Report"));
@@ -930,8 +882,7 @@ void activeunits_report_dialog_update(void)
   set(actunit_units_listview, MUIA_NList_Quiet, TRUE);
   DoMethod(actunit_units_listview, MUIM_NList_Clear);
 
-  for (i = 0; i < game.num_unit_types; i++)
-  {
+  unit_type_iterate(i) {
     if (unitarray[i].active_count)
     {
       entry.type = i;
@@ -942,7 +893,7 @@ void activeunits_report_dialog_update(void)
       entry.building_count = unitarray[i].building_count;
       DoMethod(actunit_units_listview, MUIM_NList_InsertSingle, &entry, MUIV_NList_Insert_Bottom);
     }
-  }
+  } unit_type_iterate_end;
 
   set(actunit_units_listview, MUIA_NList_Quiet, FALSE);
 }
