@@ -29,6 +29,7 @@
 #include "shared.h"		/* wordwrap_string() */
 #include "support.h"
 
+#include "climisc.h"
 #include "clinet.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -72,18 +73,20 @@ void chatline_key_send(Widget w)
 
  Now uses window's font size and width.  Assumes fixed-width font.  --jjm
 **************************************************************************/
-void append_output_window(char *astring)
+void real_append_output_window(const char *input_string)
 {
   static int m_width=0;
 
   Dimension windowwth;
   int maxlinelen;
   String theoutput;
-  char *newout, *rmcr;
-  char *input_string = astring;
+  char *newout, *rmcr, *astring = mystrdup(input_string);
 
   if (!m_width) {
-    XFontStruct *out_font;
+    /* Sometimes XtVaGetValues has garbage for the XtNfont; see PR#6452.
+     * In this case it's safest if the value is initialized to NULL. */
+    XFontStruct *out_font = NULL;
+
     XtVaGetValues(outputwindow_text, XtNfont, &out_font, NULL);
     if (out_font)
       m_width=XTextWidth(out_font, "M", 1);
@@ -99,7 +102,6 @@ void append_output_window(char *astring)
 	  m_width, (int)windowwth, maxlinelen);
 
   if (strlen(astring) > maxlinelen) {
-    astring = mystrdup(astring);
     wordwrap_string(astring, maxlinelen);
   }
   
@@ -119,9 +121,7 @@ void append_output_window(char *astring)
   XawTextEnableRedisplay(outputwindow_text);
   
   free(newout);
-  if (astring != input_string) {
-    free(astring);
-  }
+  free(astring);
 }
 
 /**************************************************************************
@@ -130,16 +130,11 @@ void append_output_window(char *astring)
  the are somewhat related and append_output_window is already here.  --dwp
 **************************************************************************/
 void log_output_window(void)
-{ 
+{
   String theoutput;
-  FILE *fp;
-  
-  append_output_window(_("Exporting output window to civgame.log ..."));
+
   XtVaGetValues(outputwindow_text, XtNstring, &theoutput, NULL);
-  fp = fopen("civgame.log", "w"); /* should allow choice of name? */
-  fprintf(fp, "%s", theoutput);
-  fclose(fp);
-  append_output_window(_("Export complete."));
+  write_chatline_content(theoutput);
 }
 
 /**************************************************************************
@@ -149,4 +144,3 @@ void clear_output_window(void)
 {
   XtVaSetValues(outputwindow_text, XtNstring, _("Cleared output window."), NULL);
 }
-
