@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <stdarg.h>
 #include <assert.h>
 
@@ -71,7 +70,7 @@ void config_ok_command_callback(Widget w, XtPointer client_data,
 
 
 /******************************************************************/
-void create_city_report_dialog(int make_modal);
+void create_city_report_dialog(bool make_modal);
 void city_close_callback(Widget w, XtPointer client_data, 
 			 XtPointer call_data);
 void city_center_callback(Widget w, XtPointer client_data, 
@@ -147,8 +146,8 @@ static char *get_city_table_header(void)
 	cat_snprintf(text, sizeof(text), "%*s", spec->space, " ");
 
       cat_snprintf(text, sizeof(text), "%*s", spec->width,
-		   (j ? (spec->title2 ? _(spec->title2) : "")
-		    : (spec->title1 ? _(spec->title1) : "")));
+		   (j ? (spec->title2 ? spec->title2 : "")
+		    : (spec->title1 ? spec->title1 : "")));
     }
     if (j==0) sz_strlcat(text, "\n");
   }
@@ -164,7 +163,7 @@ static char *get_city_table_header(void)
 /****************************************************************
 ...
 ****************************************************************/
-void popup_city_report_dialog(int make_modal)
+void popup_city_report_dialog(bool make_modal)
 {
   if(!city_dialog_shell) {
       Position x, y;
@@ -195,7 +194,7 @@ void popup_city_report_dialog(int make_modal)
 /****************************************************************
 ...
 *****************************************************************/
-void create_city_report_dialog(int make_modal)
+void create_city_report_dialog(bool make_modal)
 {
   Widget close_command;
   char *report_title;
@@ -304,13 +303,13 @@ void city_list_callback(Widget w, XtPointer client_data,
 {
   XawListReturnStruct *ret=XawListShowCurrent(city_list);
   struct city *pcity;
-  int turns;
 
   if(ret->list_index!=XAW_LIST_NONE && 
      (pcity=cities_in_list[ret->list_index])) {
-    int flag;
+    cid cids[U_LAST + B_LAST];
+    struct item items[U_LAST + B_LAST];
+    int cids_used = 0;
     size_t i;
-    char buf[512];
 
     XtSetSensitive(city_change_command, TRUE);
     XtSetSensitive(city_center_command, TRUE);
@@ -324,44 +323,32 @@ void city_list_callback(Widget w, XtPointer client_data,
 				        simpleMenuWidgetClass, 
 				        city_change_command,
 				        NULL);
-    flag = 0;
-    for(i=0; i<game.num_impr_types; i++)
-      if(can_build_improvement(pcity, i)) {
-	Widget entry;
-	if (i==B_CAPITAL) {
-	  my_snprintf(buf, sizeof(buf),
-		      "%s (XX)",
-		      get_impr_name_ex(pcity, i));
-	} else {
-	  turns = city_turns_to_build (pcity, i, FALSE);
-	  my_snprintf(buf, sizeof(buf),
-		      turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
-		      get_impr_name_ex(pcity, i),
-		      get_improvement_type(i)->build_cost,
-		      turns);
-	}
-	entry = XtVaCreateManagedWidget(buf, smeBSBObjectClass, city_popupmenu, NULL);
-	XtAddCallback(entry, XtNcallback, city_change_callback, (XtPointer) i);
-	flag=1;
-      }
 
-    for(i=0; i<game.num_unit_types; i++)
-      if(can_build_unit(pcity, i)) {
-	Widget entry;
-	turns = city_turns_to_build (pcity, i, TRUE);
-	my_snprintf(buf, sizeof(buf),
-		    turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
-		    get_unit_name(i),
-		    get_unit_type(i)->build_cost,
-		    turns);
-	entry = XtVaCreateManagedWidget(buf, smeBSBObjectClass, 
-					city_popupmenu, NULL);
-	XtAddCallback(entry, XtNcallback, city_change_callback, 
-		      (XtPointer) (i+B_LAST));
-	flag = 1;
+    impr_type_iterate(i) {
+      if (can_build_improvement(pcity, i)) {
+	cids[cids_used] = cid_encode(FALSE, i);
+	cids_used++;
       }
+    } impr_type_iterate_end;
 
-    if(!flag)
+    unit_type_iterate(i) {
+      if (can_build_unit(pcity, i)) {
+	cids[cids_used] = cid_encode(TRUE, i);
+	cids_used++;
+      }
+    } unit_type_iterate_end;
+
+    name_and_sort_items(cids, cids_used, items, TRUE, NULL);
+    
+    for (i = 0; i < cids_used; i++) {
+      Widget entry =
+	  XtVaCreateManagedWidget(items[i].descr, smeBSBObjectClass,
+				  city_popupmenu, NULL);
+      XtAddCallback(entry, XtNcallback, city_change_callback,
+		    INT_TO_XTPOINTER(items[i].cid));
+    }
+
+    if (cids_used == 0)
       XtSetSensitive(city_change_command, FALSE);
   } else {
     XtSetSensitive(city_change_command, FALSE);
@@ -385,21 +372,14 @@ void city_change_callback(Widget w, XtPointer client_data,
   if(ret->list_index!=XAW_LIST_NONE && 
      (pcity=cities_in_list[ret->list_index])) {
     struct packet_city_request packet;
-    int build_nr;
+    cid my_cid = (cid) XTPOINTER_TO_INT(client_data);
     Boolean unit;
+    int build_nr;
       
-    build_nr = (size_t) client_data;
-
-    if (build_nr >= B_LAST) {
-      build_nr -= B_LAST;
-      unit = TRUE;
-    } else {
-      unit = FALSE;
-    }
+    unit = cid_is_unit(my_cid);
+    build_nr = cid_id(my_cid);
 
     packet.city_id=pcity->id;
-    packet.name[0]='\0';
-    packet.worklist.name[0] = '\0';
     packet.build_id=build_nr;
     packet.is_build_id_unit_id=unit;
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
@@ -418,7 +398,7 @@ void city_buy_callback(Widget w, XtPointer client_data,
     struct city *pcity;
     if((pcity=cities_in_list[ret->list_index])) {
       int value;
-      char *name;
+      const char *name;
       char buf[512];
 
       value=city_buy_cost(pcity);    
@@ -431,8 +411,6 @@ void city_buy_callback(Widget w, XtPointer client_data,
 	{
 	  struct packet_city_request packet;
 	  packet.city_id=pcity->id;
-	  packet.name[0]='\0';
-	  packet.worklist.name[0] = '\0';
 	  send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 	}
       else
@@ -544,7 +522,7 @@ void city_config_callback(Widget w, XtPointer client_data,
 *****************************************************************/
 void city_report_dialog_update(void)
 {
-  if(delay_report_update) return;
+  if(is_report_dialogs_frozen()) return;
   if(city_dialog_shell) {
     int i=0, n;
     Dimension width;
@@ -627,7 +605,7 @@ void city_report_dialog_update_city(struct city *pcity)
 {
   int i;
 
-  if(delay_report_update) return;
+  if(is_report_dialogs_frozen()) return;
   if(!city_dialog_shell) return;
 
   for(i=0; cities_in_list[i]; i++)  {
@@ -638,8 +616,8 @@ void city_report_dialog_update_city(struct city *pcity)
       char new_city_line[MAX_LEN_CITY_TEXT];
 
       XtVaGetValues(city_list, XtNnumberStrings, &n, XtNlist, &list, NULL);
-      if(strncmp(pcity->name,list[i],
-		 MIN(strlen(pcity->name),REPORT_CITYNAME_ABBREV-1))) {
+      if (0 != strncmp(pcity->name, list[i],
+		       MIN(strlen(pcity->name), REPORT_CITYNAME_ABBREV - 1))) {
 	 break;
       }
       get_city_text(pcity, new_city_line, sizeof(new_city_line));
@@ -716,7 +694,7 @@ void create_city_report_config_dialog(void)
 					     config_form, NULL));
 
   for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {
-    my_snprintf(buf, sizeof(buf), "%-32s", _(spec->explanation));
+    my_snprintf(buf, sizeof(buf), "%-32s", spec->explanation);
     above = (i==1)?config_label:config_optlabel;
 
     config_optlabel = XtVaCreateManagedWidget("cityconfiglabel", 
@@ -759,14 +737,15 @@ void config_ok_command_callback(Widget w, XtPointer client_data,
 				XtPointer call_data)
 {
   struct city_report_spec *spec;
-  Boolean b;
   int i;
   
   XtDestroyWidget(config_shell);
 
   for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {
+    Boolean b;
+
     XtVaGetValues(config_toggle[i], XtNstate, &b, NULL);
-    spec->show = b;
+    spec->show = (bool) b;
   }
   config_shell=0;
   city_report_dialog_update();
@@ -790,15 +769,13 @@ struct chgall_data
     Widget cancel;
   } w;
 
-  int is_bldg_unit[U_LAST];
-  int is_bldg_impv[B_LAST];
   int fr_count;
   char *fr_list[U_LAST + B_LAST];
-  int fr_idents[U_LAST + B_LAST];
+  cid fr_cids[U_LAST + B_LAST];
 
   int to_count;
   char *to_list[U_LAST + B_LAST];
-  int to_idents[U_LAST + B_LAST];
+  cid to_cids[U_LAST + B_LAST];
 
   int fr_index;
   int to_index;
@@ -1030,12 +1007,21 @@ static void chgall_cancel_action (Widget w, XEvent *event,
 ...
 **************************************************************************/
 
-static void chgall_shell_destroy (Widget w, XtPointer client_data,
-				  XtPointer call_data)
+static void chgall_shell_destroy(Widget w, XtPointer client_data,
+				 XtPointer call_data)
 {
+  int i;
+
+  for (i = 0; i < chgall_state->fr_count; i++) {
+    free(chgall_state->fr_list[i]);
+  }
+  for (i = 0; i < chgall_state->to_count; i++) {
+    free(chgall_state->to_list[i]);
+  }
+
   chgall_state = NULL;
 
-  free (client_data);
+  free(client_data);
 }
 
 /**************************************************************************
@@ -1066,7 +1052,6 @@ static void chgall_change_command_callback (Widget w, XtPointer client_data,
 					    XtPointer call_data)
 {
   struct chgall_data *state = (struct chgall_data *)client_data;
-  char msgbuf[1024];
 
   chgall_update_widgets_state (state);
 
@@ -1075,17 +1060,8 @@ static void chgall_change_command_callback (Widget w, XtPointer client_data,
       return;
     }
 
-  my_snprintf (msgbuf, sizeof(msgbuf),
-	   _("Game: Changing production of every %s into %s."),
-	   state->fr_list[state->fr_index],
-	   state->to_list[state->to_index]);
-  append_output_window (msgbuf);
-
-  client_change_all
-    (
-     state->fr_idents[state->fr_index],
-     state->to_idents[state->to_index]
-    );
+  client_change_all(state->fr_cids[state->fr_index],
+		    state->to_cids[state->to_index]);
 
   XtDestroyWidget (state->w.shell);
 }
@@ -1094,69 +1070,32 @@ static void chgall_change_command_callback (Widget w, XtPointer client_data,
 ...
 **************************************************************************/
 
-static void chgall_refresh_command_callback (Widget w, XtPointer client_data,
-					     XtPointer call_data)
+static void chgall_refresh_command_callback(Widget w,
+					    XtPointer client_data,
+					    XtPointer call_data)
 {
-  struct chgall_data *state = (struct chgall_data *)client_data;
-  int i, n;
+  struct chgall_data *state = (struct chgall_data *) client_data;
+  cid cids[U_LAST + B_LAST];
+  struct item items[U_LAST + B_LAST];
+  int i;
 
-  memset (state->is_bldg_unit, 0, sizeof (state->is_bldg_unit));
-  memset (state->is_bldg_impv, 0, sizeof (state->is_bldg_impv));
-
-  city_list_iterate (game.player_ptr->cities, pcity) {
-    if (pcity->is_building_unit)
-      state->is_bldg_unit[pcity->currently_building] = TRUE;
-    else
-      state->is_bldg_impv[pcity->currently_building] = TRUE;
+  state->fr_count = collect_cids2(cids);
+  name_and_sort_items(cids, state->fr_count, items, FALSE, NULL);
+  for (i = 0; i < state->fr_count; i++) {
+    state->fr_list[i] = strdup(items[i].descr);
+    state->fr_cids[i] = items[i].cid;
   }
-  city_list_iterate_end;
+  XawListChange(state->w.fr, state->fr_list, state->fr_count, 0, FALSE);
 
-  n = 0;
-  for (i = 0; i < B_LAST; i++)
-    {
-      if (state->is_bldg_impv[i])
-	{
-	  state->fr_list[n] = (get_improvement_type (i))->name;
-	  state->fr_idents[n] = i;
-	  n++;
-	}
-    }
-  for (i = 0; i < U_LAST; i++)
-    {
-      if (state->is_bldg_unit[i])
-	{
-	  state->fr_list[n] = (get_unit_type (i))->name;
-	  state->fr_idents[n] = i + B_LAST;
-	  n++;
-	}
-    }
-  state->fr_count = n;
+  state->to_count = collect_cids3(cids);
+  name_and_sort_items(cids, state->to_count, items, TRUE, NULL);
+  for (i = 0; i < state->to_count; i++) {
+    state->to_list[i] = strdup(items[i].descr);
+    state->to_cids[i] = items[i].cid;
+  }
+  XawListChange(state->w.to, state->to_list, state->to_count, 0, FALSE);
 
-  n = 0;
-  for (i = 0; i < B_LAST; i++)
-    {
-      if (can_player_build_improvement (game.player_ptr, i))
-	{
-	  state->to_list[n] = (get_improvement_type (i))->name;
-	  state->to_idents[n] = i;
-	  n++;
-	}
-    }
-  for (i = 0; i < U_LAST; i++)
-    {
-      if (can_player_build_unit (game.player_ptr, i))
-	{
-	  state->to_list[n] = (get_unit_type (i))->name;
-	  state->to_idents[n] = i + B_LAST;
-	  n++;
-	}
-    }
-  state->to_count = n;
-
-  XawListChange (state->w.fr, state->fr_list, state->fr_count, 0, FALSE);
-  XawListChange (state->w.to, state->to_list, state->to_count, 0, FALSE);
-
-  chgall_update_widgets_state (state);
+  chgall_update_widgets_state(state);
 }
 
 /**************************************************************************
@@ -1181,8 +1120,10 @@ static void chgall_update_widgets_state (struct chgall_data *state)
   state->to_index = (XawListShowCurrent (state->w.to))->list_index;
 
   state->may_change =
-    ((state->fr_index != XAW_LIST_NONE) && (state->to_index != XAW_LIST_NONE) &&
-     (state->fr_idents[state->fr_index] != state->to_idents[state->to_index]));
+      ((state->fr_index != XAW_LIST_NONE)
+       && (state->to_index != XAW_LIST_NONE)
+       && (state->fr_cids[state->fr_index] !=
+	   state->to_cids[state->to_index]));
 
   XtSetSensitive (state->w.change, state->may_change);
 }
