@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <assert.h>
 
 #include <X11/Intrinsic.h>
@@ -246,8 +245,9 @@ static int fill_diplomacy_tech_menu(Widget popupmenu,
       Widget entry=
 	XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
 				popupmenu, NULL);
-      XtAddCallback(entry, XtNcallback, diplomacy_dialog_tech_callback, 
-		    (XtPointer)(plr0->player_no*10000+plr1->player_no*100+i)); 
+      XtAddCallback(entry, XtNcallback, diplomacy_dialog_tech_callback,
+		    INT_TO_XTPOINTER(plr0->player_no * 10000 +
+				     plr1->player_no * 100 + i));
       flag=1;
     }
   }
@@ -285,10 +285,10 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
     Widget entry=
       XtVaCreateManagedWidget(city_list_ptrs[j]->name, smeBSBObjectClass, 
 			      popupmenu, NULL);
-    XtAddCallback(entry, XtNcallback, diplomacy_dialog_city_callback, 
-		  (XtPointer)(city_list_ptrs[j]->id*1024
-			      + plr0->player_no*32
-			      + plr1->player_no));
+    XtAddCallback(entry, XtNcallback, diplomacy_dialog_city_callback,
+		  INT_TO_XTPOINTER(city_list_ptrs[j]->id * 1024
+				   + plr0->player_no * 32
+				   + plr1->player_no));
   }
   free(city_list_ptrs);
   return i;
@@ -483,7 +483,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 				commandWidgetClass, 
 				pdialog->dip_form0,
 				NULL));
-  if (plr0->gives_shared_vision & (1<<plr1->player_no))
+  if (gives_shared_vision(plr0, plr1))
     XtSetSensitive(pdialog->dip_vision_button0, FALSE);
 
   pdialog->dip_vision_button1 =
@@ -491,7 +491,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 				commandWidgetClass, 
 				pdialog->dip_form1,
 				NULL));
-  if (plr1->gives_shared_vision & (1<<plr0->player_no))
+  if (gives_shared_vision(plr1, plr0))
     XtSetSensitive(pdialog->dip_vision_button1, FALSE);
 
   XtAddCallback(pdialog->dip_vision_button0, XtNcallback, 
@@ -530,7 +530,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 	  get_ruler_title(plr0->government, plr0->is_male, plr0->nation),
 	  plr0->name,
 	  get_nation_name(plr1->nation),
-	  get_ruler_title(plr1->government, plr0->is_male, plr0->nation),
+	  get_ruler_title(plr1->government, plr1->is_male, plr1->nation),
 	  plr1->name);
   
   pheadlinem=create_centered_string(buf);
@@ -644,18 +644,14 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 **************************************************************************/
 void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
-  int i;
-  struct genlist_iterator myiter;
+  int i = 0;
   const int n = sizeof(pdialog->clauselist_strings[0]);
   
-  genlist_iterator_init(&myiter, &pdialog->treaty.clauses, 0);
-  
-  for(i=0; i<MAX_NUM_CLAUSES && ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
+  clause_list_iterate(pdialog->treaty.clauses, pclause) {
     client_diplomacy_clause_string(pdialog->clauselist_strings[i], n, pclause);
     pdialog->clauselist_strings_ptrs[i]=pdialog->clauselist_strings[i];
     i++;
-  }
+  } clause_list_iterate_end;
 
   pdialog->clauselist_strings_ptrs[i]=0;
   XawListChange(pdialog->dip_clauselist, pdialog->clauselist_strings_ptrs, 
@@ -731,15 +727,11 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
   ret=XawListShowCurrent(pdialog->dip_clauselist);
 
   if(ret->list_index!=XAW_LIST_NONE) {
-    int i;
-    struct genlist_iterator myiter;
-  
-    genlist_iterator_init(&myiter, &pdialog->treaty.clauses, 0);
+    int i = 0;
 
-    for(i=0; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter), i++) {
+    clause_list_iterate(pdialog->treaty.clauses, pclause) {
       if(i==ret->list_index) {
 	struct packet_diplomacy_info pa;
-	struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
 
 	pa.plrno0=pdialog->treaty.plr0->player_no;
 	pa.plrno1=pdialog->treaty.plr1->player_no;
@@ -750,7 +742,8 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
 				   &pa);
 	return;
       }
-    }
+      i++;
+    } clause_list_iterate_end;
   }
 }
 
@@ -890,6 +883,7 @@ void diplomacy_dialog_close_callback(Widget w, XtPointer client_data,
 
   pa.plrno0=game.player_idx;
   pa.plrno1=pdialog->treaty.plr1->player_no;
+  pa.plrno_from=pa.plrno0;
   send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CANCEL_MEETING, 
 			     &pa);
   
@@ -985,9 +979,8 @@ void diplodlg_key_gold(Widget w)
       pdialog->treaty.plr0 : pdialog->treaty.plr1;
     
     XtVaGetValues(w, XtNstring, &dp, NULL);
-    amount=atoi(dp);
-    
-    if(amount>=0 && amount<=pgiver->economic.gold) {
+    if (sscanf(dp, "%d", &amount) == 1 && amount >= 0
+	&& amount <= pgiver->economic.gold) {
       struct packet_diplomacy_info pa;
       pa.plrno0=pdialog->treaty.plr0->player_no;
       pa.plrno1=pdialog->treaty.plr1->player_no;
