@@ -17,7 +17,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
@@ -69,17 +68,17 @@ static struct Sprite *ctor_sprite_mask(Pixmap mypixmap, Pixmap mask,
 /***************************************************************************
 ...
 ***************************************************************************/
-int isometric_view_supported(void)
+bool isometric_view_supported(void)
 {
-  return 0;
+  return FALSE;
 }
 
 /***************************************************************************
 ...
 ***************************************************************************/
-int overhead_view_supported(void)
+bool overhead_view_supported(void)
 {
-  return 1;
+  return TRUE;
 }
 
 /***************************************************************************
@@ -93,7 +92,7 @@ void load_intro_gfx(void)
   char s[64];
   XColor face;
   int have_face;
-  char *motto = freeciv_motto();
+  const char *motto = freeciv_motto();
 
   /* metrics */
 
@@ -200,10 +199,13 @@ void load_cursors(void)
   XQueryColor(display, cmap, &black);
 
   /* goto */
-  pixmap = XCreateBitmapFromData(display, root_window, goto_cursor_bits,
-				 goto_cursor_width, goto_cursor_height);
-  mask   = XCreateBitmapFromData(display, root_window, goto_cursor_mask_bits,
-				 goto_cursor_mask_width, goto_cursor_mask_height);
+  pixmap =
+      XCreateBitmapFromData(display, root_window, (char *) goto_cursor_bits,
+			    goto_cursor_width, goto_cursor_height);
+  mask =
+      XCreateBitmapFromData(display, root_window,
+			    (char *) goto_cursor_mask_bits,
+			    goto_cursor_mask_width, goto_cursor_mask_height);
   goto_cursor = XCreatePixmapCursor(display, pixmap, mask,
 				    &white, &black,
 				    goto_cursor_x_hot, goto_cursor_y_hot);
@@ -211,10 +213,13 @@ void load_cursors(void)
   XFreePixmap(display, mask);
 
   /* drop */
-  pixmap = XCreateBitmapFromData(display, root_window, drop_cursor_bits,
-				 drop_cursor_width, drop_cursor_height);
-  mask   = XCreateBitmapFromData(display, root_window, drop_cursor_mask_bits,
-				 drop_cursor_mask_width, drop_cursor_mask_height);
+  pixmap =
+      XCreateBitmapFromData(display, root_window, (char *) drop_cursor_bits,
+			    drop_cursor_width, drop_cursor_height);
+  mask =
+      XCreateBitmapFromData(display, root_window,
+			    (char *) drop_cursor_mask_bits,
+			    drop_cursor_mask_width, drop_cursor_mask_height);
   drop_cursor = XCreatePixmapCursor(display, pixmap, mask,
 				    &white, &black,
 				    drop_cursor_x_hot, drop_cursor_y_hot);
@@ -222,10 +227,13 @@ void load_cursors(void)
   XFreePixmap(display, mask);
 
   /* nuke */
-  pixmap = XCreateBitmapFromData(display, root_window, nuke_cursor_bits,
-				 nuke_cursor_width, nuke_cursor_height);
-  mask   = XCreateBitmapFromData(display, root_window, nuke_cursor_mask_bits,
-				 nuke_cursor_mask_width, nuke_cursor_mask_height);
+  pixmap =
+      XCreateBitmapFromData(display, root_window, (char *) nuke_cursor_bits,
+			    nuke_cursor_width, nuke_cursor_height);
+  mask =
+      XCreateBitmapFromData(display, root_window,
+			    (char *) nuke_cursor_mask_bits,
+			    nuke_cursor_mask_width, nuke_cursor_mask_height);
   nuke_cursor = XCreatePixmapCursor(display, pixmap, mask,
 				    &white, &black,
 				    nuke_cursor_x_hot, nuke_cursor_y_hot);
@@ -233,10 +241,15 @@ void load_cursors(void)
   XFreePixmap(display, mask);
 
   /* patrol */
-  pixmap = XCreateBitmapFromData(display, root_window, patrol_cursor_bits,
-				 patrol_cursor_width, patrol_cursor_height);
-  mask   = XCreateBitmapFromData(display, root_window, patrol_cursor_mask_bits,
-				 patrol_cursor_mask_width, patrol_cursor_mask_height);
+  pixmap =
+      XCreateBitmapFromData(display, root_window,
+			    (char *) patrol_cursor_bits, patrol_cursor_width,
+			    patrol_cursor_height);
+  mask =
+      XCreateBitmapFromData(display, root_window,
+			    (char *) patrol_cursor_mask_bits,
+			    patrol_cursor_mask_width,
+			    patrol_cursor_mask_height);
   patrol_cursor = XCreatePixmapCursor(display, pixmap, mask,
 				      &white, &black,
 				      patrol_cursor_x_hot, patrol_cursor_y_hot);
@@ -332,7 +345,7 @@ again:
       goto again;
     }
     freelog(LOG_FATAL, _("Failed reading XPM file: %s"), filename);
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   mysprite=fc_malloc(sizeof(struct Sprite));
