@@ -35,7 +35,6 @@
 #include "timing.h"
 #include "unit.h"
 
-
 #include "civclient.h"
 #include "climisc.h"
 #include "colors.h"
@@ -50,28 +49,9 @@
 
 #include "mapview.h"
 
-/*
-The bottom row of the map was sometimes hidden.
-
-As of now the top left corner is always aligned with the tiles. This is what
-causes the problem in the first place. The ideal solution would be to align the
-window with the bottom left tiles if you tried to center the window on a tile
-closer than (screen_tiles_height/2 -1) to the south pole.
-
-But, for now, I just grepped for occurences where the ysize (or the values
-derived from it) were used, and those places that had relevance to drawing the
-map, and I added 1 (using the EXTRA_BOTTOM_ROW constant).
-
--Thue
-*/
-#define EXTRA_BOTTOM_ROW 1
-
 /* contains the x0, y0 coordinates of the upper left corner block */
 int map_view_x0, map_view_y0;
 
-/* used by map_canvas expose func */ 
-int force_full_repaint;
-
 static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
  				    struct Sprite *ssprite);
 static void show_city_descriptions(void);
@@ -83,42 +63,36 @@ Pixmap scaled_intro_pixmap;
 int scaled_intro_pixmap_width, scaled_intro_pixmap_height;
 
 /**************************************************************************
-Finds the pixel coordinates of a tile.
-Beside setting the results in canvas_x,canvas_y it returns whether the tile
-is inside the visible map.
+Finds the pixel coordinates of a tile.  Beside setting the results in
+canvas_x,canvas_y it returns whether the tile is inside the visible map.
+
+This function is almost identical between all GUI's.
 **************************************************************************/
-static int get_canvas_xy(int map_x, int map_y, int *canvas_x, int *canvas_y)
+static int get_canvas_xy(int map_x, int map_y, int *canvas_x,
+			 int *canvas_y)
 {
-  if (map_view_x0+map_canvas_store_twidth <= map.xsize)
-    *canvas_x = map_x-map_view_x0;
-  else if(map_x >= map_view_x0)
-    *canvas_x = map_x-map_view_x0;
-  else if(map_x < map_adjust_x(map_view_x0+map_canvas_store_twidth))
-    *canvas_x = map_x+map.xsize-map_view_x0;
-  else *canvas_x = -1;
+  Dimension width, height;
 
-  *canvas_y = map_y - map_view_y0;
+  XtVaGetValues(map_canvas, XtNwidth, &width, XtNheight, &height, NULL);
 
-  *canvas_x *= NORMAL_TILE_WIDTH;
-  *canvas_y *= NORMAL_TILE_HEIGHT;
-
-  return *canvas_x >= 0
-    && *canvas_x < map_canvas_store_twidth*NORMAL_TILE_WIDTH
-    && *canvas_y >= 0
-    && *canvas_y < map_canvas_store_twidth*NORMAL_TILE_HEIGHT;
+  return map_pos_to_canvas_pos(map_x, map_y, canvas_x, canvas_y,
+			       map_view_x0, map_view_y0, width, height);
 }
 
 /**************************************************************************
 Finds the map coordinates corresponding to pixel coordinates.
+
+This function is almost identical between all GUI's.
 **************************************************************************/
 void get_map_xy(int canvas_x, int canvas_y, int *map_x, int *map_y)
 {
-  *map_x = map_adjust_x(map_view_x0 + canvas_x/NORMAL_TILE_WIDTH);
-  *map_y = map_adjust_y(map_view_y0 + canvas_y/NORMAL_TILE_HEIGHT);
+  canvas_pos_to_map_pos(canvas_x, canvas_y, map_x, map_y, map_view_x0,
+			map_view_y0);
 }
 
 /**************************************************************************
-...
+ This function is called to decrease a unit's HP smoothly in battle
+ when combat_animation is turned on.
 **************************************************************************/
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0, 
 			     struct unit *punit1, int hp1)
@@ -128,17 +102,6 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   int i;
   int canvas_x, canvas_y;
 
-  if (!do_combat_animation) {
-    punit0->hp = hp0;
-    punit1->hp = hp1;
-
-    set_units_in_combat(NULL, NULL);
-    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
-
-    return;
-  }
-
   set_units_in_combat(punit0, punit1);
 
   do {
@@ -152,8 +115,8 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
     else
       punit0->hp--;
 
-    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+    refresh_tile_mapcanvas(punit0->x, punit0->y, TRUE);
+    refresh_tile_mapcanvas(punit1->x, punit1->y, TRUE);
 
     XSync(display, 0);
     usleep_since_timer_start(anim_timer, 10000);
@@ -179,8 +142,8 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   }
 
   set_units_in_combat(NULL, NULL);
-  refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-  refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+  refresh_tile_mapcanvas(punit0->x, punit0->y, TRUE);
+  refresh_tile_mapcanvas(punit1->x, punit1->y, TRUE);
 }
 
 /**************************************************************************
@@ -215,26 +178,25 @@ void set_overview_dimensions(int x, int y)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_turn_done_button(int do_restore)
+void update_turn_done_button(bool do_restore)
 {
-   static int flip;
-   Pixel fore, back;
+  static bool flip = FALSE;
  
-   if(game.player_ptr->ai.control && !ai_manual_turn_done)
-     return;
-   if((do_restore && flip) || !do_restore) { 
-   
-      XtVaGetValues(turn_done_button, 
-		    XtNforeground, &fore,
-		    XtNbackground, &back, NULL);
-      
-      
-      XtVaSetValues(turn_done_button, 
-		    XtNforeground, back,
-		    XtNbackground, fore, NULL);
-      
-      flip=!flip;
-   }
+  if (!get_turn_done_button_state()) {
+    return;
+  }
+
+  if ((do_restore && flip) || !do_restore) {
+    Pixel fore, back;
+
+    XtVaGetValues(turn_done_button, XtNforeground, &fore,
+		  XtNbackground, &back, NULL);
+
+    XtVaSetValues(turn_done_button, XtNforeground, back,
+		  XtNbackground, fore, NULL);
+
+    flip = !flip;
+  }
 }
 
 
@@ -265,7 +227,7 @@ void update_info_label(void)
 		"Year: %s\n"
 		"Gold: %d\n"
 		"Tax:%d Lux:%d Sci:%d"),
-	  int_to_text(civ_population(game.player_ptr)),
+	  population_to_text(civ_population(game.player_ptr)),
 	  textyear(game.year),
 	  game.player_ptr->economic.gold,
 	  game.player_ptr->economic.tax,
@@ -309,7 +271,7 @@ void update_unit_info_label(struct unit *punit)
     struct city *pcity;
     pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
     my_snprintf(buffer, sizeof(buffer), "%s %s\n%s\n%s\n%s", 
-		get_unit_type(punit->type)->name,
+		unit_type(punit)->name,
 		(punit->veteran) ? _("(veteran)") : "",
 		(hover_unit==punit->id) ? 
 		_("Select destination") : unit_activity_text(punit), 
@@ -391,21 +353,7 @@ void set_indicator_icons(int bulb, int sol, int flake, int gov)
 /**************************************************************************
 ...
 **************************************************************************/
-void refresh_tile_mapcanvas(int x, int y, int write_to_screen)
-{
-  x=map_adjust_x(x);
-  y=map_adjust_y(y);
-
-  if(tile_visible_mapcanvas(x, y)) {
-    update_map_canvas(x, y, 1, 1, write_to_screen);
-  }
-  overview_update_tile(x, y);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int tile_visible_mapcanvas(int x, int y)
+bool tile_visible_mapcanvas(int x, int y)
 {
   return (y>=map_view_y0 && y<map_view_y0+map_canvas_store_theight &&
 	  ((x>=map_view_x0 && x<map_view_x0+map_canvas_store_twidth) ||
@@ -416,7 +364,7 @@ int tile_visible_mapcanvas(int x, int y)
 /**************************************************************************
 ...
 **************************************************************************/
-int tile_visible_and_not_on_border_mapcanvas(int x, int y)
+bool tile_visible_and_not_on_border_mapcanvas(int x, int y)
 {
   return ((y>=map_view_y0+2 || (y >= map_view_y0 && map_view_y0 == 0))
 	  && (y<map_view_y0+map_canvas_store_theight-2 ||
@@ -435,7 +383,7 @@ Note: Works only for adjacent-square moves.
 void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 {
   static struct timer *anim_timer = NULL; 
-  int dest_x, dest_y;
+  int dest_x, dest_y, is_real;
 
   /* only works for adjacent-square moves */
   if ((dx < -1) || (dx > 1) || (dy < -1) || (dy > 1) ||
@@ -443,8 +391,10 @@ void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
     return;
   }
 
-  dest_x = map_adjust_x(x0+dx);
-  dest_y = map_adjust_y(y0+dy);
+  dest_x = x0 + dx;
+  dest_y = y0 + dy;
+  is_real = normalize_map_pos(&dest_x, &dest_y);
+  assert(is_real);
 
   if (player_can_see_unit(game.player_ptr, punit) &&
       (tile_visible_mapcanvas(x0, y0) ||
@@ -499,25 +449,21 @@ void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 **************************************************************************/
 void get_center_tile_mapcanvas(int *x, int *y)
 {
-  *x = map_adjust_x(map_view_x0+map_canvas_store_twidth/2);
-  *y = map_adjust_y(map_view_y0+map_canvas_store_theight/2);
+  Dimension width, height;
+  XtVaGetValues(map_canvas, XtNwidth, &width, XtNheight, &height, NULL);
+
+  /* This sets the pointers x and y */
+  get_map_xy(width/2, height/2, x, y);
 }
 
 /**************************************************************************
-...
+Centers the mapview around (x, y).
+
+This function is almost identical between all GUI's.
 **************************************************************************/
 void center_tile_mapcanvas(int x, int y)
 {
-  int new_map_view_x0, new_map_view_y0;
-
-  new_map_view_x0=map_adjust_x(x-map_canvas_store_twidth/2);
-  new_map_view_y0=map_adjust_y(y-map_canvas_store_theight/2);
-  if(new_map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight)
-     new_map_view_y0=
-       map_adjust_y(map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight);
-
-  map_view_x0=new_map_view_x0;
-  map_view_y0=new_map_view_y0;
+  base_center_tile_mapcanvas(x, y, &map_view_x0, &map_view_y0, map_canvas_store_twidth, map_canvas_store_theight);
 
   update_map_canvas_visible();
   update_map_canvas_scrollbars();
@@ -566,14 +512,11 @@ static void set_overview_tile_foreground_color(int x, int y)
 **************************************************************************/
 void refresh_overview_canvas(void)
 {
-  int x, y;
-  
-  for(y=0; y<map.ysize; y++)
-    for(x=0; x<map.xsize; x++) {
-      set_overview_tile_foreground_color(x, y);
-      XFillRectangle(display, overview_canvas_store, fill_bg_gc, x*2, y*2, 
-		     2, 2);
-    }
+  whole_map_iterate(x, y) {
+    set_overview_tile_foreground_color(x, y);
+    XFillRectangle(display, overview_canvas_store, fill_bg_gc, x * 2,
+		   y * 2, 2, 2);
+  } whole_map_iterate_end;
 
   XSetForeground(display, fill_bg_gc, 0);
 }
@@ -735,26 +678,29 @@ void map_canvas_resize(void)
 ...
 **************************************************************************/
 void update_map_canvas(int x, int y, int width, int height, 
-		       int write_to_screen)
+		       bool write_to_screen)
 {
-  int x_itr, y_itr;
-  int canvas_x, canvas_y;
+  int map_x, map_y;
 
-  for (y_itr=y; y_itr<y+height; y_itr++) {
-    for (x_itr=x; x_itr<x+width; x_itr++) {
-      int map_x = map_adjust_x(x_itr);
-      int map_y = y_itr;
-      get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y);
+  for (map_y = y; map_y < y + height; map_y++) {
+    for (map_x = x; map_x < x + width; map_x++) {
+      int canvas_x, canvas_y;
 
-      if (tile_visible_mapcanvas(map_x, map_y)) {
+      /*
+       * We don't normalize until later because we want to draw
+       * black tiles for unreal positions.
+       */
+      if (get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y)) {
 	pixmap_put_tile(map_canvas_store, map_x, map_y,
 			canvas_x, canvas_y, 0);
       }
     }
   }
 
-  get_canvas_xy(x, y, &canvas_x, &canvas_y);
   if (write_to_screen) {
+    int canvas_x, canvas_y;
+
+    get_canvas_xy(x, y, &canvas_x, &canvas_y);
     XCopyArea(display, map_canvas_store, XtWindow(map_canvas), 
 	      civ_gc, 
 	      canvas_x, canvas_y,
@@ -824,11 +770,15 @@ static void show_city_descriptions(void)
   if (!draw_city_names && !draw_city_productions)
     return;
 
-  for(y=0; y<map_canvas_store_theight; ++y) { 
-    int ry=map_view_y0+y;
-    for(x=0; x<map_canvas_store_twidth; ++x) { 
-      int rx=(map_view_x0+x)%map.xsize;
+  for (y = 0; y < map_canvas_store_theight; ++y) {
+    for (x = 0; x < map_canvas_store_twidth; ++x) {
+      int rx = map_view_x0 + x;
+      int ry = map_view_y0 + y;
       struct city *pcity;
+
+      if (!normalize_map_pos(&rx, &ry))
+        continue;
+
       if((pcity=map_get_city(rx, ry))) {
 
 	if (draw_city_names) {
@@ -839,28 +789,9 @@ static void show_city_descriptions(void)
 	}
 
 	if (draw_city_productions && (pcity->owner==game.player_idx)) {
-	  int turns, show_turns = 1;
-	  struct unit_type *punit_type;
-	  struct impr_type *pimpr_type;
-	  char *name;
 	  char buffer[512];
-
-	  turns = city_turns_to_build(pcity, pcity->currently_building,
-				      pcity->is_building_unit);
-
-	  if (pcity->is_building_unit) {
-	    punit_type = get_unit_type(pcity->currently_building);
-	    name = punit_type->name;
-	  } else {
-	    pimpr_type = get_improvement_type(pcity->currently_building);
-	    name = pimpr_type->name;
-	    show_turns = (pcity->currently_building != B_CAPITAL);
-	  }
-	  if (show_turns) {
-	    my_snprintf(buffer, sizeof(buffer), "%s %d", name, turns);
-	  } else {
-	    sz_strlcpy(buffer, name);
-	  }
+	
+          get_city_mapview_production(pcity, buffer, sizeof(buffer));
 
 	  draw_shadowed_string(prod_font_struct, prod_font_gc,
 			       x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2,
@@ -983,9 +914,9 @@ void put_nuke_mushroom_pixmaps(int x, int y)
     }
   }
   XSync(display, 0);
-  sleep(1);
+  myusleep(1000000);
 
-  update_map_canvas(x-1, y-1, 3, 3, 1);
+  update_map_canvas(x-1, y-1, 3, 3, TRUE);
 }
 
 /**************************************************************************
@@ -1030,7 +961,7 @@ void pixmap_put_tile(Pixmap pm, int x, int y, int canvas_x, int canvas_y,
   int fill_bg;
   struct player *pplayer;
 
-  if (normalize_map_pos(&x, &y) && tile_is_known(x, y)) {
+  if (normalize_map_pos(&x, &y) && tile_get_known(x, y)) {
     int count = fill_tile_sprite_array(tile_sprs, x, y, citymode, &fill_bg, &pplayer);
     int i = 0;
 
@@ -1059,27 +990,16 @@ void pixmap_put_tile(Pixmap pm, int x, int y, int canvas_x, int canvas_y,
     }
 
     if (draw_map_grid && !citymode) {
-      int here_in_radius =
-	player_in_city_radius(game.player_ptr, x, y);
       /* left side... */
-      if ((map_get_tile(x-1, y))->known &&
-	  (here_in_radius ||
-	   player_in_city_radius(game.player_ptr, x-1, y))) {
-	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_WHITE]);
-      } else {
-	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_BLACK]);
-      }
+      XSetForeground(display, civ_gc, colors_standard[get_grid_color
+						      (x, y, x - 1, y)]);
       XDrawLine(display, pm, civ_gc,
 		canvas_x, canvas_y,
 		canvas_x, canvas_y + NORMAL_TILE_HEIGHT);
+
       /* top side... */
-      if((map_get_tile(x, y-1))->known &&
-	 (here_in_radius ||
-	  player_in_city_radius(game.player_ptr, x, y-1))) {
-	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_WHITE]);
-      } else {
-	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_BLACK]);
-      }
+      XSetForeground(display, civ_gc, colors_standard[get_grid_color
+						      (x, y, x, y - 1)]);
       XDrawLine(display, pm, civ_gc,
 		canvas_x, canvas_y,
 		canvas_x + NORMAL_TILE_WIDTH, canvas_y);
@@ -1113,7 +1033,7 @@ void pixmap_put_tile(Pixmap pm, int x, int y, int canvas_x, int canvas_y,
 
   if (!citymode) {
     /* put any goto lines on the tile. */
-    if (y >= 0 && y < map.ysize) {
+    if (is_real_tile(x, y)) {
       int dir;
       for (dir = 0; dir < 8; dir++) {
 	if (get_drawn(x, y, dir)) {
@@ -1159,12 +1079,10 @@ static void pixmap_put_overlay_tile(Pixmap pixmap, int canvas_x, int canvas_y,
 **************************************************************************/
 void put_cross_overlay_tile(int x,int y)
 {
-  int canvas_x, canvas_y;
-  x=map_adjust_x(x);
-  y=map_adjust_y(y);
-  get_canvas_xy(x, y, &canvas_x, &canvas_y);
+  int canvas_x, canvas_y, is_real = normalize_map_pos(&x, &y);
+  assert(is_real);
 
-  if (tile_visible_mapcanvas(x, y)) {
+  if (get_canvas_xy(x, y, &canvas_x, &canvas_y)) {
     pixmap_put_overlay_tile(XtWindow(map_canvas), canvas_x, canvas_y,
 			    sprites.user.attention);
   }
@@ -1187,35 +1105,32 @@ void put_city_workers(struct city *pcity, int color)
 
   XSetForeground(display, fill_tile_gc, colors_standard[color]);
   get_canvas_xy(pcity->x, pcity->y, &canvas_x, &canvas_y);
-  city_map_iterate(i, j)  {
+  city_map_checked_iterate(pcity->x, pcity->y, i, j, x, y) {
     enum city_tile_type worked = get_worker_city(pcity, i, j);
-    int x = pcity->x + i - CITY_MAP_SIZE/2;
-    int y = pcity->y + j - CITY_MAP_SIZE/2;
-    if (normalize_map_pos(&x, &y)) {
-      get_canvas_xy(x, y, &canvas_x, &canvas_y);
-      if (!(i==2 && j==2)) {
-	if (worked == C_TILE_EMPTY) {
-	  XSetStipple(display, fill_tile_gc,gray25);
-	} else if (worked == C_TILE_WORKER) {
-	  XSetStipple(display, fill_tile_gc,gray50);
-	} else continue;
-	XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc, 
-		  canvas_x, canvas_y,
-		  NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-		  canvas_x, canvas_y);
-	XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
-		       canvas_x, canvas_y,
-		       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-      }
-      if (worked == C_TILE_WORKER) {
-	put_city_tile_output(XtWindow(map_canvas),
-			     canvas_x, canvas_y, 
-			     city_get_food_tile(i, j, pcity),
-			     city_get_shields_tile(i, j, pcity), 
-			     city_get_trade_tile(i, j, pcity) );
-      }
+
+    get_canvas_xy(x, y, &canvas_x, &canvas_y);
+    if (!is_city_center(i, j)) {
+      if (worked == C_TILE_EMPTY) {
+	XSetStipple(display, fill_tile_gc, gray25);
+      } else if (worked == C_TILE_WORKER) {
+	XSetStipple(display, fill_tile_gc, gray50);
+      } else
+	continue;
+      XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc,
+		canvas_x, canvas_y,
+		NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, canvas_x, canvas_y);
+      XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
+		     canvas_x, canvas_y,
+		     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
     }
-  } city_map_iterate_end;
+    if (worked == C_TILE_WORKER) {
+      put_city_tile_output(XtWindow(map_canvas),
+			   canvas_x, canvas_y,
+			   city_get_food_tile(i, j, pcity),
+			   city_get_shields_tile(i, j, pcity),
+			   city_get_trade_tile(i, j, pcity));
+    }
+  } city_map_checked_iterate_end;
 
   last_pcity = pcity;
 }
@@ -1256,7 +1171,7 @@ void scrollbar_jump_callback(Widget w, XtPointer client_data,
 void scrollbar_scroll_callback(Widget w, XtPointer client_data,
 			     XtPointer position_val)
 {
-  int position=(int)position_val;
+  int position = XTPOINTER_TO_INT(position_val), is_real;
 
 
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE)
@@ -1275,9 +1190,8 @@ void scrollbar_scroll_callback(Widget w, XtPointer client_data,
       map_view_y0--;
   }
 
-  map_view_x0=map_adjust_x(map_view_x0);
-  map_view_y0=map_adjust_y(map_view_y0);
-
+  is_real = normalize_map_pos(&map_view_x0, &map_view_y0);
+  assert(is_real);
 
   update_map_canvas_visible();
   update_map_canvas_scrollbars();
@@ -1307,13 +1221,10 @@ static void put_line(Pixmap pm, int x, int y, int dir)
 **************************************************************************/
 void draw_segment(int src_x, int src_y, int dir)
 {
-  int dest_x, dest_y;
+  int dest_x, dest_y, is_real;
 
-  dest_x = src_x + DIR_DX[dir];
-  dest_y = src_y + DIR_DY[dir];
-
-  assert(is_real_tile(dest_x, dest_y));
-  normalize_map_pos(&dest_x, &dest_y);
+  is_real = MAPSTEP(dest_x, dest_y, src_x, src_y, dir);
+  assert(is_real);
 
   /* A previous line already marks the place */
   if (get_drawn(src_x, src_y, dir)) {
@@ -1327,8 +1238,8 @@ void draw_segment(int src_x, int src_y, int dir)
   }
 
   if (tile_visible_mapcanvas(dest_x, dest_y)) {
-    put_line(map_canvas_store, dest_x, dest_y, 7-dir);
-    put_line(XtWindow(map_canvas), dest_x, dest_y, 7-dir);
+    put_line(map_canvas_store, dest_x, dest_y, DIR_REVERSE(dir));
+    put_line(XtWindow(map_canvas), dest_x, dest_y, DIR_REVERSE(dir));
   }
 
   increment_drawn(src_x, src_y, dir);
@@ -1340,10 +1251,12 @@ penalty so I will be lazy...
 **************************************************************************/
 void undraw_segment(int src_x, int src_y, int dir)
 {
-  int dest_x = src_x + DIR_DX[dir];
-  int dest_y = src_y + DIR_DY[dir];
+  int dest_x, dest_y, is_real;
   int drawn = get_drawn(src_x, src_y, dir);
 
+  is_real = MAPSTEP(dest_x, dest_y, src_x, src_y, dir);
+  assert(is_real);
+
   assert(drawn > 0);
   /* If we walk on a path twice it looks just like walking on it once. */
   if (drawn > 1) {
@@ -1352,24 +1265,25 @@ void undraw_segment(int src_x, int src_y, int dir)
   }
 
   decrement_drawn(src_x, src_y, dir);
-  refresh_tile_mapcanvas(src_x, src_y, 1);
-  assert(is_real_tile(dest_x, dest_y));
-  normalize_map_pos(&dest_x, &dest_y);
-  refresh_tile_mapcanvas(dest_x, dest_y, 1);
+  refresh_tile_mapcanvas(src_x, src_y, TRUE);
+  refresh_tile_mapcanvas(dest_x, dest_y, TRUE);
   if (NORMAL_TILE_WIDTH%2 == 0 || NORMAL_TILE_HEIGHT%2 == 0) {
-    if (dir == 2) { /* Since the tle doesn't have a middle we draw an extra pixel
-		       on the adjacent tile when drawing in this direction. */
+    int is_real;
+
+    if (dir == DIR8_NORTHEAST) {
+      /* Since the tile doesn't have a middle we draw an extra pixel
+         on the adjacent tile when drawing in this direction. */
       dest_x = src_x + 1;
       dest_y = src_y;
-      assert(is_real_tile(dest_x, dest_y));
-      normalize_map_pos(&dest_x, &dest_y);
-      refresh_tile_mapcanvas(dest_x, dest_y, 1);
-    } else if (dir == 5) { /* the same */
+      is_real = normalize_map_pos(&dest_x, &dest_y);
+      assert(is_real);
+      refresh_tile_mapcanvas(dest_x, dest_y, TRUE);
+    } else if (dir == DIR8_SOUTHWEST) {	/* the same */
       dest_x = src_x;
       dest_y = src_y + 1;
-      assert(is_real_tile(dest_x, dest_y));
-      normalize_map_pos(&dest_x, &dest_y);
-      refresh_tile_mapcanvas(dest_x, dest_y, 1);
+      is_real = normalize_map_pos(&dest_x, &dest_y);
+      assert(is_real);
+      refresh_tile_mapcanvas(dest_x, dest_y, TRUE);
     }
   }
 }
