@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -76,25 +75,6 @@ static void players_sship_callback(Widget w, XtPointer client_data,
 
 /******************************************************************/
 
-static int delay_plrdlg_update=0;
-
-/******************************************************************
- Turn off updating of player dialog
-*******************************************************************/
-void plrdlg_update_delay_on(void)
-{
-  delay_plrdlg_update=1;
-}
-
-/******************************************************************
- Turn on updating of player dialog
-*******************************************************************/
-void plrdlg_update_delay_off(void)
-{
-  delay_plrdlg_update=0;
-}
-
-
 /****************************************************************
 popup the dialog somewhat inside the main-window 
 *****************************************************************/
@@ -203,7 +183,7 @@ void create_players_dialog(void)
 **************************************************************************/
 void update_players_dialog(void)
 {
-   if(players_dialog_shell && !delay_plrdlg_update) {
+   if(players_dialog_shell && !is_plrdlg_frozen()) {
     int i,j;
     Dimension width;
     static char *namelist_ptrs[MAX_NUM_PLAYERS];
@@ -219,8 +199,10 @@ void update_players_dialog(void)
 
       /* text for idleness */
       if(game.players[i].nturns_idle>3) {
-	my_snprintf(idlebuf, sizeof(idlebuf), _("(idle %d turns)"),
-		    game.players[i].nturns_idle-1);
+	my_snprintf(idlebuf, sizeof(idlebuf),
+		    PL_("(idle %d turn)", "(idle %d turns)",
+			game.players[i].nturns_idle - 1),
+		    game.players[i].nturns_idle - 1);
       } else {
 	idlebuf[0]='\0';
       }
@@ -250,7 +232,7 @@ void update_players_dialog(void)
       if (i == game.player_idx) {
 	strcpy(dsbuf, "-");
       } else {
-	pds = player_get_diplstate(game.player_idx, i);
+	pds = pplayer_get_diplstate(game.player_ptr, get_player(i));
 	if (pds->type == DS_CEASEFIRE) {
 	  my_snprintf(dsbuf, sizeof(dsbuf), "%s (%d)",
 		      diplstate_text(pds->type), pds->turns_left);
@@ -301,8 +283,7 @@ void players_list_callback(Widget w, XtPointer client_data,
   ret=XawListShowCurrent(players_list);
 
   if(ret->list_index!=XAW_LIST_NONE) {
-    int player_index = list_index_to_player_index[ret->list_index];
-    struct player *pplayer = &game.players[player_index];
+    struct player *pplayer = get_player(list_index_to_player_index[ret->list_index]);
 
     if(pplayer->spaceship.state != SSHIP_NONE)
       XtSetSensitive(players_sship_command, TRUE);
@@ -310,20 +291,17 @@ void players_list_callback(Widget w, XtPointer client_data,
       XtSetSensitive(players_sship_command, FALSE);
 
     if(pplayer->is_alive) {
-      if (players_at_war(game.player_idx, player_index) ||
-	  game.player_idx == player_index)
-	XtSetSensitive(players_war_command, FALSE);
-      else
-	XtSetSensitive(players_war_command, TRUE);
+      XtSetSensitive(players_war_command,
+		     !pplayers_at_war(game.player_ptr, pplayer)
+		     && game.player_ptr != pplayer);
     }
 
-    if (game.player_ptr->gives_shared_vision & (1<<player_index)) {
-      XtSetSensitive(players_vision_command, TRUE);
-    } else {
-      XtSetSensitive(players_vision_command, FALSE);
-    }
+    XtSetSensitive(players_vision_command,
+		   gives_shared_vision(game.player_ptr, pplayer));
 
-    if(pplayer->is_alive && player_has_embassy(game.player_ptr, pplayer)) {
+    if (pplayer->is_alive
+        && pplayer != game.player_ptr
+        && player_has_embassy(game.player_ptr, pplayer)) {
       if(pplayer->is_connected)
 	XtSetSensitive(players_meet_command, TRUE);
       else
@@ -372,6 +350,7 @@ void players_meet_callback(Widget w, XtPointer client_data,
 
       pa.plrno0=game.player_idx;
       pa.plrno1=player_index;
+      pa.plrno_from=pa.plrno0;
       send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_INIT_MEETING,
 				 &pa);
     }
