@@ -13,8 +13,6 @@
 #ifndef FC__COLORS_G_H
 #define FC__COLORS_G_H
 
-#define MAX_COLORS 256
-
 enum color_std { 
   COLOR_STD_BLACK, COLOR_STD_WHITE, COLOR_STD_RED,
   COLOR_STD_YELLOW, COLOR_STD_CYAN,
@@ -35,7 +33,7 @@ enum Display_color_type {
 
 enum Display_color_type get_visual(void);
 void init_color_system(void);
+void free_color_system(void);
 void color_error(void);
 
 #endif  /* FC__COLORS_G_H */
-
