@@ -18,14 +18,16 @@ enum help_page_type { HELP_ANY, HELP_TEXT, HELP_UNIT, HELP_IMPROVEMENT,
                       HELP_GOVERNMENT };
 
 void popup_help_dialog(int item);
-void popup_help_dialog_string(char *item);
-void popup_help_dialog_typed(char *item, enum help_page_type);
+void popup_help_dialog_string(const char *item);
+void popup_help_dialog_typed(const char *item, enum help_page_type);
 void popdown_help_dialog(void);
 
 #define HELP_PLAYING_ITEM "Strategy and tactics"
 #define HELP_LANGUAGES_ITEM "Languages"
 #define HELP_CONNECTING_ITEM "Connecting"
 #define HELP_CHATLINE_ITEM "Chatline"
+#define HELP_WORKLIST_EDITOR_ITEM "Worklist Editor"
+#define HELP_CMA_ITEM "Citizen Management Agent"
 #define HELP_CONTROLS_ITEM "Controls"
 #define HELP_IMPROVEMENTS_ITEM "City Improvements"
 #define HELP_UNITS_ITEM "Units"
