@@ -13,16 +13,18 @@
 #ifndef FC__REPODLGS_G_H
 #define FC__REPODLGS_G_H
 
-void report_update_delay_on(void);
-void report_update_delay_off(void);
+#include "shared.h"		/* bool type */
+
+#include "repodlgs_common.h"
+
 char *get_report_title(char *report_name);
 void update_report_dialogs(void);
 
 void science_dialog_update(void);
-void popup_science_dialog(int make_modal);
+void popup_science_dialog(bool make_modal);
 void economy_report_dialog_update(void);
-void popup_economy_report_dialog(int make_modal);
+void popup_economy_report_dialog(bool make_modal);
 void activeunits_report_dialog_update(void);
-void popup_activeunits_report_dialog(int make_modal);
+void popup_activeunits_report_dialog(bool make_modal);
 
 #endif  /* FC__REPODLGS_G_H */
