@@ -26,14 +26,17 @@ struct Sprite;			/* opaque; gui-dep */
 struct unit;
 struct player;
 
+const char **get_tileset_list(void);
+
 void tilespec_read_toplevel(const char *tileset_name);
 void tilespec_load_tiles(void);
+void tilespec_free_tiles(void);
 
 void tilespec_setup_unit_type(int id);
 void tilespec_setup_tile_type(int id);
 void tilespec_setup_government(int id);
 void tilespec_setup_nation_flag(int id);
-void tilespec_setup_city_tiles(int id);
+void tilespec_setup_city_tiles(int style);
 void tilespec_alloc_city_tiles(int count);
 void tilespec_free_city_tiles(int count);
 
@@ -41,18 +44,18 @@ void tilespec_free_city_tiles(int count);
 
 int fill_tile_sprite_array_iso(struct Sprite **sprs, struct Sprite **coasts,
 			       struct Sprite **dither,
-			       int abs_x0, int abs_y0, int citymode,
+			       int x, int y, bool citymode,
 			       int *solid_bg);
 int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0,
-			   int citymode, int *solid_bg, struct player **pplayer);
+			   bool citymode, int *solid_bg, struct player **pplayer);
 int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit, int *solid_bg);
 int fill_city_sprite_array_iso(struct Sprite **sprs, struct city *pcity);
 
 enum color_std player_color(struct player *pplayer);
 enum color_std overview_tile_color(int x, int y);
 
-void set_focus_unit_hidden_state(int hide);
-struct unit *get_drawable_unit(int x, int y, int citymode);
+void set_focus_unit_hidden_state(bool hide);
+struct unit *get_drawable_unit(int x, int y, bool citymode);
 
 
 /* This the way directional indices are now encoded: */
@@ -62,16 +65,20 @@ struct unit *get_drawable_unit(int x, int y, int citymode);
 #define BIT_EAST  (0x04)
 #define BIT_WEST  (0x08)
 
-#define INDEX_NSEW(n,s,e,w) ((!!(n))*BIT_NORTH | (!!(s))*BIT_SOUTH | \
-                             (!!(e))*BIT_EAST  | (!!(w))*BIT_WEST)
+#define INDEX_NSEW(n,s,e,w) (((n) ? BIT_NORTH : 0) | \
+                             ((s) ? BIT_SOUTH : 0) | \
+                             ((e) ? BIT_EAST  : 0) | \
+                             ((w) ? BIT_WEST  : 0))
 
 #define NUM_TILES_PROGRESS 8
-#define NUM_TILES_CITIZEN 9
+#define NUM_TILES_CITIZEN 11
 #define NUM_TILES_HP_BAR 11
 #define NUM_TILES_DIGITS 10
 
-enum Directions { DIR_NORTH=0, DIR_SOUTH, DIR_EAST, DIR_WEST };
-
+/* This could be moved to common/map.h if there's more use for it. */
+enum direction4 {
+  DIR4_NORTH = 0, DIR4_SOUTH, DIR4_EAST, DIR4_WEST
+};
 
 struct named_sprites {
   struct Sprite
@@ -81,6 +88,7 @@ struct named_sprites {
     *citizen[NUM_TILES_CITIZEN],   /* internal code... */
     *treaty_thumb[2],     /* 0=disagree, 1=agree */
     *right_arrow,
+
     *black_tile,      /* only used for isometric view */
     *dither_tile,     /* only used for isometric view */
     *coast_color;     /* only used for isometric view */
@@ -160,12 +168,13 @@ struct named_sprites {
       *pollution,
       *village,
       *fortress,
+      *fortress_back,
       *airbase,
       *fallout,
       *fog,
       *spec_river[NUM_DIRECTION_NSEW],
       *darkness[NUM_DIRECTION_NSEW],         /* first unused */
-      *river_outlet[4],		/* indexed by enum Directions */
+      *river_outlet[4],		/* indexed by enum direction4 */
       /* for isometric */
       *spec_forest[NUM_DIRECTION_NSEW],
       *spec_mountain[NUM_DIRECTION_NSEW],
@@ -218,7 +227,7 @@ extern int UNIT_TILE_HEIGHT;
 extern int SMALL_TILE_WIDTH;
 extern int SMALL_TILE_HEIGHT;
 
-extern int is_isometric;
+extern bool is_isometric;
 
 /* name of font to use to draw city names on main map */
 
@@ -228,7 +237,7 @@ extern char *city_names_font;
 
 extern char *city_productions_font_name;
 
-extern int flags_are_transparent;
+extern bool flags_are_transparent;
 
 extern int num_tiles_explode_unit;
 
