@@ -14,6 +14,7 @@
 #define FC__DIPTREATY_H
 
 #include "genlist.h"
+#include "shared.h"		/* bool type */
 
 enum clause_type { CLAUSE_ADVANCE, CLAUSE_GOLD, CLAUSE_MAP,
 		   CLAUSE_SEAMAP, CLAUSE_CITY, 
@@ -23,6 +24,16 @@ enum clause_type { CLAUSE_ADVANCE, CLAUSE_GOLD, CLAUSE_MAP,
 #define is_pact_clause(x) \
   ((x == CLAUSE_CEASEFIRE) || (x == CLAUSE_PEACE) || (x == CLAUSE_ALLIANCE))
 
+/* For when we need to iterate over treaties */
+struct Clause;
+#define SPECLIST_TAG clause
+#define SPECLIST_TYPE struct Clause
+#include "speclist.h"
+
+#define clause_list_iterate(clauselist, pclause) \
+    TYPED_LIST_ITERATE(struct Clause, clauselist, pclause)
+#define clause_list_iterate_end  LIST_ITERATE_END
+
 struct Clause {
   enum clause_type type;
   struct player *from;
@@ -31,15 +42,15 @@ struct Clause {
 
 struct Treaty {
   struct player *plr0, *plr1;
-  int accept0, accept1;
-  struct genlist clauses;
+  bool accept0, accept1;
+  struct clause_list clauses;
 };
 
 void init_treaty(struct Treaty *ptreaty, 
 		 struct player *plr0, struct player *plr1);
-int add_clause(struct Treaty *ptreaty, struct player *pfrom, 
+bool add_clause(struct Treaty *ptreaty, struct player *pfrom, 
 	       enum clause_type type, int val);
-int remove_clause(struct Treaty *ptreaty, struct player *pfrom, 
+bool remove_clause(struct Treaty *ptreaty, struct player *pfrom, 
 		  enum clause_type type, int val);
 
 #endif /* FC__DIPTREATY_H */
