@@ -19,9 +19,12 @@
 #include "shared.h"		/* MAX_LEN_NAME, MAX_LEN_ADDR */
 #include "spaceship.h"
 #include "worklist.h"
+#include "unittype.h"
 
-#define MAX_LEN_USERNAME    10	     /* see below */
-#define MAX_LEN_MSG       1536
+#define MAX_LEN_USERNAME        10        /* see below */
+#define MAX_LEN_MSG             1536
+#define MAX_ATTRIBUTE_BLOCK     (256*1024)	/* largest attribute block */
+#define ATTRIBUTE_CHUNK_SIZE    (1024*2)  /* attribute chunk size to use */
 
 /* Note that MAX_LEN_USERNAME cannot be expanded, because it
    is used for the name in the first packet sent by the client,
@@ -31,6 +34,8 @@
 enum packet_type {
   PACKET_REQUEST_JOIN_GAME,
   PACKET_JOIN_GAME_REPLY,
+  PACKET_PROCESSING_STARTED,
+  PACKET_PROCESSING_FINISHED,
   PACKET_SERVER_SHUTDOWN,
   PACKET_UNIT_INFO,
   PACKET_MOVE_UNIT,
@@ -57,7 +62,6 @@ enum packet_type {
   PACKET_PLAYER_REVOLUTION,
   PACKET_PLAYER_GOVERNMENT,
   PACKET_PLAYER_RESEARCH,
-  PACKET_PLAYER_WORKLIST,
   PACKET_UNIT_BUILD_CITY,
   PACKET_UNIT_DISBAND,
   PACKET_REMOVE_UNIT,
@@ -118,6 +122,12 @@ enum packet_type {
   PACKET_CONN_PING,
   PACKET_CONN_PONG,
   PACKET_UNIT_AIRLIFT,
+  PACKET_ATTRIBUTE_CHUNK,
+  PACKET_PLAYER_ATTRIBUTE_BLOCK,
+  PACKET_START_TURN,
+  PACKET_SELECT_NATION_OK,
+  PACKET_FREEZE_HINT,
+  PACKET_THAW_HINT,
   PACKET_LAST  /* leave this last */
 };
 
@@ -232,8 +242,7 @@ struct packet_player_request
   int tax, luxury, science;              /* rates */
   int government;                        /* government */
   int tech;                              /* research */
-  struct worklist worklist;              /* one worklist */
-  int wl_idx;                            /* which worklist */
+  bool attribute_block;                   /* send attribute block as chunks */
 };
 
 /*********************************************************
@@ -243,7 +252,7 @@ struct packet_city_request
 {
   int city_id;                           /* all */
   int build_id;                          /* change, sell */
-  int is_build_id_unit_id;               /* change */
+  bool is_build_id_unit_id;               /* change */
   int worker_x, worker_y;                /* make_worker, make_specialist */
   int specialist_from, specialist_to;    /* change_specialist */
   char name[MAX_LEN_NAME];            /* rename */
@@ -264,7 +273,7 @@ struct packet_tile_info {
 send to each client whenever the turn has ended.
 *********************************************************/
 struct packet_new_year {
-  int year;
+  int year, turn;
 };
 
 
@@ -287,7 +296,7 @@ struct packet_unit_info {
   int id;
   int owner;
   int x, y;
-  int veteran;
+  bool veteran;
   int homecity;
   int type;
   int movesleft;
@@ -298,15 +307,15 @@ struct packet_unit_info {
   int upkeep;
   int upkeep_food;
   int upkeep_gold;
-  int ai;
+  bool ai;
   int fuel;
   int goto_dest_x, goto_dest_y;
-  int activity_target;
-  int paradropped;
-  int connecting;
+  enum tile_special_type activity_target;
+  bool paradropped;
+  bool connecting;
   /* in packet only, not in unit struct */
-  int carried;
-  int select_it;
+  bool carried;
+  bool select_it;
   int packet_use;	/* see enum unit_info_use */
   int info_city_id;	/* for UNIT_INFO_CITY_SUPPORTED
 			   and UNIT_INFO_CITY_PRESENT uses */
@@ -325,12 +334,12 @@ struct packet_city_info {
   char name[MAX_LEN_NAME];
 
   int size;
-  int ppl_happy[5], ppl_content[5], ppl_unhappy[5];
+  int ppl_happy[5], ppl_content[5], ppl_unhappy[5], ppl_angry[5];
   int ppl_elvis, ppl_scientist, ppl_taxman;
   int food_prod, food_surplus;
   int shield_prod, shield_surplus;
-  int trade_prod, corruption;
-  int trade[4],trade_value[4];
+  int trade_prod, tile_trade, corruption;
+  int trade[NUM_TRADEROUTES], trade_value[NUM_TRADEROUTES];
   int luxury_total, tax_total, science_total;
 
   /* the physics */
@@ -338,13 +347,13 @@ struct packet_city_info {
   int shield_stock;
   int pollution;
 
-  int is_building_unit;
+  bool is_building_unit;
   int currently_building;
 
   int turn_last_built;
   int turn_changed_target;
   int changed_from_id;
-  int changed_from_is_unit;
+  bool changed_from_is_unit;
   int before_change_shields;
   int disbanded_shields;
   int caravan_shields;
@@ -354,11 +363,12 @@ struct packet_city_info {
   char improvements[B_LAST+1];
   char city_map[CITY_MAP_SIZE*CITY_MAP_SIZE+1];
 
-  int did_buy, did_sell;
-  int was_happy;
-  int airlift;
-  int diplomat_investigate;
+  bool did_buy, did_sell;
+  bool was_happy;
+  bool airlift;
+  bool diplomat_investigate;
   int city_options;
+  int turn_founded;
 };
 
 
@@ -368,9 +378,10 @@ struct packet_short_city {
   int x, y;			/* uint8 */
   char name[MAX_LEN_NAME];
   int size;			/* uint8 */
-  int happy;			/* boolean */
-  int capital;			/* boolean */
-  int walls;			/* boolean */
+  bool happy;			/* boolean */
+  bool capital;			/* boolean */
+  bool walls;			/* boolean */
+  int tile_trade;		/* same as in packet_city_info */
 };
 
 
@@ -396,7 +407,7 @@ struct packet_req_join_game {
  ... and the server replies.
 *********************************************************/
 struct packet_join_game_reply {
-  int you_can_join;             /* true/false */
+  bool you_can_join;             /* true/false */
   char message[MAX_LEN_MSG];
   char capability[MAX_LEN_CAPSTR];
   int conn_id;			/* clients conn id as known in server */
@@ -409,7 +420,7 @@ struct packet_join_game_reply {
 struct packet_alloc_nation {
   Nation_Type_id nation_no;
   char name[MAX_LEN_NAME];
-  int is_male;
+  bool is_male;
   int city_style;
 };
 
@@ -448,27 +459,28 @@ struct packet_generic_empty {
 struct packet_player_info {
   int playerno;
   char name[MAX_LEN_NAME];
-  int is_male;
+  bool is_male;
+  int team;
   int government;
   int embassy;
   int city_style;
   int nation;
-  int turn_done, nturns_idle;
-  int is_alive;
+  bool turn_done;
+  int nturns_idle;
+  bool is_alive;
   int reputation;
   struct player_diplstate diplstates[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
   int gold, tax, science, luxury;
-  int researched;
-  int researchpoints;
+  int bulbs_researched;
+  int techs_researched;
   int researching;
   int future_tech;
   int tech_goal;
   unsigned char inventions[A_LAST+1];
-  int is_connected;
+  bool is_connected;
   int revolution;
-  int ai;
-  int is_barbarian;
-  struct worklist worklists[MAX_NUM_WORKLISTS];
+  bool ai;
+  int barbarian_type;
   unsigned int gives_shared_vision;
 };
 
@@ -479,11 +491,11 @@ struct packet_player_info {
 **************************************************************************/
 struct packet_conn_info {
   int id;
-  int used;			/* boolean; 0 means client should forget its
+  bool used;			/* 0 means client should forget its
 				   info about this connection */
-  int established;		/* boolean */
+  bool established;
   int player_num;		/* range uchar; index in game.players, or 255 */
-  int observer;			/* boolean (?) */
+  bool observer;
   
   enum cmdlevel_id access_level;   /* range uchar */
   
@@ -545,11 +557,11 @@ struct packet_ruleset_control {
   int aqueduct_size;
   int sewer_size;
   int add_to_size_limit;
+  int notradesize, fulltradesize;
   int num_unit_types;
   int num_impr_types;
   int num_tech_types;
   struct {
-    int get_bonus_tech;
     int cathedral_plus;
     int cathedral_minus;
     int colosseum_plus;
@@ -562,6 +574,7 @@ struct packet_ruleset_control {
   int nation_count;
   int playable_nation_count;
   int style_count;
+  char team_name[MAX_NUM_TEAMS][MAX_LEN_NAME];
 };
 
 /*********************************************************
@@ -572,8 +585,13 @@ struct packet_ruleset_unit {
   char name[MAX_LEN_NAME];
   char graphic_str[MAX_LEN_NAME];
   char graphic_alt[MAX_LEN_NAME];
+  char sound_move[MAX_LEN_NAME];
+  char sound_move_alt[MAX_LEN_NAME];
+  char sound_fight[MAX_LEN_NAME];
+  char sound_fight_alt[MAX_LEN_NAME];
   int move_type;
   int build_cost;
+  int pop_cost;
   int attack_strength;
   int defense_strength;
   int move_rate;
@@ -584,8 +602,9 @@ struct packet_ruleset_unit {
   int firepower;
   int obsoleted_by;
   int fuel;
-  int flags;
-  int roles;			/* a client-side-ai might be interested */
+
+  bv_flags flags;
+  bv_roles roles;
 
   int happy_cost;  /* unhappy people in home city */
   int shield_cost; /* normal upkeep cost */
@@ -609,6 +628,8 @@ struct packet_ruleset_tech {
   int flags;
   char name[MAX_LEN_NAME];
   char *helptext;		/* same as for packet_ruleset_unit, above */
+  int preset_cost;
+  int num_reqs;
 };
 
 struct packet_ruleset_building {
@@ -622,13 +643,15 @@ struct packet_ruleset_building {
   Impr_Type_id *equiv_dupl;
   Impr_Type_id *equiv_repl;
   Tech_Type_id obsolete_by;
-  int is_wonder;
+  bool is_wonder;
   int build_cost;
   int upkeep;
   int sabotage;
   struct impr_effect *effect;
   int variant;		/* FIXME: remove when gen-impr obsoletes */
   char *helptext;		/* same as for packet_ruleset_unit, above */
+  char soundtag[MAX_LEN_NAME];
+  char soundtag_alt[MAX_LEN_NAME];
 };
 
 struct packet_ruleset_terrain {
@@ -751,8 +774,9 @@ struct packet_ruleset_nation {
 
   int leader_count;
   char leader_name[MAX_NUM_LEADERS][MAX_LEN_NAME];
-  int leader_sex[MAX_NUM_LEADERS];
+  bool leader_sex[MAX_NUM_LEADERS];
   int city_style;
+  int init_techs[MAX_NUM_TECH_LIST];
 };
 
 struct packet_ruleset_city {
@@ -771,10 +795,13 @@ struct packet_ruleset_game {
   int min_dist_bw_cities;
   int init_vis_radius_sq;
   int hut_overflight;
-  int pillage_select;
+  bool pillage_select;
   int nuke_contamination;
   int granary_food_ini;
   int granary_food_inc;
+  int tech_cost_style;
+  int tech_leakage;
+  int global_init_techs[MAX_NUM_TECH_LIST];
 };
 
 /*********************************************************
@@ -789,6 +816,7 @@ struct packet_game_info {
   int timeout;
   int end_year;
   int year;
+  int turn;
   int min_players, max_players, nplayers;
   int player_idx;
   int globalwarming;
@@ -797,12 +825,13 @@ struct packet_game_info {
   int cooling;
   int cityfactor;
   int unhappysize;
+  bool angrycitizen;
   int diplcost,freecost,conquercost;
   int global_advances[A_LAST];
   int global_wonders[B_LAST];
   int foodbox;
   int techpenalty;
-  int spacerace;
+  bool spacerace;
   /* the following values are computed each time packet_game_info is sent */
   int seconds_to_turndone;
 };
@@ -812,7 +841,7 @@ struct packet_game_info {
 *********************************************************/
 struct packet_map_info {
   int xsize, ysize;
-  int is_earth;
+  bool is_earth;
 };
 
 /*********************************************************
@@ -855,9 +884,20 @@ struct packet_goto_route
   int unit_id;
 };
 
-/* These two are non-static for meta.c; others are now static --dwp */
-unsigned char *put_uint16(unsigned char *buffer, int val);
-unsigned char *put_string(unsigned char *buffer, const char *mystring);
+struct packet_attribute_chunk
+{
+  int offset, total_length, chunk_length;
+  /* to keep memory management simple don't allocate dynamic memory */
+  unsigned char data[ATTRIBUTE_CHUNK_SIZE];
+};
+
+/*********************************************************
+...
+*********************************************************/
+struct packet_nations_used {
+  int num_nations_used;
+  Nation_Type_id nations_used[MAX_NUM_PLAYERS];
+};
 
 int send_packet_diplomacy_info(struct connection *pc, enum packet_type pt,
 			       const struct packet_diplomacy_info *packet);
@@ -936,12 +976,12 @@ int send_packet_alloc_nation(struct connection *pc,
 struct packet_alloc_nation *receive_packet_alloc_nation(struct connection *pc);
 
 
-int send_packet_generic_message(struct connection *pc, int type,
-				const struct packet_generic_message *message);
+int send_packet_generic_message(struct connection *pc, enum packet_type type,
+				const struct packet_generic_message *packet);
 struct packet_generic_message *receive_packet_generic_message(struct 
 							      connection *pc);
 
-int send_packet_generic_integer(struct connection *pc, int type,
+int send_packet_generic_integer(struct connection *pc, enum packet_type type,
 				const struct packet_generic_integer *packet);
 struct packet_generic_integer *receive_packet_generic_integer(struct 
 							      connection *pc);
@@ -978,7 +1018,7 @@ int send_packet_unittype_info(struct connection *pc, int type, int action);
 struct packet_unittype_info *receive_packet_unittype_info(struct connection *pc);
 
 int send_packet_ruleset_control(struct connection *pc, 
-				const struct packet_ruleset_control *pinfo);
+				const struct packet_ruleset_control *packet);
 struct packet_ruleset_control *
 receive_packet_ruleset_control(struct connection *pc);
 
@@ -1030,7 +1070,7 @@ int send_packet_ruleset_game(struct connection *pc,
 struct packet_ruleset_game *
 receive_packet_ruleset_game(struct connection *pc);
 
-int send_packet_generic_values(struct connection *pc, int type,
+int send_packet_generic_values(struct connection *pc, enum packet_type type,
 			       const struct packet_generic_values *req);
 struct packet_generic_values *
 receive_packet_generic_values(struct connection *pc);
@@ -1055,7 +1095,7 @@ int send_packet_sabotage_list(struct connection *pc,
 struct packet_sabotage_list *
 receive_packet_sabotage_list(struct connection *pc);
 
-void *get_packet_from_connection(struct connection *pc, int *ptype, int *presult);
+void *get_packet_from_connection(struct connection *pc, enum packet_type *ptype, bool *presult);
 void remove_packet_from_buffer(struct socket_packet_buffer *buffer);
 
 int send_packet_goto_route(struct connection *pc,
@@ -1063,10 +1103,25 @@ int send_packet_goto_route(struct connection *pc,
 			   enum goto_route_type type);
 struct packet_goto_route *receive_packet_goto_route(struct connection *pc);
 
-int send_packet_generic_empty(struct connection *pc, int type);
+int send_packet_attribute_chunk(struct connection *pc,
+				struct packet_attribute_chunk *packet);
+struct packet_attribute_chunk *receive_packet_attribute_chunk(struct
+							      connection
+							      *pc);
+void send_attribute_block(const struct player *pplayer,
+			  struct connection *pconn);
+void generic_handle_attribute_chunk(struct player *pplayer,
+				    struct packet_attribute_chunk *chunk);
+
+int send_packet_generic_empty(struct connection *pc, enum packet_type type);
 struct packet_generic_empty *
 receive_packet_generic_empty(struct connection *pc);
 
+int send_packet_nations_used(struct connection *pc,
+			     const struct packet_nations_used *packet);
+struct packet_nations_used *receive_packet_nations_used(struct connection
+							*pc);
+
 #include "packets_lsend.h"		/* lsend_packet_* functions */
 
 #endif  /* FC__PACKETS_H */
