@@ -15,7 +15,20 @@
 
 #include <stdlib.h>		/* size_t */
 
-#include "attribute.h"
+
+/* Want to use GCC's __attribute__ keyword to check variadic
+ * parameters to printf-like functions, without upsetting other
+ * compilers: put any required defines magic here.
+ * If other compilers have something equivalent, could also
+ * work that out here.   Should this use configure stuff somehow?
+ * --dwp
+ */
+#if defined(__GNUC__)
+#define fc__attribute(x)  __attribute__(x)
+#else
+#define fc__attribute(x)
+#endif
+
 
 /* Note: the capability string is now in capstr.c --dwp */
 /* Version stuff is now in version.h --dwp */
@@ -23,14 +36,16 @@
 #define BUG_EMAIL_ADDRESS "bugs@freeciv.freeciv.org"
 #define WEBSITE_URL "http://www.freeciv.org/"
 
-#define BROADCAST_EVENT -2
-
 #define MAX_NUM_PLAYERS  30
 #define MAX_NUM_BARBARIANS   2
 #define MAX_NUM_ITEMS   200	/* eg, unit_types */
 #define MAX_NUM_TECH_LIST 10
 #define MAX_LEN_NAME     32
-#define MAX_LEN_ADDR     32
+#define MAX_LEN_ADDR     256	/* see also MAXHOSTNAMELEN and RFC 1123 2.1 */
+
+/* Use FC_INFINITY to denote that a certain event will never occur or
+   another unreachable condition. */
+#define FC_INFINITY    	(1000 * 1000 * 1000)
 
 #ifndef TRUE
 #define TRUE (1)
@@ -39,6 +54,8 @@
 #define FALSE (0)
 #endif
 
+typedef int bool;
+
 #ifndef MAX
 #define MAX(x,y) (((x)>(y))?(x):(y))
 #define MIN(x,y) (((x)<(y))?(x):(y))
@@ -53,17 +70,55 @@
    and inserts a zero at the top. */
 #define WIPEBIT(val, no) ( ((~(-1<<(no)))&(val)) \
                            | (( (-1<<((no)+1)) & (val)) >>1) )
+/*
+ * Yields TRUE iff the bit bit_no is set in val.
+ */
+#define TEST_BIT(val, bit_no)      	(((val) & (1u << (bit_no))) == (1u << (bit_no)))
+
+/*
+ * If cond is TRUE it yields a value where only the bit bit_no is
+ * set. If cond is FALSE it yields 0.
+ */
+#define COND_SET_BIT(cond, bit_no) 	((cond) ? (1u << (bit_no)) : 0)
 
 /* This is duplicated in rand.h to avoid extra includes: */
 #define MAX_UINT32 0xFFFFFFFF
 
-char *create_centered_string(char *s);
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
+#define ADD_TO_POINTER(p, n) ((void *)((char *)(p)+(n)))
+
+/* Bitvectors. */
+#define _BV_BYTES(bits)		((((bits) - 1) / 8) + 1)
+#define _BV_BYTE_INDEX(bits)	((bits) / 8)
+#define _BV_BITMASK(bit)	(1u << ((bit) & 0x7))
+#define BV_ISSET(bv, bit) \
+  (((bv).vec[_BV_BYTE_INDEX(bit)] & _BV_BITMASK(bit)) != 0)
+#define BV_SET(bv, bit) \
+  do { (bv).vec[_BV_BYTE_INDEX(bit)] |= _BV_BITMASK(bit); } while(FALSE)
+#define BV_CLR(bv, bit) \
+  do { (bv).vec[_BV_BYTE_INDEX(bit)] &= ~_BV_BITMASK(bit); } while(FALSE)
+#define BV_CLR_ALL(bv) \
+  do { memset((bv).vec, 0, sizeof((bv).vec)); } while(FALSE)
+bool bv_check_mask(unsigned char *vec1, unsigned char *vec2, size_t size1,
+		   size_t size2);
+#define BV_CHECK_MASK(vec1, vec2) \
+  bv_check_mask((vec1).vec, (vec2).vec, sizeof((vec1).vec), sizeof((vec2).vec))
+
+#define BV_DEFINE(name, bits) \
+  typedef struct { unsigned char vec[_BV_BYTES(bits)]; } name
+
+char *create_centered_string(const char *s);
 
 char * get_option(const char *option_name,char **argv,int *i,int argc);
-int is_option(const char *option_name,char *option);
-char *int_to_text(int nr);
-char *get_sane_name(char *name);
-char *textyear(int year);
+bool is_option(const char *option_name,char *option);
+int get_tokens(const char *str, char **tokens, size_t num_tokens,
+	       const char *delimiterset);
+const char *general_int_to_text(int nr, int decade_exponent);
+const char *int_to_text(int nr);
+const char *population_to_text(int thousand_citizen);
+
+const char *get_sane_name(const char *name);
+const char *textyear(int year);
 int compare_strings(const void *first, const void *second);
 int compare_strings_ptrs(const void *first, const void *second);
 
@@ -74,7 +129,7 @@ void remove_leading_trailing_spaces(char *s);
 void remove_trailing_char(char *s, char trailing);
 int wordwrap_string(char *s, int len);
 
-int check_strlen(const char *str, size_t len, const char *errmsg);
+bool check_strlen(const char *str, size_t len, const char *errmsg);
 size_t loud_strlcpy(char *buffer, const char *str, size_t len,
 		   const char *errmsg);
 /* Convenience macro. */
@@ -87,6 +142,7 @@ int cat_snprintf(char *str, size_t n, const char *format, ...)
 
 char *user_home_dir(void);
 char *user_username(void);
+const char **datafilelist(const char *suffix);
 char *datafilename(const char *filename);
 char *datafilename_required(const char *filename);
 
@@ -120,6 +176,9 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
 			       const char *prefix,
 			       int *ind_result);
 
-char *freeciv_motto(void);
+const char *freeciv_motto(void);
+
+char *convert_data_string_malloc(const char *text);
+char *get_local_charset(void);
 
 #endif  /* FC__SHARED_H */
