@@ -51,7 +51,7 @@ int SPECLIST_FOO(_list_size) (SPECLIST_LIST *tthis)
 
 SPECLIST_TYPE *SPECLIST_FOO(_list_get) (SPECLIST_LIST *tthis, int index)
 {
-  return genlist_get(&tthis->list, index);
+  return (SPECLIST_TYPE *)genlist_get(&tthis->list, index);
 }
 
 void SPECLIST_FOO(_list_insert) (SPECLIST_LIST *tthis, SPECLIST_TYPE *pfoo)
@@ -74,6 +74,11 @@ void SPECLIST_FOO(_list_unlink_all) (SPECLIST_LIST *tthis)
   genlist_unlink_all(&tthis->list);
 }
 
+void SPECLIST_FOO(_list_sort) (SPECLIST_LIST * tthis, int (*compar) (const void *, const void *))
+{
+  genlist_sort(&tthis->list, compar);
+}
+
 #undef SPECLIST_TAG
 #undef SPECLIST_TYPE
 #undef SPECLIST_PASTE_
