@@ -25,10 +25,10 @@
 #define MAJOR_VERSION		1
 #endif
 #ifndef MINOR_VERSION
-#define MINOR_VERSION		12
+#define MINOR_VERSION		14
 #endif
 #ifndef PATCH_VERSION
-#define PATCH_VERSION		0
+#define PATCH_VERSION		2
 #endif
 #ifndef VERSION_LABEL
 #define VERSION_LABEL		""
@@ -42,12 +42,12 @@
 
 /* This is only used if IS_BETA_VERSION is true. */
 #ifndef NEXT_STABLE_VERSION
-#define NEXT_STABLE_VERSION	"1.12.0"
+#define NEXT_STABLE_VERSION	"1.14.2"
 #endif
 /* This is only used in version.c, and only if IS_BETA_VERSION is true.
    The month[] array is defined in version.c (index: 1==Jan, 2==Feb, ...). */
 #ifndef NEXT_RELEASE_MONTH
-#define NEXT_RELEASE_MONTH	(month[8])
+#define NEXT_RELEASE_MONTH	(month[9])
 #endif
 
 #ifndef VERSION_STRING
@@ -59,10 +59,10 @@
 #endif
 
 /* version informational strings */
-char *freeciv_name_version(void);
-char *word_version(void);
+const char *freeciv_name_version(void);
+const char *word_version(void);
 
 /* If returns NULL, not a beta version. */
-char *beta_message(void);
+const char *beta_message(void);
 
 #endif  /* FC__VERSION_H */
