@@ -60,14 +60,14 @@
 Is player a sea or land barbarian
 **************************************************************************/
 
-int is_land_barbarian(struct player *pplayer)
+bool is_land_barbarian(struct player *pplayer)
 {
-  return (pplayer->ai.is_barbarian == LAND_BARBARIAN);
+  return (pplayer->ai.barbarian_type == LAND_BARBARIAN);
 }
 
-int is_sea_barbarian(struct player *pplayer)
+bool is_sea_barbarian(struct player *pplayer)
 {
-  return (pplayer->ai.is_barbarian == SEA_BARBARIAN);
+  return (pplayer->ai.barbarian_type == SEA_BARBARIAN);
 }
 
 /**************************************************************************
@@ -76,64 +76,67 @@ If barbarian player already exists, return player pointer.
 If barbarians are dead, revive them with a new leader :-)
 Dead barbarians forget the map and lose the money.
 **************************************************************************/
-
-static struct player *create_barbarian_player(int land)
+static struct player *create_barbarian_player(bool land)
 {
   int newplayer = game.nplayers;
   struct player *barbarians;
-  int i, j, k;
 
-  for( i = 0; i < game.nplayers; i++ ) {
-    barbarians = &game.players[i];
+  players_iterate(barbarians) {
     if( (land && is_land_barbarian(barbarians)) ||
         (!land && is_sea_barbarian(barbarians)) ) {
-      if( barbarians->is_alive == 0 ) {
+      if (!barbarians->is_alive) {
         barbarians->economic.gold = 0;
-        barbarians->is_alive = 1;
+        barbarians->is_alive = TRUE;
         pick_ai_player_name( game.nation_count-1, barbarians->name);
 	sz_strlcpy(barbarians->username, barbarians->name);
         /* I need to make them to forget the map, I think */
-        for( j = 0; j < map.xsize; j++ )
-          for( k = 0; k < map.ysize; k++ )
-            map_clear_known( j, k, barbarians);
+	whole_map_iterate(x, y) {
+	  map_clear_known(x, y, barbarians);
+	} whole_map_iterate_end;
       }
       barbarians->economic.gold += 100;  /* New leader, new money */
       return barbarians;
     }
-  }
+  } players_iterate_end;
 
   if( newplayer >= MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS ) {
-    freelog( LOG_FATAL, "Too many players?");
-    exit(1);
+    freelog(LOG_ERROR, "Too many players for barbarian.c (%d)", newplayer);
+    return NULL;
   }
 
   barbarians = &game.players[newplayer];
 
   /* make a new player */
 
-  server_player_init(barbarians, 1);
+  server_player_init(barbarians, TRUE);
 
   barbarians->nation = game.nation_count-1;
   pick_ai_player_name( game.nation_count-1, barbarians->name);
 
   sz_strlcpy(barbarians->username, barbarians->name);
-  barbarians->is_connected = 0;
+  barbarians->is_connected = FALSE;
   barbarians->government = game.government_when_anarchy; 
-  barbarians->capital = 0;
+  barbarians->capital = FALSE;
   barbarians->economic.gold = 100;
 
-  barbarians->turn_done = 1;
+  barbarians->turn_done = TRUE;
 
   /* Do the ai */
-
-  barbarians->ai.control = 1;
-  if( land )
-    barbarians->ai.is_barbarian = LAND_BARBARIAN;
-  else
-    barbarians->ai.is_barbarian = SEA_BARBARIAN;
+  barbarians->ai.control = TRUE;
+  if (land) {
+    barbarians->ai.barbarian_type = LAND_BARBARIAN;
+  } else {
+    barbarians->ai.barbarian_type = SEA_BARBARIAN;
+  }
   set_ai_level_directer(barbarians, game.skill_level);
   init_tech(barbarians, game.tech);
 
+  /* Ensure that we are at war with everyone else */
+  players_iterate(pplayer) {
+    pplayer->diplstates[barbarians->player_no].type = DS_WAR;
+    barbarians->diplstates[pplayer->player_no].type = DS_WAR;
+  } players_iterate_end;
+
   game.nplayers++;
   game.nbarbarians++;
   game.max_players = game.nplayers;
@@ -141,8 +144,8 @@ static struct player *create_barbarian_player(int land)
   freelog(LOG_VERBOSE, "Created barbarian %s, player %d", barbarians->name, 
                        barbarians->player_no);
 
-  send_game_info(0);
-  send_player_info(barbarians,0);
+  send_game_info(NULL);
+  send_player_info(barbarians, NULL);
 
   return barbarians;
 }
@@ -150,27 +153,19 @@ static struct player *create_barbarian_player(int land)
 /**************************************************************************
 Check if a tile is land and free of enemy units
 **************************************************************************/
-static int is_free_land(int x, int y, int who)
+static bool is_free_land(int x, int y, struct player *who)
 {
-  if( y < 0 || y >= map.ysize ||
-      map_get_terrain(x,y) == T_OCEAN ||
-      is_non_allied_unit_tile(map_get_tile(x, y), who) )
-    return 0;
-  else
-    return 1;
+  return (is_real_tile(x, y) && map_get_terrain(x, y) != T_OCEAN
+	  && !is_non_allied_unit_tile(map_get_tile(x, y), who));
 }
 
 /**************************************************************************
 Check if a tile is sea and free of enemy units
 **************************************************************************/
-static int is_free_sea(int x, int y, int who)
+static bool is_free_sea(int x, int y, struct player *who)
 {
-  if( y < 0 || y >= map.ysize ||
-      map_get_terrain(x,y) != T_OCEAN ||
-      is_non_allied_unit_tile(map_get_tile(x, y), who) )
-    return 0;
-  else
-    return 1;
+  return (is_real_tile(x, y) && map_get_terrain(x, y) == T_OCEAN
+	  && !is_non_allied_unit_tile(map_get_tile(x, y), who));
 }
 
 /**************************************************************************
@@ -182,44 +177,49 @@ otherwise (not much land and no sea) kill enemy unit and stay in a village.
 The return value indicates if the explorer survived entering the vilage.
 **************************************************************************/
 
-int unleash_barbarians(struct player* victim, int x, int y)
+bool unleash_barbarians(struct player* victim, int x, int y)
 {
   struct player *barbarians;
   int unit, unit_cnt, land_cnt=0, sea_cnt=0;
   int boat;
   int i, xu, yu, me;
-  int alive = 1;     /* explorer survived */
+  bool alive = TRUE;     /* explorer survived */
 
-  if(!game.barbarianrate || (game.year < game.onsetbarbarian)) {
+  if(game.barbarianrate == 0 || (game.year < game.onsetbarbarian)) {
     unit_list_iterate(map_get_tile(x, y)->units, punit) {
       wipe_unit_safe(punit, &myiter);
     } unit_list_iterate_end;
-    return 0;
+    return FALSE;
   }
 
   unit_cnt = 3 + myrand(4);
 
-  barbarians = create_barbarian_player(1);
+  barbarians = create_barbarian_player(TRUE);
+  if (barbarians == NULL) {
+    return FALSE;
+  }
   me = barbarians->player_no;
 
   for( i=0; i<unit_cnt; i++) {
     unit = find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
-    create_unit( barbarians, x, y, unit, 0, 0, -1);
+    (void) create_unit(barbarians, x, y, unit, FALSE, 0, -1);
     freelog(LOG_DEBUG, "Created barbarian unit %s",unit_types[unit].name);
   }
 
   adjc_iterate(x, y, x1, y1) {
-    land_cnt += is_free_land(x1, y1, me);
-    sea_cnt += is_free_sea(x1, y1, me);
+    land_cnt += is_free_land(x1, y1, barbarians) ? 1 : 0;
+    sea_cnt += is_free_sea(x1, y1, barbarians) ? 1 : 0;
   } adjc_iterate_end;
 
   if( land_cnt >= 3 ) {           /* enough land, scatter guys around */
     unit_list_iterate(map_get_tile(x, y)->units, punit2) {
       if( punit2->owner == me ) {
-        send_unit_info( 0, punit2);
-        do { 
-          do rand_neighbour(x, y, &xu, &yu); 
-          while( !is_free_land(xu, yu, me) );
+        send_unit_info(NULL, punit2);
+	do {
+	  do {
+	    rand_neighbour(x, y, &xu, &yu);
+	  }
+	  while (!is_free_land(xu, yu, barbarians));
         } while( !handle_unit_move_request(punit2, xu, yu, TRUE, FALSE) );
         freelog(LOG_DEBUG, "Moved barbarian unit from %d %d to %d, %d", x,y,xu,yu);
       }
@@ -231,22 +231,23 @@ int unleash_barbarians(struct player* victim, int x, int y)
       int xb = -1, yb = -1;
       unit_list_iterate(map_get_tile(x, y)->units, punit2)
         if( punit2->owner == me ) {
-          send_unit_info( 0, punit2);
-          while(1) {
+          send_unit_info(NULL, punit2);
+          while(TRUE) {
 	    rand_neighbour(x, y, &xu, &yu);
-	    if( can_unit_move_to_tile(punit2, xu, yu, TRUE) ) break;
-	    if( can_unit_move_to_tile(punit2, xb, yb, TRUE) ) {
+	    if (can_unit_move_to_tile(punit2, xu, yu, TRUE))
+	      break;
+	    if (can_unit_move_to_tile(punit2, xb, yb, TRUE)) {
               xu = xb; yu = yb;
               break;
 	    }
-            if( is_free_sea(xu, yu, me) ) {
+	    if (is_free_sea(xu, yu, barbarians)) {
               boat = find_a_unit_type(L_BARBARIAN_BOAT, -1);
-	      create_unit( barbarians, xu, yu, boat, 0, 0, -1);
+	      (void) create_unit(barbarians, xu, yu, boat, FALSE, 0, -1);
 	      xb = xu; yb = yu;
 	      break;
 	    }
           }
-          handle_unit_move_request(punit2, xu, yu, TRUE, FALSE);
+          (void) handle_unit_move_request(punit2, xu, yu, TRUE, FALSE);
         }
       unit_list_iterate_end;
     }
@@ -254,10 +255,10 @@ int unleash_barbarians(struct player* victim, int x, int y)
       unit_list_iterate(map_get_tile(x, y)->units, punit2)
         if( punit2->owner != me ) {
           wipe_unit_safe(punit2, &myiter);
-          alive = 0;
+          alive = FALSE;
         }
         else
-          send_unit_info( 0, punit2);
+          send_unit_info(NULL, punit2);
       unit_list_iterate_end;
     }
   }
@@ -271,30 +272,30 @@ int unleash_barbarians(struct player* victim, int x, int y)
 /**************************************************************************
 Is sea not further than a couple of tiles away from land
 **************************************************************************/
-static int is_near_land(int x0, int y0)
+static bool is_near_land(int x0, int y0)
 {
   square_iterate(x0, y0, 4, x, y) {
     if (map_get_terrain(x,y) != T_OCEAN)
-      return 1;
+      return TRUE;
   } square_iterate_end;
 
-  return 0;
+  return FALSE;
 }
 
 /**************************************************************************
 return this or a neighbouring tile that is free of any units
 **************************************************************************/
-static int find_empty_tile_nearby(int x0, int y0, int *x, int *y)
+static bool find_empty_tile_nearby(int x0, int y0, int *x, int *y)
 {
   square_iterate(x0, y0, 1, x1, y1) {
     if (unit_list_size(&map_get_tile(x1, y1)->units) == 0) {
       *x = x1;
       *y = y1;
-      return 1;
+      return TRUE;
     }
   } square_iterate_end;
 
-  return 0;
+  return FALSE;
 }
 
 /**************************************************************************
@@ -323,13 +324,15 @@ static void try_summon_barbarians(void)
   struct city *pc;
   struct player *barbarians, *victim;
 
-  x = myrand(map.xsize);
-  y = 1 + myrand(map.ysize-2);  /* No uprising on North or South Pole */
+  /* No uprising on North or South Pole */
+  do {
+    rand_map_pos(&x, &y);
+  } while (y == 0 || y == map.ysize - 1);
 
-  if( !(pc = dist_nearest_city(NULL, x, y, 1, 0)) )       /* any city */
+  if( !(pc = dist_nearest_city(NULL, x, y, TRUE, FALSE)) )       /* any city */
     return;
 
-  victim = &game.players[pc->owner];
+  victim = city_owner(pc);
 
   dist = real_map_distance(x, y, pc->x, pc->y);
   freelog(LOG_DEBUG,"Closest city to %d %d is %s at %d %d which is %d far", 
@@ -351,51 +354,60 @@ static void try_summon_barbarians(void)
     return;
   freelog(LOG_DEBUG,"Barbarians are willing to fight");
 
-  if(map_get_special(x,y) & S_HUT) { /* remove the hut in place of uprising */
+  if(map_has_special(x, y, S_HUT)) { /* remove the hut in place of uprising */
     map_clear_special(x,y,S_HUT);
-    send_tile_info(0, x, y);
+    send_tile_info(NULL, x, y);
   }
 
   if( map_get_terrain(xu,yu) != T_OCEAN ) {        /* land barbarians */
-    barbarians = create_barbarian_player(1);
+    barbarians = create_barbarian_player(TRUE);
+    if (barbarians == NULL) {
+      return;
+    }
     if( city_list_size(&victim->cities) > UPRISE_CIV_MOST )
       uprise = 3;
     for( i=0; i < myrand(3) + uprise*(game.barbarianrate); i++) {
       unit = find_a_unit_type(L_BARBARIAN,L_BARBARIAN_TECH);
-      create_unit( barbarians, xu, yu, unit, 0, 0, -1);
+      (void) create_unit(barbarians, xu, yu, unit, FALSE, 0, -1);
       freelog(LOG_DEBUG, "Created barbarian unit %s",unit_types[unit].name);
     }
-    create_unit( barbarians, xu, yu, get_role_unit(L_BARBARIAN_LEADER,0), 0, 0, -1);
+    (void) create_unit(barbarians, xu, yu,
+		       get_role_unit(L_BARBARIAN_LEADER, 0), FALSE, 0, -1);
   }
   else {                   /* sea raiders - their units will be veteran */
 
-    barbarians = create_barbarian_player(0);
+    barbarians = create_barbarian_player(FALSE);
+    if (barbarians == NULL) {
+      return;
+    }
     boat = find_a_unit_type(L_BARBARIAN_BOAT,-1);
-    create_unit( barbarians, xu, yu, boat, 1, 0, -1);
+    (void) create_unit(barbarians, xu, yu, boat, TRUE, 0, -1);
     cap = get_transporter_capacity(unit_list_get(&map_get_tile(xu, yu)->units, 0));
     for( i=0; i<cap-1; i++) {
       unit = find_a_unit_type(L_BARBARIAN_SEA,L_BARBARIAN_SEA_TECH);
-      create_unit( barbarians, xu, yu, unit, 1, 0, -1);
+      (void) create_unit(barbarians, xu, yu, unit, TRUE, 0, -1);
       freelog(LOG_DEBUG, "Created barbarian unit %s",unit_types[unit].name);
     }
-    create_unit( barbarians, xu, yu, get_role_unit(L_BARBARIAN_LEADER,0), 0, 0, -1);
+    (void) create_unit(barbarians, xu, yu,
+		       get_role_unit(L_BARBARIAN_LEADER, 0), FALSE, 0, -1);
   }
 
-  unit_list_iterate(map_get_tile(x,y)->units, punit2)
-    send_unit_info(0, punit2);
-  unit_list_iterate_end;
+  unit_list_iterate(map_get_tile(x, y)->units, punit2) {
+    send_unit_info(NULL, punit2);
+  } unit_list_iterate_end;
 
   /* to let them know where to get you */
   show_area(barbarians, xu, yu, 3);
   show_area(barbarians, pc->x, pc->y, 3);
 
   /* There should probably be a different message about Sea Raiders */
-  if( is_land_barbarian(barbarians) )
-    notify_player_ex( victim, xu, yu, E_UPRISING, _("Native unrest near %s led by %s."),
-                      pc->name, barbarians->name);
-  else if( map_get_known_and_seen(xu, yu, victim->player_no) )
-    notify_player_ex( victim, xu, yu, E_UPRISING, _("Sea raiders seen near %s!"),
-                      pc->name);
+  if (is_land_barbarian(barbarians))
+    notify_player_ex(victim, xu, yu, E_UPRISING,
+		     _("Native unrest near %s led by %s."), pc->name,
+		     barbarians->name);
+  else if (map_get_known_and_seen(xu, yu, victim))
+    notify_player_ex(victim, xu, yu, E_UPRISING,
+		     _("Sea raiders seen near %s!"), pc->name);
 }
 
 /**************************************************************************
@@ -407,17 +419,16 @@ void summon_barbarians(void)
 {
   int i, n;
 
-  if(!game.barbarianrate)
+  if(game.barbarianrate == 0)
     return;
 
   if(game.year < game.onsetbarbarian)
     return;
 
-  n = map.xsize*map.ysize / MAP_FACTOR;    /* map size adjustment */
+  n = map_num_tiles() / MAP_FACTOR;
   if (n == 0) /* Allow barbarians on maps smaller than MAP_FACTOR */
     n = 1;
 
   for( i=0; i < n*(game.barbarianrate-1); i++)
     try_summon_barbarians();
 }
-
