@@ -30,18 +30,18 @@
 
 #include "console.h"
 
-static int console_show_prompt=0;
-static int console_prompt_is_showing=0;
-static int console_rfcstyle=0;
+static bool console_show_prompt = FALSE;
+static bool console_prompt_is_showing = FALSE;
+static bool console_rfcstyle = FALSE;
 #ifdef HAVE_LIBREADLINE
-static int readline_received_enter = 1;
+static bool readline_received_enter = TRUE;
 #endif
 
 /************************************************************************
 Function to handle log messages.
 This must match the log_callback_fn typedef signature.
 ************************************************************************/
-static void con_handle_log(int level, char *message)
+static void con_handle_log(int level, const char *message)
 {
   if(console_rfcstyle) {
     con_write(C_LOG_BASE+level, "%s", message);
@@ -60,7 +60,7 @@ static void con_update_prompt(void)
 
 #ifdef HAVE_LIBREADLINE
   if (readline_received_enter) {
-    readline_received_enter = 0;
+    readline_received_enter = FALSE;
   } else {
     rl_forced_update_display();
   }
@@ -69,13 +69,13 @@ static void con_update_prompt(void)
   con_flush();
 #endif
 
-  console_prompt_is_showing = 1;
+  console_prompt_is_showing = TRUE;
 }
 
 /************************************************************************
 Initialize logging via console.
 ************************************************************************/
-void con_log_init(char *log_filename, int log_level)
+void con_log_init(const char *log_filename, int log_level)
 {
   log_init(log_filename, log_level, (log_filename ? NULL : con_handle_log));
   logdebug_suppress_warning;
@@ -84,7 +84,7 @@ void con_log_init(char *log_filename, int log_level)
 /************************************************************************
 Write to console without line-break, don't print prompt.
 ************************************************************************/
-int con_dump(int i, char *message, ...)
+int con_dump(enum rfc_status rfc_status, const char *message, ...)
 {
   static char buf[MAX_LEN_CONSOLE_LINE];
   va_list args;
@@ -96,19 +96,19 @@ int con_dump(int i, char *message, ...)
   if(console_prompt_is_showing) {
     printf("\n");
   }
-  if ((console_rfcstyle) && (i >= 0)) {
-    printf("%.3d %s", i, buf);
+  if ((console_rfcstyle) && (rfc_status >= 0)) {
+    printf("%.3d %s", rfc_status, buf);
   } else {
     printf("%s", buf);
   }
-  console_prompt_is_showing = 0;
+  console_prompt_is_showing = FALSE;
   return (int) strlen(buf);
 }
 
 /************************************************************************
 Write to console and add line-break, and show prompt if required.
 ************************************************************************/
-void con_write(int i, char *message, ...)
+void con_write(enum rfc_status rfc_status, const char *message, ...)
 {
   static char buf[MAX_LEN_CONSOLE_LINE];
   va_list args;
@@ -117,7 +117,7 @@ void con_write(int i, char *message, ...)
   my_vsnprintf(buf, sizeof(buf), message, args);
   va_end(args);
 
-  con_puts(i, buf);
+  con_puts(rfc_status, buf);
 }
 
 /************************************************************************
@@ -127,36 +127,17 @@ The real reason for this is because __attribute__ complained
 with con_write(C_COMMENT,"") of "warning: zero-length format string";
 this allows con_puts(C_COMMENT,"");
 ************************************************************************/
-void con_puts(int i, char *str)
+void con_puts(enum rfc_status rfc_status, const char *str)
 {
   if(console_prompt_is_showing) {
     printf("\n");
   }
-  if ((console_rfcstyle) && (i >= 0)) {
-    printf("%.3d %s\n", i, str);
+  if ((console_rfcstyle) && (rfc_status >= 0)) {
+    printf("%.3d %s\n", rfc_status, str);
   } else {
     printf("%s\n", str);
   }
-  console_prompt_is_showing = 0;
-  con_update_prompt();
-}
-
-/************************************************************************
-For rfc-specific information only.
-Adds line-break, and shows prompt if required.
-************************************************************************/
-void con_rfconly(int i, char *message, ...)
-{
-  static char buf[MAX_LEN_CONSOLE_LINE];
-  va_list args;
-  
-  va_start(args, message);
-  my_vsnprintf(buf, sizeof(buf), message, args);
-  va_end(args);
-  
-  if ((console_rfcstyle) && (i >= 0))
-    printf("%.3d %s\n", i, buf);
-  console_prompt_is_showing = 0;
+  console_prompt_is_showing = FALSE;
   con_update_prompt();
 }
 
@@ -171,7 +152,7 @@ void con_flush(void)
 /************************************************************************
 Set style.
 ************************************************************************/
-void con_set_style( int i )
+void con_set_style(bool i)
 {
   console_rfcstyle = i;
   if (console_rfcstyle) 
@@ -183,7 +164,7 @@ void con_set_style( int i )
 /************************************************************************
 Returns rfc-style.
 ************************************************************************/
-int con_get_style(void)
+bool con_get_style(void)
 {
   return console_rfcstyle;
 }
@@ -193,11 +174,12 @@ Initialize prompt; display initial message.
 ************************************************************************/
 void con_prompt_init(void)
 {
-  static int first = 1;
+  static bool first = TRUE;
+
   if (first) {
     con_puts(C_COMMENT, "");
     con_puts(C_COMMENT, _("For introductory help, type 'help'."));
-    first = 0;
+    first = FALSE;
   }
 }
 
@@ -206,7 +188,7 @@ Make sure a prompt is printed, and re-printed after every message.
 ************************************************************************/
 void con_prompt_on(void)
 {
-  console_show_prompt = 1;
+  console_show_prompt = TRUE;
   con_update_prompt();
 }
 
@@ -215,7 +197,7 @@ Do not print a prompt after log messages.
 ************************************************************************/
 void con_prompt_off(void)
 {
-  console_show_prompt = 0;
+  console_show_prompt = FALSE;
 }
 
 /************************************************************************
@@ -223,9 +205,9 @@ User pressed enter: will need a new prompt
 ************************************************************************/
 void con_prompt_enter(void)
 {
-  console_prompt_is_showing = 0;
+  console_prompt_is_showing = FALSE;
 #ifdef HAVE_LIBREADLINE
-  readline_received_enter = 1;
+  readline_received_enter = TRUE;
 #endif
 }
 
@@ -234,8 +216,8 @@ Clear "user pressed enter" state (used in special cases).
 ************************************************************************/
 void con_prompt_enter_clear(void)
 {
-  console_prompt_is_showing = 1;
+  console_prompt_is_showing = TRUE;
 #ifdef HAVE_LIBREADLINE
-  readline_received_enter = 0;
+  readline_received_enter = FALSE;
 #endif
 }
