@@ -14,7 +14,7 @@
 #ifndef FC__GAMELOG_H
 #define FC__GAMELOG_H
 
-#include "attribute.h"
+#include "shared.h"		/* fc__attribute */
 
 #define GAMELOG_FATAL  0
 #define GAMELOG_NORMAL 1
@@ -30,6 +30,8 @@
 #define GAMELOG_REVOLT 11
 #define GAMELOG_GENO 12
 #define GAMELOG_TREATY 13
+#define GAMELOG_TEAM 15
+#define GAMELOG_STATUS 16
 #define GAMELOG_RANK 17
 #define GAMELOG_LAST 18
 #define GAMELOG_EOT 19
