@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <ctype.h>
 
 #include "fcintl.h"
 #include "game.h"
@@ -39,15 +38,15 @@
   name, assume connection name is a "modified" player name and don't use
   both (eg "(1-Shaka)" instead of "Shaka (1-Shaka)").  
 **************************************************************************/
-static void form_chat_name(struct connection *pconn, char *buffer, int len)
+static void form_chat_name(struct connection *pconn, char *buffer, size_t len)
 {
   struct player *pplayer = pconn->player;
 
-  if (pplayer==NULL) {
+  if (!pplayer) {
     my_snprintf(buffer, len, "(%s)", pconn->name);
   } else if (conn_list_size(&pplayer->connections)==1) {
-    mystrlcpy(buffer, pplayer->name, len);
-  } else if (strstr(pconn->name, pplayer->name) != NULL) {
+    (void) mystrlcpy(buffer, pplayer->name, len);
+  } else if (strstr(pconn->name, pplayer->name)) {
     /* Fixme: strstr above should be case-independent */
     my_snprintf(buffer, len, "(%s)", pconn->name);
   } else {
@@ -134,7 +133,8 @@ static void chat_msg_to_player_multi(struct connection *sender,
   2. Otherwise work out whether it is directed to a single player, or
      to a single connection, and send there.  (For a player, send to
      all clients connected as that player, in multi-connect case);
-  3. Else send to all connections (game.est_connections).
+  3. Or it may be intended for all allied players.
+  4. Else send to all connections (game.est_connections).
 
   In case 2, there can sometimes be ambiguity between player and
   connection names.  By default this tries to match player name first,
@@ -150,21 +150,21 @@ static void chat_msg_to_player_multi(struct connection *sender,
   avoiding sending both original and echo if sender is in destination
   set.
 **************************************************************************/
-void handle_chat_msg(struct connection *pconn,
-		     struct packet_generic_message *packet)
+void handle_chat_msg(struct connection *pconn, 
+                     struct packet_generic_message *packet)
 {
   struct packet_generic_message genmsg;
   char sender_name[MAX_LEN_CHAT_NAME];
   char *cp;
-  int double_colon;
+  bool double_colon;
 
   /* this loop to prevent players from sending multiple lines
    * which can be abused */
   genmsg.x = -1;
   genmsg.y = -1;
   genmsg.event =-1;
-  for(cp=packet->message; *cp; ++cp)
-    if(!isprint(*cp & 0x7f)) {
+  for (cp = packet->message; *cp != '\0'; ++cp)
+    if(!my_isprint(*cp & 0x7f)) {
       *cp='\0';
       break;
     }
@@ -181,6 +181,27 @@ void handle_chat_msg(struct connection *pconn,
     return;
   }
 
+  /* Send to allies command */
+  if (packet->message[0] == ALLIESCHAT_COMMAND_PREFIX) {
+    char sender_name[MAX_LEN_CHAT_NAME];
+    struct packet_generic_message genmsg;
+
+    genmsg.x = genmsg.y = genmsg.event = -1;
+    packet->message[0] = ' '; /* replace command prefix */
+    form_chat_name(pconn, sender_name, sizeof(sender_name));
+    my_snprintf(genmsg.message, sizeof(genmsg.message),
+                _("%s to allies: %s"), sender_name,
+                skip_leading_spaces(packet->message));
+    players_iterate(aplayer) {
+      if (!pplayers_allied(pconn->player, aplayer)) {
+        continue;
+      }
+      lsend_packet_generic_message(&aplayer->connections, 
+                                   PACKET_CHAT_MSG, &genmsg);
+    } players_iterate_end;
+    return;
+  }
+
   /* Want to allow private messages with "player_name: message",
      (or "connection_name: message"), including unambiguously
      abbreviated player/connection name, but also want to allow
@@ -206,14 +227,15 @@ void handle_chat_msg(struct connection *pconn,
   
   cp=strchr(packet->message, ':');
 
-  if (cp != NULL && (cp != &packet->message[0])) {
+  if (cp && (cp != &packet->message[0])) {
     enum m_pre_result match_result_player, match_result_conn;
     struct player *pdest = NULL;
     struct connection *conn_dest = NULL;
     char name[MAX_LEN_NAME];
     char *cpblank;
 
-    mystrlcpy(name, packet->message, MIN(sizeof(name), cp-packet->message+1));
+    (void) mystrlcpy(name, packet->message,
+		     MIN(sizeof(name), cp - packet->message + 1));
 
     double_colon = (*(cp+1) == ':');
     if (double_colon) {
