@@ -13,11 +13,12 @@
 #ifndef FC__HANDCHAT_H
 #define FC__HANDCHAT_H
 
-#include "packets.h"
+struct connection;
+struct packet_generic_message;
 
-struct player;
+#define ALLIESCHAT_COMMAND_PREFIX '.'
 
-void handle_chat_msg(struct player *pplayer, 
-		     struct packet_generic_message *packet);
+void handle_chat_msg(struct connection *pconn, 
+                   struct packet_generic_message *packet);
 
 #endif  /* FC__HANDCHAT_H */
