@@ -19,7 +19,7 @@
 #include <string.h>
 #include <time.h>
 #include <errno.h>
-#include <signal.h>
+#include <assert.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -27,9 +27,6 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
-#ifdef HAVE_SYS_SIGNAL_H
-#include <sys/signal.h>
-#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -73,7 +70,7 @@
 #include "plrhand.h"
 #include "shared.h"
 #include "support.h"
-
+#include "capability.h"
 #include "console.h"
 #include "meta.h"
 #include "srv_main.h"
@@ -81,7 +78,7 @@
 
 #include "sernet.h"
 
-struct connection connections[MAX_NUM_CONNECTIONS];
+static struct connection connections[MAX_NUM_CONNECTIONS];
 
 #ifdef GENERATING_MAC      /* mac network globals */
 TEndpointInfo serv_info;
@@ -108,21 +105,25 @@ static int sock;
    void user_interrupt_callback();
 #endif
 
+#define PROCESSING_TIME_STATISTICS 0
 
 static int server_accept_connection(int sockfd);
+static void start_processing_request(struct connection *pconn,
+				     int request_id);
+static void finish_processing_request(struct connection *pconn);
 
+static bool no_input = FALSE;
 
 /*****************************************************************************
   This happens if you type an EOF character with nothing on the current line.
 *****************************************************************************/
+#ifndef SOCKET_ZERO_ISNT_STDIN
 static void handle_stdin_close(void)
 {
-  notify_player(0, _("Server quitting because it cannot read standard input."));
-
-  printf("quit");
-  quit_game(NULL);
+  freelog(LOG_NORMAL, _("Server cannot read standard input. Ignoring input."));
+  no_input = TRUE;
 }
-
+#endif
 #ifdef HAVE_LIBREADLINE
 /****************************************************************************/
 
@@ -131,24 +132,30 @@ static void handle_stdin_close(void)
 
 static char *history_file = NULL;
 
-static int readline_handled_input = 0;
+static bool readline_handled_input = FALSE;
+
+static bool readline_initialized = FALSE;
 
 /*****************************************************************************
 ...
 *****************************************************************************/
 static void handle_readline_input_callback(char *line)
 {
+  if (no_input)
+    return;
+
   if (!line) {
     handle_stdin_close();	/* maybe print an 'are you sure?' message? */
+    return;
   }
 
-  if (*line)
+  if (line[0] != '\0')
     add_history(line);
 
   con_prompt_enter();		/* just got an 'Enter' hit */
   handle_stdin_input((struct connection*)NULL, line);
 
-  readline_handled_input = 1;
+  readline_handled_input = TRUE;
 }
 
 #endif /* HAVE_LIBREADLINE */
@@ -163,8 +170,8 @@ void close_connection(struct connection *pconn)
   conn_list_unlink(&game.game_connections, pconn);
 
   my_closesocket(pconn->sock);
-  pconn->used = 0;
-  pconn->established = 0;
+  pconn->used = FALSE;
+  pconn->established = FALSE;
   pconn->player = NULL;
   pconn->access_level = ALLOW_NONE;
   free_socket_packet_buffer(pconn->buffer);
@@ -179,7 +186,13 @@ void close_connection(struct connection *pconn)
 void close_connections_and_socket(void)
 {
   int i;
-  
+
+  struct packet_generic_message gen_packet;
+  gen_packet.message[0]='\0';
+
+  lsend_packet_generic_message(&game.all_connections, PACKET_SERVER_SHUTDOWN,
+			       &gen_packet);
+
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     if(connections[i].used) {
       close_connection(&connections[i]);
@@ -192,7 +205,14 @@ void close_connections_and_socket(void)
     write_history(history_file);
     history_truncate_file(history_file, HISTORY_LENGTH);
   }
+  if (readline_initialized) {
+    rl_callback_handler_remove();
+  }
 #endif
+
+  server_close_udp();
+
+  my_shutdown_network();
 }
 
 /*****************************************************************************
@@ -219,7 +239,7 @@ void flush_packets(void)
   struct timeval tv;
   time_t start;
 
-  time(&start);
+  (void) time(&start);
 
   for(;;) {
     tv.tv_sec=(game.netwait - (time(NULL) - start));
@@ -234,7 +254,7 @@ void flush_packets(void)
 
     for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
       struct connection *pconn = &connections[i];
-      if(pconn->used && pconn->send_buffer->ndata) {
+      if(pconn->used && pconn->send_buffer->ndata > 0) {
 	FD_SET(pconn->sock, &writefs);
 	FD_SET(pconn->sock, &exceptfs);
 	max_desc=MAX(pconn->sock, max_desc);
@@ -257,11 +277,11 @@ void flush_packets(void)
 		  conn_description(pconn));
 	  close_socket_callback(pconn);
         } else {
-	  if(pconn->send_buffer && pconn->send_buffer->ndata) {
+	  if(pconn->send_buffer && pconn->send_buffer->ndata > 0) {
 	    if(FD_ISSET(pconn->sock, &writefs)) {
 	      flush_connection_send_buffer_all(pconn);
 	    } else {
-	      if(game.tcptimeout && pconn->last_write
+	      if(game.tcptimeout != 0 && pconn->last_write != 0
 		 && (time(NULL)>pconn->last_write + game.tcptimeout)) {
 	        freelog(LOG_NORMAL, "cut connection %s due to lagging player",
 			conn_description(pconn));
@@ -302,9 +322,7 @@ int sniff_packets(void)
 
 #ifdef HAVE_LIBREADLINE
   {
-    static int readline_initialized = 0;
-
-    if (!readline_initialized) {
+    if (!no_input && !readline_initialized) {
       char *home_dir = user_home_dir();
       if (home_dir) {
 	history_file =
@@ -322,7 +340,8 @@ int sniff_packets(void)
       rl_callback_handler_install("> ", handle_readline_input_callback);
       rl_attempted_completion_function = freeciv_completion;
 
-      readline_initialized = 1;
+      readline_initialized = TRUE;
+      atexit(rl_callback_handler_remove);
     }
   }
 #endif /* HAVE_LIBREADLINE */
@@ -330,31 +349,32 @@ int sniff_packets(void)
   if(year!=game.year) {
     if (server_state == RUN_GAME_STATE) year=game.year;
   }
-  if (!game.timeout)
+  if (game.timeout == 0)
     game.turn_start = time(NULL);
   
-  while(1) {
+  while(TRUE) {
     con_prompt_on();		/* accepting new input */
     
     if(force_end_of_sniff) {
-      force_end_of_sniff=0;
+      force_end_of_sniff = FALSE;
       con_prompt_off();
       return 2;
     }
 
 
     /* quit server if no players for 'srvarg.quitidle' seconds */
-    if (srvarg.quitidle && server_state != PRE_GAME_STATE) {
+    if (srvarg.quitidle != 0 && server_state != PRE_GAME_STATE) {
       static time_t last_noplayers;
       if(conn_list_size(&game.est_connections) == 0) {
-	if (last_noplayers) {
+	if (last_noplayers != 0) {
 	  if (time(NULL)>last_noplayers + srvarg.quitidle) {
 	    sz_strlcpy(srvarg.metaserver_info_line,
 		       "restarting for lack of players");
 	    freelog(LOG_NORMAL, srvarg.metaserver_info_line);
-	    send_server_info_to_metaserver(1,0);
+	    (void) send_server_info_to_metaserver(TRUE, FALSE);
 
-	    quit_game(NULL);
+	    close_connections_and_socket();
+	    exit(EXIT_SUCCESS);
 	  }
 	} else {
 	  last_noplayers = time(NULL);
@@ -364,7 +384,7 @@ int sniff_packets(void)
 		      "restarting in %d seconds for lack of players",
 		      srvarg.quitidle);
 	  freelog(LOG_NORMAL, srvarg.metaserver_info_line);
-	  send_server_info_to_metaserver(1,0);
+	  (void) send_server_info_to_metaserver(TRUE, FALSE);
 	}
       } else {
         last_noplayers = 0;
@@ -379,7 +399,7 @@ int sniff_packets(void)
 	  send_packet_generic_empty(pconn, PACKET_CONN_PING);
 
 	  if (pconn->ponged) {
-	    pconn->ponged = 0;
+	    pconn->ponged = FALSE;
 	  } else {
 	    freelog(LOG_NORMAL, "cut connection %s due to ping timeout",
 		    conn_description(pconn));
@@ -390,6 +410,16 @@ int sniff_packets(void)
       game.last_ping = time(NULL);
     }
 
+    /* Don't wait if timeout == -1 (i.e. on auto games) */
+    if (server_state != PRE_GAME_STATE && game.timeout == -1) {
+      (void) send_server_info_to_metaserver(FALSE, FALSE);
+
+      /* kick out of the srv_main loop */
+      if (server_state == GAME_OVER_STATE) {
+	server_state = PRE_GAME_STATE;
+      }
+      return 0;
+    }
 
     tv.tv_sec=1;
     tv.tv_usec=0;
@@ -397,13 +427,17 @@ int sniff_packets(void)
     MY_FD_ZERO(&readfs);
     MY_FD_ZERO(&writefs);
     MY_FD_ZERO(&exceptfs);
+
+    if (!no_input) {
 #ifdef SOCKET_ZERO_ISNT_STDIN
-    my_init_console();
+      my_init_console();
 #else
 #   if !defined(__VMS)
       FD_SET(0, &readfs);
 #   endif	
 #endif
+    }
+
     FD_SET(sock, &readfs);
     FD_SET(sock, &exceptfs);
     max_desc=sock;
@@ -411,7 +445,7 @@ int sniff_packets(void)
     for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
       if(connections[i].used) {
 	FD_SET(connections[i].sock, &readfs);
-	if(connections[i].send_buffer->ndata) {
+	if(connections[i].send_buffer->ndata > 0) {
 	  FD_SET(connections[i].sock, &writefs);
 	}
 	FD_SET(connections[i].sock, &exceptfs);
@@ -421,13 +455,15 @@ int sniff_packets(void)
     con_prompt_off();		/* output doesn't generate a new prompt */
 
     if(select(max_desc+1, &readfs, &writefs, &exceptfs, &tv)==0) { /* timeout */
-      send_server_info_to_metaserver(0,0);
-      if((game.timeout) 
+      (void) send_server_info_to_metaserver(FALSE, FALSE);
+      if(game.timeout != 0
 	&& (time(NULL)>game.turn_start + game.timeout)
 	&& (server_state == RUN_GAME_STATE)){
 	con_prompt_off();
 	return 0;
       }
+
+      if (!no_input) {
 #if defined(__VMS)
       {
 	struct { short numchars; char firstchar; char reserved; int reserved2; } ttchar;
@@ -445,8 +481,9 @@ int sniff_packets(void)
       continue;
 #endif /* SOCKET_ZERO_ISNT_STDIN */
 #endif /* !__VMS */
+      }
     }
-    if (!game.timeout)
+    if (game.timeout == 0)
       game.turn_start = time(NULL);
 
     if(FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
@@ -466,27 +503,28 @@ int sniff_packets(void)
 	close_socket_callback(pconn);
       }
     }
+    
 #ifdef SOCKET_ZERO_ISNT_STDIN
-    if ((bufptr = my_read_console())) {
+    if (!no_input && (bufptr = my_read_console())) {
       con_prompt_enter();	/* will need a new prompt, regardless */
       handle_stdin_input((struct connection *)NULL, bufptr);
     }
 #else  /* !SOCKET_ZERO_ISNT_STDIN */
-    if(FD_ISSET(0, &readfs)) {    /* input from server operator */
+    if(!no_input && FD_ISSET(0, &readfs)) {    /* input from server operator */
 #ifdef HAVE_LIBREADLINE
       rl_callback_read_char();
       if (readline_handled_input) {
-	readline_handled_input = 0;
+	readline_handled_input = FALSE;
 	con_prompt_enter_clear();
       }
       continue;
 #else  /* !HAVE_LIBREADLINE */
-      int didget;
+      ssize_t didget;
       char buf[BUF_SIZE+1];
       
       if((didget=read(0, buf, BUF_SIZE))==-1) {
 	freelog(LOG_FATAL, "read from stdin failed");
-	exit(1);
+	exit(EXIT_FAILURE);
       }
 
       if(didget==0) {
@@ -506,12 +544,53 @@ int sniff_packets(void)
   	struct connection *pconn = &connections[i];
 	if(pconn->used && FD_ISSET(pconn->sock, &readfs)) {
 	  if(read_socket_data(pconn->sock, pconn->buffer)>=0) {
-	    char *packet;
-	    int type, result;
-	    while (1) {
+	    void *packet;
+	    enum packet_type type;
+	    bool result;
+
+	    while (TRUE) {
 	      packet = get_packet_from_connection(pconn, &type, &result);
 	      if (result) {
-		handle_packet_input(pconn, packet, type);
+		bool command_ok;
+
+		pconn->server.last_request_id_seen =
+		    get_next_request_id(pconn->server.
+					last_request_id_seen);
+#if PROCESSING_TIME_STATISTICS
+		{
+		  int err;
+		  struct timeval start, end;
+		  struct timezone tz;
+		  long us;
+
+		  err = gettimeofday(&start, &tz);
+		  assert(!err);
+#endif
+		connection_do_buffer(pconn);
+		start_processing_request(pconn,
+					 pconn->server.
+					 last_request_id_seen);
+		command_ok = handle_packet_input(pconn, packet, type);
+		packet = NULL;
+
+		finish_processing_request(pconn);
+		connection_do_unbuffer(pconn);
+		if (!command_ok) {
+		  close_connection(pconn);
+		}
+#if PROCESSING_TIME_STATISTICS
+		  err = gettimeofday(&end, &tz);
+		  assert(!err);
+
+		  us = (end.tv_sec - start.tv_sec) * 1000000 +
+		      end.tv_usec - start.tv_usec;
+
+		  freelog(LOG_NORMAL,
+			  "processed request %d in %ld.%03ldms",
+			  pconn->server.last_request_id_seen, us / 1000,
+			  us % 1000);
+                }
+#endif
 	      } else {
 		break;
 	      }
@@ -524,11 +603,11 @@ int sniff_packets(void)
 
       for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
         struct connection *pconn = &connections[i];
-        if(pconn->used && pconn->send_buffer && pconn->send_buffer->ndata) {
+        if(pconn->used && pconn->send_buffer && pconn->send_buffer->ndata > 0) {
 	  if(FD_ISSET(pconn->sock, &writefs)) {
 	    flush_connection_send_buffer_all(pconn);
 	  } else {
-	    if(game.tcptimeout && pconn->last_write
+	    if(game.tcptimeout != 0 && pconn->last_write != 0
 	       && (time(NULL)>pconn->last_write + game.tcptimeout)) {
 	      freelog(LOG_NORMAL, "cut connection %s due to lagging player",
 		      conn_description(pconn));
@@ -542,9 +621,9 @@ int sniff_packets(void)
   }
   con_prompt_off();
 
-  if((game.timeout) 
-     && (time(NULL)>game.turn_start + game.timeout))
+  if (game.timeout != 0 && (time(NULL) > game.turn_start + game.timeout)) {
     return 0;
+  }
   return 1;
 }
 
@@ -603,26 +682,31 @@ static int server_accept_connection(int sockfd)
 
   my_nonblock(new_sock);
 
-  from=gethostbyaddr((char*)&fromend.sin_addr, sizeof(fromend.sin_addr),
+  from=gethostbyaddr((char *)&fromend.sin_addr, sizeof(fromend.sin_addr),
 		     AF_INET);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     struct connection *pconn = &connections[i];
     if (!pconn->used) {
-      pconn->used = 1;
+      pconn->used = TRUE;
       pconn->sock = new_sock;
-      pconn->established = 0;
-      pconn->observer = 0;
+      pconn->established = FALSE;
+      pconn->observer = FALSE;
       pconn->player = NULL;
       pconn->buffer = new_socket_packet_buffer();
       pconn->send_buffer = new_socket_packet_buffer();
       pconn->last_write = 0;
-      pconn->ponged = 1;
-      pconn->first_packet = 1;
-      pconn->byte_swap = 0;
+      pconn->ponged = TRUE;
+      pconn->first_packet = TRUE;
+      pconn->byte_swap = FALSE;
       pconn->capability[0] = '\0';
       pconn->access_level = access_level_for_next_connection();
-      pconn->delayed_disconnect = 0;
+      pconn->delayed_disconnect = FALSE;
+      pconn->notify_of_writable_data = NULL;
+      pconn->server.currently_processed_request_id = 0;
+      pconn->server.last_request_id_seen = 0;
+      pconn->incoming_packet_notify = NULL;
+      pconn->outgoing_packet_notify = NULL;
 
       sz_strlcpy(pconn->name, makeup_connection_name(&pconn->id));
       sz_strlcpy(pconn->addr,
@@ -648,19 +732,14 @@ int server_open_socket(void)
   struct sockaddr_in src;
   int opt;
 
-  /* broken pipes are ignored. */
-#ifdef HAVE_SIGPIPE
-  signal (SIGPIPE, SIG_IGN);
-#endif
-
   if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
     freelog(LOG_FATAL, "socket failed: %s", mystrerror(errno));
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   opt=1; 
   if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
-		(char*)&opt, sizeof(opt)) == -1) {
+		(char *)&opt, sizeof(opt)) == -1) {
     freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror(errno));
   }
 
@@ -671,12 +750,12 @@ int server_open_socket(void)
 
   if(bind(sock, (struct sockaddr *) &src, sizeof (src)) == -1) {
     freelog(LOG_FATAL, "bind failed: %s", mystrerror(errno));
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   if(listen(sock, MAX_NUM_CONNECTIONS) == -1) {
     freelog(LOG_FATAL, "listen failed: %s", mystrerror(errno));
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   close_socket_set_callback(close_socket_callback);
@@ -690,9 +769,14 @@ int server_open_socket(void)
 void init_connections(void)
 {
   int i;
+
+  conn_list_init(&game.all_connections);
+  conn_list_init(&game.est_connections);
+  conn_list_init(&game.game_connections);
+
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) { 
     struct connection *pconn = &connections[i];
-    pconn->used = 0;
+    pconn->used = FALSE;
     conn_list_init(&pconn->self);
     conn_list_insert(&pconn->self, pconn);
     pconn->route = NULL;
@@ -706,3 +790,29 @@ void init_connections(void)
   }
 #endif
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+static void start_processing_request(struct connection *pconn,
+				     int request_id)
+{
+  assert(request_id);
+  assert(pconn->server.currently_processed_request_id == 0);
+  freelog(LOG_DEBUG, "start processing packet %d from connection %d",
+	  request_id, pconn->id);
+  send_packet_generic_empty(pconn, PACKET_PROCESSING_STARTED);
+  pconn->server.currently_processed_request_id = request_id;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void finish_processing_request(struct connection *pconn)
+{
+  assert(pconn->server.currently_processed_request_id);
+  freelog(LOG_DEBUG, "finish processing packet %d from connection %d",
+	  pconn->server.currently_processed_request_id, pconn->id);
+  send_packet_generic_empty(pconn, PACKET_PROCESSING_FINISHED);
+  pconn->server.currently_processed_request_id = 0;
+}
